
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  MVWNType.h        (Multimedia Video Window Typedefs)
// VERSION: 1.00
//
// DESCRIPTION:
//
//    This module contains Typedefs for structures used by the Multimedia
//    Video Window (MVWN).
//
//    This header file is imbedded by MVWN.H.  It is used by toolkit
//    developers that are including the Multimedia Video Window in their
//    applications.
//
//    The following structures are declared:
//
//       MVWNDATA       -- The MVWN Data structure
//       MVWNCMD        -- The MVWN Command structure
//       MVWNVIDIMAGE   -- The MVWN Video Image structure
//       MVWNVIDST      -- The MVWN Video Status structure (used by MVWNQRY)
//       MVWNAUDST      -- The MVWN Audio Status structure (used by MVWNQRY)
//       MVWNQRY        -- The MVWN Query structure
//       MVWNADV        -- The MVWN Advise structure
//       MVWNCTL        -- The MVWN Control Data structure
//
//    -------------------------------------------------------------------------
//    Here is an example of how to send a command to an MVWN object:
//
//    PMVWNCMD pmvwncmd;
//
//    if (MVWNCreateCmd (&pmvwncmd, MVWN_CMD_VIDEOSOURCE))
//       {
//       pmvwncmd->data.mpCmdParam1 = MPFROM2SHORT (1, 0);  // Source: 1
//
//       WinPostMsg (hwndMVWN, msgMVWN_COMMAND, hwnd, pmvwncmd);
//       }
//
//    -------------------------------------------------------------------------
//    Here is an example of how to send a query to an MVWN object:
//
//    PMVWNQRY pmvwnqry;
//    MRESULT  rc;
//    BOOL     usVideoSource;
//
//    if (MVWNCreateQry (&pmvwnqry, MVWN_QRY_STATUS))
//       {
//       rc = WinSendMsg (hwndMVWN, msgMVWN_QUERY, hwnd, pmvwnqry);
//       if (!rc)
//          {
//          usVideoSource = pmvwnqry->mvidst.usVideoSource;
//          }
//       }
//
//*****************************************************************************

//*****************************************************************************
typedef struct _MVWNDATA               // Video Window Data Structure
   {
   MPARAM      mpCmdParam1;            // Command Parameter #1
   MPARAM      mpCmdParam2;            // Command Parameter #2
   MPARAM      mpCmdParam3;            // Command Parameter #3
   MPARAM      mpCmdParam4;            // Command Parameter #4
   MPARAM      mpCmdParam5;            // Command Parameter #5
   } MVWNDATA;

typedef MVWNDATA FAR *PMVWNDATA;


//*****************************************************************************
#ifdef MVWN
typedef struct _MVWNCMD                // Used to send an MVWN_CMD message
   {
   MPREFIX     prefix;                 // Multimedia Prefix structure
   MCOMMAND    command;                // Multimedia Command Structure
   MVWNDATA    data;                   // Video Window Data structure
   } MVWNCMD;

typedef MVWNCMD FAR *PMVWNCMD;
#endif


//*****************************************************************************
typedef struct _MVWNVIDIMAGE           // MVWN Video Image Structure
   {
   SHORT       xLen;                   // X-Length of the Video Image
   SHORT       yLen;                   // Y-Length of the Video Image
   ULONG       cImage;                 // Size (in bytes) of the Video Image
   PVOID       pImageBuffer;           // Pointer to the Video Image Buffer
   } MVWNVIDIMAGE;

typedef MVWNVIDIMAGE FAR *PMVWNVIDIMAGE;

//*****************************************************************************
typedef struct _MVWNVIDST              // Video Status Structure
   {
   USHORT      usVideoSource;          // Video Source #1, 2, or 3.
   BOOL        fBlackWhite;            // TRUE: Video is in Black/White
   BOOL        fFreeze;                // TRUE: Video is frozen
   USHORT      usScanMethod;           // MVWN_SCANM_*
   USHORT      usScanAlgorithm;        // MVWN_SCANA_*
   USHORT      usBrightness;           // 0 to 100%
   USHORT      usSharpness;            // 0 to 100%
   USHORT      usContrast;             // 0 to 100%
   USHORT      usColor;                // 0 to 100%
   SHORT       sTint;                  // -100 to 100
   USHORT      usFade;                 // 0 to 100%
   } MVWNVIDST;

typedef MVWNVIDST FAR *PMVWNVIDST;


//*****************************************************************************
typedef struct _MVWNAUDST              // Audio Status Structure
   {
   BOOL        fMuted;                 // TRUE: Audio is muted
   USHORT      usVolume;               // 0 to 100%
   SHORT       sBalance;               // -100 to 100
   SHORT       sBass;                  // -100 to 100
   SHORT       sTreble;                // -100 to 100
   USHORT      usFade;                 // 0 to 100%
   } MVWNAUDST;

typedef MVWNAUDST FAR *PMVWNAUDST;


//*****************************************************************************
typedef struct _MVWNQRY                // Query structure
   {
   // INPUT:
   ULONG       cbData;                 // cbData = sizeof (MVWNQRY)
   USHORT      usQuery;                // Query item (MVWN_QRY_*)

   // OUTPUT (everything from here down is Output only):
   USHORT      usReturnCode;           // Return code (MVWN_ERR_*)
   BOOL        fActive;                // TRUE = this video window is active

   // Video & Audio support information
   BOOL        fScale;                 // TRUE = video window is scaled (compressed)
   MVWNVIDST   mvidst;                 // VIDEO STATUS STRUCTURE
   MVWNAUDST   maudst;                 // AUDIO STATUS STRUCTURE

   // User-controllable (via commands or mvwnctl) attributes:
   HWND        hwndAdviseActive;       // This hwnd is notified when the video
                                       // window becomes active/inactive.
   BOOL        fUserActivate;          // TRUE: Mouse Button 1 activates
   USHORT      fsInactiveStyle;        // MVWN_UPD_* values
   LONG        lInactiveColor;         // CLR_* value for COLORFILL
   HPOINTER    hptrSrc1;               // Pointer when Video Source == 1
   HPOINTER    hptrSrc2;               // Pointer when Video Source == 2
   HPOINTER    hptrSrc3;               // Pointer when Video Source == 3
   HPOINTER    hptrInactive;           // Pointer when inactive
   USHORT      usSourceHorg ;
   USHORT      usSourceVorg ;
   USHORT      usSourceHlen ;
   USHORT      usSourceVlen ;
   USHORT      usDestHorg ;
   USHORT      usDestVorg ;
   USHORT      usDestHlen ;
   USHORT      usDestVlen ;
   } MVWNQRY;

typedef MVWNQRY FAR *PMVWNQRY;


//*****************************************************************************
typedef struct _MVWNADV                // Used to send an MVWN_ADV message
   {
   ULONG       cbData;                 // cbData = sizeof (MVWNADV);
   USHORT      usAdvise;               // Advise Item (MVWN_ADV_*)

   MPARAM      mpAdvParam1;            // Advise parameter
   } MVWNADV;

typedef MVWNADV FAR *PMVWNADV;


//*****************************************************************************
// MVWNCTL = Video Window Control Data Block (used at WM_CREATE time):
typedef struct _MVWNCTL
   {
   ULONG       cbData;                 // Length of this data structure
   USHORT      usReturnCode;           // Return code (ie. MVWN_RC_CREATE_*)

   BOOL        fScale;                 // TRUE = video window is scaled (compressed)
   MVWNVIDST   mvidst;                 // VIDEO STATUS STRUCTURE
   MVWNAUDST   maudst;                 // AUDIO STATUS STRUCTURE

   HWND        hwndAdviseActive;       // This hwnd is notified when the video
                                       //    window becomes active/inactive.
   BOOL        fUserActivate;          // TRUE: Mouse Button 1 activates
   USHORT      fsInactiveStyle;        // MVWN_UPD_* values
   LONG        lInactiveColor;         // CLR_* value for COLORFILL
   HPOINTER    hptrSrc1;               // Pointer when Video Source == 1
   HPOINTER    hptrSrc2;               // Pointer when Video Source == 2
   HPOINTER    hptrSrc3;               // Pointer when Video Source == 3
   HPOINTER    hptrInactive;           // Pointer when inactive
   } MVWNCTL;

typedef MVWNCTL FAR *PMVWNCTL;


//*****************************************************************************
// END OF FILE
//*****************************************************************************
