//**********************************************************************
// Copyright IBM Corporation 1989, 1992
//**********************************************************************
// File : AR.C
//
// Description :
//      This program will record line input into an IBM MMotion Audio
// file.  It also permits the user to change the Record Volume, and
// indicate when to stop recording.
//
//      To run:
//              1. Compile
//              2. Determine a name for the audio file to be created,
//                      SONG.AUD for example
//              3. Type "AR SONG.AUD"
//              4. Indicate when you are ready to start recording
//              5. By default, recording is done from CHANNEL 1.
//                 You may use channel 2 by changing the call to
//                 cpaud_Record () to use AUD_CHANNEL_2.
//              6. Change the Record Volume as desired
//              7. Press the ESC key to stop recording and return to
//                      DOS
//
// NOTE : ONLY single segment audio files will be created.  Any data
//        in the file will be overwritten.
//
// Compilation Instructions:
//      1) First, compile MCALLS.C into MCALLS.OBJ.
//
//      2) Compile AR.C with IBM C/2 version 1.1 using the following parms
//
//         cl /AL /G2s /Zp ar.c mcalls.obj /link graphics
//
//         If you created combined libraries with graphics you will not
//         need to link graphics at this stage.
//
//         Do NOT select/use ANSI compatibility for your compiler.
//**********************************************************************

/****************************/
/* Standard C include files */
/****************************/
#include <graph.h>
#include <dos.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

/***********/
/* Defines */
/***********/
#define AUD_CHANNEL_1    0x01
#define AUD_CHANNEL_2    0x02

/**********************************/
/* M-Control Program include file */
/**********************************/
#include "mcalls.h"

/***********************/
/* Function Prototypes */
/***********************/
void set_rec_vol (void);

/*************/
/* Variables */
/*************/
BYTE              mybuff[16000];
ULONG             offsets[] = { 1L, 0L };
AUDIO_MONITOR     rts;
char              rec_vol = 60;

//*************************************************************
// Function : set_rec_vol
//
// Description :
//    Sets the NEW record volume (changed by the user) and
// displays the new level.
//*************************************************************
void set_rec_vol ( )
  {
  char textline[100];

  cpaud_RecordVolume (FALSE, rec_vol);
  sprintf (textline,  "Digital Record Volume is %d%%\n", rec_vol);
  _settextcolor ( 15 );
  _settextposition ( 15, 10);
  _outtext (textline);
  }


//*************************************************************
// Function : main
//
// Description :
//    Checks the incoming parms for a record file, then starts
// the record process.  The program will also wait for the user
// to indicate that everything is ready for the recording to
// begin.
//
//    Once recording is started a peak/saturation meter is
// displayed to assist in adjusting the Record Volume to obtain
// optimal results.
//
//    User controls include the following:
//              U or u : increases the Record Volume by 5%
//              D or d : decreases the Record Volume by 5%
//              ESC    : stops recording, closes the files, and
//                       returns to DOS
//
// Input Parms :
//      filename to be used to hold the audio data
//*************************************************************
main ( int argc, char *argv[] )
  {
  int   action;
  BYTE ahand;
  CP_FRCVBUF prc;

  /************************************************************/
  /* There must be a filename provided to store to audio into */
  /************************************************************/
  if ( argc < 2 )
     {
     printf ("AR will record audio into a file, and provides controls to raise\N");
     printf ("and lower the record volume to avoid oversaturation distortion.\n\n");
     printf ("To use AR type:\n");
     printf ("    AR <filename>\n");
     printf ("where <filename> is the name of the file you wish to use. For example,\n");
     printf ("    AR SONG.AUD\n");
     exit ( -1 );
     }

  /* Check to make sure that the M-Motion code has been loaded */
  if ( !cp_loaded ( ) ) {
    printf ( "M-Motion not yet active, install M-Motion and try again" );
    return ( 0 );
    }

  /*********************************************************************/
  /* Start a graphics mode so we can display the peak/saturation meter */
  /*********************************************************************/
  _setvideomode ( _VRES16COLOR );
  _settextcolor ( 9 );
  _settextposition ( 5, 17);
  _outtext ("M-Motion Digital Audio Recording Sample Program");


  /*****************************/
  /* Attempt to start M-Motion */
  /*****************************/
  cp_init ( );
  cpaud_Reset ();
  cpaud_Mute ( 0 );

  prc = cpaud_BufferInit ( mybuff, sizeof ( mybuff ) );
  prc = cpaud_BufferActivate ( mybuff );

  /***************************************************************************/
  /* Set up to record at the beginning of the file.  Obtain the audio handle */
  /***************************************************************************/
  prc = cpaud_OpenRecord ( argv[1], 0L, &ahand );

  cpaud_AhandleStatus ( );

  /************************************************************/
  /* Wait for user to indicate that they are ready to proceed */
  /************************************************************/
  _settextposition ( 10, 10);
  _outtext ("Ensure your audio is audible or ready.  Press any key to continue.");
  getch ( );

  /****************************/
  /* Display the instructions */
  /****************************/
  _settextcolor (11);
  _settextposition ( 10, 10);
  _outtext ("                                                                  ");
  _settextposition ( 10, 10);
  _outtext ("Press the 'U' key to raise the record volume");
  _settextposition ( 11, 10);
  _outtext (" or press 'D' to lower the record volume");
  _settextposition ( 13, 10);
  _outtext ("Press ESC to stop recording and return to DOS");
  _settextposition ( 19, 10);
  _outtext ("Record Level Peak/Saturation Meter");

  /* display the STARTING record volume */
  set_rec_vol ( );

  cpaud_Record ( ahand, AUD_CHANNEL_1);


  /*************************************************/
  /* Continue recording until the user presses ESC */
  /*************************************************/
  while (TRUE)
     {
     if ( kbhit () )
        {
        action = getch ( );

        /************************/
        /* ESC = quit recording */
        /************************/
        if ( action == 0x1B )
           break;

        /**************************/
        /* u or U = UP the volume */
        /**************************/
        else if ( action == 'u' || action == 'U' )
           {
           // raise the record volume
           rec_vol += 5;
           if ( rec_vol > 100 )
             rec_vol = 100;

           set_rec_vol ( );
           }

        /****************************/
        /* d or D = DOWN the volume */
        /****************************/
        else if ( action == 'd' || action == 'D' )
           {
           // lower the record volume
           rec_vol -= 5;
           if ( rec_vol < 0 )
             rec_vol = 0;

           set_rec_vol ( );
           }
        }

     /**************************/
     /* Display the peak meter */
     /**************************/
     cpaud_Monitor ( &rts );
     if ( rts.sample_time )
        {
        _settextcolor ( 15 );
        _settextposition ( 20, 10);
        switch ( rts.peak_percent / 10 )
           {
           case 0:
             _outtext ( "*          " );
             break;
           case 1:
             _outtext ( "*-         " );
             break;
           case 2:
             _outtext ( "*--        " );
             break;
           case 3:
             _outtext ( "*---       " );
             break;
           case 4:
             _outtext ( "*----      " );
             break;
           case 5:
             _outtext ( "*-----     " );
             break;
           case 6:
             _outtext ( "*------    " );
             break;
           case 7:
             _outtext ( "*-------   " );
             break;
           case 8:
             _outtext ( "*--------  " );
             break;
           case 9:
             _outtext ( "*--------- " );
             break;
           case 10:
             _outtext ( "*----------" );
             break;
           }

        if ( rts.saturations )
           {
           _settextcolor ( 4 );
           _settextposition ( 20, 21);
           switch ( rts.saturations / 10 )
              {
              case 0:
                _outtext ( "*           " );
                break;
              case 1:
                _outtext ( "**          " );
                break;
              case 2:
                _outtext ( "***         " );
                break;
              case 3:
                _outtext ( "****        " );
                break;
              case 4:
                _outtext ( "*****       " );
                break;
              case 5:
                _outtext ( "******      " );
                break;
              case 6:
                _outtext ( "*******     " );
                break;
              case 7:
                _outtext ( "********    " );
                break;
              case 8:
                _outtext ( "*********   " );
                break;
              case 9:
                _outtext ( "**********  " );
                break;
              default:
                _outtext ( "*********** " );
                break;
              }
           }
        }
     }

  _setvideomode ( _DEFAULTMODE );

  /**********************/
  /* Clean up and close */
  /**********************/
  cpaud_Stop ( );
  cpaud_Close ( ahand, 0L );
  cpaud_BufferClose ( );
  cpaud_Mute ( 1 );

  cp_exit ( );

  return ( 0 );
  }

