//**********************************************************************
// Copyright IBM Corporation 1989, 1992
//**********************************************************************
// File : CAP2BUF.C
//
// Description :  Demonstrates ability to capture and restore video
//                to and from a buffer defined in memory.
//
//      To run:
//              1. Compile
//              2. Type "CAP2BUF 0 0 640 480" or "CAP2BUF" for
//                 a list of the allowable parms.
//
// Compilation Instructions:
//      1) First, compile MCALLS.C into MCALLS.OBJ.
//
//      2) Compile CAP2BUF.C with IBM C/2 version 1.1 using the following parms
//
//         cl /AL /G2s /Zp cap2buf.c mcalls.obj /link graphics
//
//         If you created combined libraries with graphics you will not
//         need to link graphics at this stage.
//
//         Do NOT select/use ANSI compatibility for your compiler.
//**********************************************************************

/****************************/
/* Standard C include files */
/****************************/
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <conio.h>
#include <graph.h>

/**********************************/
/* M-Control Program include file */
/**********************************/
#include "mcalls.h"


/***********************/
/* Function prototypes */
/***********************/
int main ( int argc, char *argv[] );
void setBIOSbordercolor ( void );
void print_rcvbuffer ( CP_FRCVBUF p );

/******************************/
/* M-Control Global Variables */
/******************************/
CP_FRCVBUF  prcvbuffer;


/**********************/
/* Begin main program */
/**********************/
int main ( int argc, char *argv[] )
{
   BYTE huge    *buffer;                     /* Pointer to user's buffer */
   ULONG         bufferneed;                 /* Size of needed buffer    */
   ULONG         bufferalloc;                /* Size of user's buffer    */
   SHORT         xorg;                       /* x origin of capture area */
   SHORT         yorg;                       /* y origin of capture area */
   USHORT        xlen;                       /* x length of capture area */
   USHORT        ylen;                       /* y length of capture area */


   /* Check for correct number of parms */
   if ( argc < 5 ) {
     printf ( "SYNTAX: CAP2BUF 0 0 200 150 65000\n" );
     printf ( "Where the first two parms are the origin of x and y\n" );
     printf ( "and the next two parms are the width and height.\n" );
     printf ( "The last parm is the buffer size to allocate.\n" );
     printf ( "Buffer size is calculated internally if omitted.\n" );
     return ( 2 );
   }

   xorg = atoi ( argv[1] );
   yorg = atoi ( argv[2] );
   xlen = atoi ( argv[3] );
   ylen = atoi ( argv[4] );

   /* Check if we can start API */
   if ( !cp_loaded ( ) ) {
     printf ( "MMedia must be started before this program will run " );
     return ( 3 );
   }

   /* Set mode to 640x480x16 colors ( mode 0x12 ) */
   _setvideomode ( _VRES16COLOR );
   setBIOSbordercolor ( );

   /* Initialize the M-Motion Card */
   cp_init ( );
   cp_purge_que ( );
   cpvid_Reset ( );
   cpvid_TranspColor ( FALSE, 0 );
   cpvid_Transparency ( TRUE );
   cpvid_Freeze ( FALSE );

   /* Calculate buffer size needed */
   /* We could issue a CAPTURE VIDEO (X'12) without bytes 10-23
      and checked response bytes 2-5 for size but this is easier.
      Note: 16L is for the header. */
   bufferneed = (ULONG)xlen * (ULONG)ylen * 3L / 2L + 16L;

   /* Check to see if user defined the buffer */
   if (argc > 5)
      bufferalloc = atol ( argv[5] ); /* Use user's buffer size */
   else
      bufferalloc = bufferneed;       /* Use size needed */

   /* allocate a huge buffer since it may need to be larger than 64K. */
   if ( buffer = (BYTE huge *)halloc(bufferalloc, sizeof(BYTE))) {

      printf( " Buffer size required: %ld bytes\n", bufferneed );
      printf( "Allocated buffer size: %ld bytes\n\n", bufferalloc );
      printf( "Press any key to capture video..." );
      getch ( );
      cpvid_Freeze ( TRUE );

      printf ( "\n\nCapturing video to buffer..." );

      /* This is a new MCALLS function! */
      prcvbuffer = cpvid_CaptureBuffer ( 0,xorg,yorg,xlen,ylen,buffer,bufferalloc);

      /* Check to see if it failed */
      if ( prcvbuffer->data[0] != 0 ) {
         printf("\aFailed ");
         print_rcvbuffer ( prcvbuffer );
         }
      else {
         cpvid_ClearAllBlack ( );
         printf("Capture complete\n\nPress any key to restore video...");
         getch ( );

         printf("\n\nRestoring video from buffer...");

         /* This is a new MCALLS function! */
         prcvbuffer = cpvid_RestoreBuffer ( 9,xorg,yorg,xlen,ylen,buffer,bufferalloc,0,0);

         /* Check to see if it failed */
         if ( prcvbuffer->data[0] != 0 ) {
            printf("\aFailed ");
            print_rcvbuffer ( prcvbuffer );
            }
         else
            printf( "Restore complete\n\nPress any key to exit..." );
         }

      /* free the buffer */
      hfree ( buffer );
      }
   else {
      printf ( "\n\nMemory allocation error...could not allocate buffer." );
      printf ( "\nTry capturing a smaller region.\n" );
      printf ( "\nPress any key to exit...\n" );
      }

   getch ( );
   cpvid_Reset ( );
   cp_exit ( );

   _setvideomode ( _DEFAULTMODE );
}

//***********************************************************
// SPECIAL FUNCTIONS
//***********************************************************
void setBIOSbordercolor (void)
  {
  union REGS myregs;

  /* just set the border color to  0 */
  myregs.x.ax = 0x1001;
  myregs.x.bx = 0x0000;

  int86 ( 0x10, &myregs, &myregs );
  }

/*****************************************************************************
*  FUNCTION: print_rcvbuffer()
*
*  DESCRIPTION:  Print contents of the M-Control Receive Buffer
*
*  INPUT:
*     Pointer to the Receive Buffer
*
*  RETURNS:  NONE
*
*****************************************************************************/

void print_rcvbuffer ( CP_FRCVBUF p )
{
   int i, n;

   n = p->count - 5;

   for ( i = 0; i < n; ++i )
      printf(" %x", p->data[i] );

}
