//**********************************************************************
// Copyright IBM Corporation 1989, 1992
//**********************************************************************
// File : clipaud.c
//
// Description :
//      This program will clip audio from the beginning or end of an
// M-Motion audio file.
//
// NOTE : ONLY single-segment audio files are valid input.  Libraries of
//        audio segments will not be accepted.
//
// To run:
//      1. Compile
//      2. Obtain a digized audio file, for example "SONG.AUD"
//      3. Run clipaud
//
// Input Parms:
//      #1 : M-Motion audio file name (e.g. SONG.AUD)
//      #2 : Number of milliseconds to remove (e.g. 16)
//      #3 : Beginning or end indicator : "Begin" or "End"
//
// Compilation Instructions:
//      1) First, compile MCALLS.C into MCALLS.OBJ.
//
//      2) Compile CLIPAUD.C with IBM C/2 version 1.1 using the
//         following parms:
//
//              cl /AL /G2s /Zp clipaud.c mcalls.obj /link graphics
//
//         If you created combined libraries with graphics you will not
//         need to link graphics at this stage.
//
//         Do NOT select/use ANSI compatibility for your compiler.
//**********************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <memory.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>

#define  TRUE  1
#define  FALSE 0

/****************************/
/* Global types & variables */
/****************************/
typedef struct _AUDIO_HEADER {  // Audio file header structure
  char  name[8];
  char  reserved[4];
  unsigned long msec;
  unsigned long size;
} AUDIO_HEADER;

AUDIO_HEADER aud_header;

char    pattern[12] = { "PCM08M08" };
char    buffer[4096];
char    mytmp[] = { "clipaud.tmp" };

//*************************************************************
// Function Prototypes
//*************************************************************
int  copy_files ( int in_fh, int out_fh, unsigned long copy_length );
int  main (int argc,  char *argv[]);

//*************************************************************
// Function : copy_files
//
// Description :
//      Copies the data from one file to another, for the
// specified length.
//*************************************************************
int copy_files ( int in_fh, int out_fh, unsigned long copy_length )

{
  int   read_length;
  int   write_length;

  do {
    /* Is the current buffer large enough to hold the entire copy? */
    /*          If not, we will need multiple read/writes */

    if ( copy_length > (long)sizeof ( buffer ) )
      read_length = sizeof ( buffer );
    else
      read_length = (unsigned int) copy_length;

    // read a chunk into the buffer
    write_length = read ( in_fh, buffer, read_length );

    // write the buffer back out
    if ( write ( out_fh, buffer, write_length ) == -1 ) {
      return ( 1 );
      }

    if ( write_length != read_length ) {
      return ( 1 );
      }

    }
  while ( copy_length -= (unsigned long)write_length );

  return ( 0 );
}



// *************************************************************
// Function : main
//
// Description :
//    Draws the grid every time the position in the segment is
// changed.
// *************************************************************
main(argc, argv)
  int argc;
  char *argv[];
  {
  int             fh;
  int             fht;
  unsigned long   msec;
  unsigned long   clipsize;
  unsigned long   filelength;
  char            c;
  char            copystring[256];

  // we need three arguments, the file name, milliseconds, and position
  if ( argc != 4 )
    {
    printf ("CLIPAUD will clip audio data from the beginning or end of an\n");
    printf ("  M-Motion audio file.  Only 1 segment audio files are supported.\n\n");
    printf ("SYNTAX: clipaud <filename> <milliseconds to clip> <Begin / End>\n");
    printf ("Examples :  clipaud song.aud 16 Begin\n");
    printf ("     -OR-   clipaud song.aud 16 End\n");
    return (1);
    }

  // the first argument must be a valid file name, we will open the file
  // to test for this
  if ((fh = open ( argv[1], O_RDONLY | O_BINARY)) == -1)
    {
    printf ( "Cannot open the file named %s\n", argv[1] );
    return ( 1 );
    }

  // open a temporary file to write into
  if ((fht = open ( mytmp, O_CREAT | O_EXCL | O_RDWR | O_BINARY,
                     S_IREAD | S_IWRITE )) == -1)
    {
    close ( fh );
    printf ( "Cannot create my temporary file named %s\n", mytmp );
    return ( 1 );
    }

  /* determine the TOTAL file length */
  filelength = lseek ( fh, 0L, SEEK_END);


  // Read in the first 20 bytes - it should be an audio header
  lseek ( fh, 0L, SEEK_SET);
  read ( fh, (char *)&aud_header, sizeof ( aud_header ) );

  // Check for the validity of the M-Motion audio header
  if ( memcmp ( aud_header.name, pattern, sizeof ( pattern ) ) )
    {
    // not a valid M-Motion audio file header
    printf ( "%s is not a valid M-Motion Audio file\n", argv[1] );
    close ( fh );
    close ( fht );
    remove ( mytmp );
    return ( 1 );
    }

  /* ENSURE that this is a single segment audio file - NO OTHER DATA */
  if (filelength != aud_header.size)
    {
    printf ("There is either extra data or more than one audio segment \n");
    printf ("   in the file.  CLIPAUD will not run so that your other  \n");
    printf ("   data will not be destroyed.");
    close ( fh );
    close ( fht );
    remove ( mytmp );
    return ( 1 );
    }

  // we now have the header, get the milliseconds to clip
  msec = atol ( argv[2] );
  clipsize = ( msec * 63 ) / 8;
  aud_header.msec -= msec;
  aud_header.size -= clipsize;

  // determine if we are to use the Beginning or the End of the file
  c = argv[3][0];


  /************************/
  /* Is it the beginning? */
  /************************/
  if ( ( c == 'b') || ( c == 'B' ) )
    {
    // write the header back out
    write ( fht, (char *)&aud_header, sizeof( aud_header ) );

    // move past the length of data to be clipped from the audio file
    lseek ( fh, clipsize, SEEK_CUR);

    // The remaining data will be clipped later
    }

  /*********************/
  /* Is it at the end? */
  /*********************/
  else if ( ( c == 'e' ) || ( c == 'E' ) )
    {
    // write the header back out
    write ( fht, (char *)&aud_header, sizeof( aud_header ) );

    // clip the end
    // Don't do anything now - we will just subtract the "length" parm
    //    from the copy length later
    }

  /*********************/
  /* Invalid reference */
  /*********************/
  else
    {
    printf ( "You must specify Begin or End as the last parm" );
    close ( fh );
    close ( fht );
    remove ( mytmp );
    return ( 1 );
    }

  /******************************************************************/
  /* Copy from the current loc (begin or clipped begin) to the end  */
  /*        Copy is from the source file to the temp file           */
  /******************************************************************/
  if ( copy_files ( fh, fht, aud_header.size - sizeof ( aud_header ) ) )
    {
    printf ( "file copy failed" );
    close ( fh );
    close ( fht );
    remove ( mytmp );
    return ( 1 );
    }

  close ( fh );
  close ( fht );

  // now use dos to copy the file back on to the original
  sprintf ( copystring, "copy %s/b %s/b >NUL", mytmp, argv[1] );

  if ( system ( copystring ) != -1 )
    printf ( "       Complete\n" );
  else
    printf ( "Problem in DOS Copy operation, operation not completed" );

  remove ( mytmp );

  return ( 0 );
  }
