//**********************************************************************
// Copyright IBM Corporation 1989, 1992
//**********************************************************************
// File : graphaud.c
//
// Description :
//      This program will display a graph of an MMotion digitized audio file.
// The graph represents the frequency of signals that are recorded in
// the file.
//
//      To run:
//              1. Compile
//              2. Obtain a digized audio file, for example "SONG.AUD"
//              3. Type "graphaud song.aud"
//              4. A graph will be displayed.
//              5. Press the ESC key to quit.
//
// NOTE : ONLY single-segment audio files are displayed.  Libraries of
//        audio segments will not be accepted.
//
// Compilation Instructions:
//      1) First, compile MCALLS.C into MCALL.OBJ.
//
//      2) Compile GRAPHAUD.C with IBM C/2 version 1.1 using the
//         following parms
//
//              cl /AL /G2s /Zp graphaud.c mcalls.obj /link graphics
//
//         If you created combined libraries with graphics you will not
//         need to link graphics at this stage.
//
//         Do NOT select/use ANSI compatibility for your compiler.
//**********************************************************************

#include <stdio.h>
#include <graph.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>

typedef unsigned char BYTE;

/***********/
/* Defines */
/***********/
#define OL 6
#define OB 261
#define OT 6
#define OR 633

/****************************/
/* Global types & variables */
/****************************/
typedef struct _AUDIO_HEADER {  // Audio file header structure
  char  name[8];
  char  reserved[4];
  unsigned long msec;
  unsigned long size;
} AUDIO_HEADER;

AUDIO_HEADER aud_header;
#define HDSIZE sizeof(aud_header)

BYTE adata[628];
#define ADSIZE sizeof(adata)

BYTE filename[100];             // Name of the audio file to display

//*************************************************************
// Function Prototypes
//*************************************************************
int  draw_grid (void);
void draw_instructions (void);
int  plot (BYTE *data);
int  plot_at (int fh, unsigned long msec);
int  main (int argc,  char *argv[]);

//*************************************************************
// Function : draw_grid
//
// Description :
//    Draws the grid every time the position in the segment is
// changed.
//*************************************************************
int draw_grid ( )

{
   _settextcolor ( 14 );

   // clear the old grid
   _setcolor ( 0 );
   _rectangle ( _GFILLINTERIOR, OL, OT, OR, OB );

   _setcolor ( 15 );

   // draw a box with an interior dimension of 630 X 258
   _rectangle ( _GBORDER, 0, 0, 639, 267 );

   _setcolor ( 3 );

   // dotted lines
   _setlinestyle ( 0xAAAA );

   // draw 0 line
   _moveto ( OL, OB - 128 );
   _lineto ( OR, OB - 128 );

   // draw bounding box
   _rectangle ( _GBORDER, OL, OT, OR, OB );

   // draw 20 ms markers
   _moveto ( OL + 156, OB );
   _lineto ( OL + 156, OT );

   _moveto ( OL + 313, OB );
   _lineto ( OL + 313, OT );

   _moveto ( OL + 471, OB );
   _lineto ( OL + 471, OT );


   // draw 10 ms markers
   _moveto ( OL + 156 - 78, OB );
   _lineto ( OL + 156 - 78, OB - 20 );

   _moveto ( OL + 313 - 78, OB );
   _lineto ( OL + 313 - 78, OB - 20 );

   _moveto ( OL + 471 - 78, OB );
   _lineto ( OL + 471 - 78, OB - 20 );

   _moveto ( OL + 471 + 78, OB );
   _lineto ( OL + 471 + 78, OB - 20 );

   _setlinestyle ( 0xFFFF );

   return ( 0 );
}

//*************************************************************
// Function : draw_instructions
//
// Description :
//    Draws the instructions when the program first starts.
//*************************************************************
void draw_instructions ()
  {
  char number[12];

  _settextcolor ( 15 );

  _settextposition ( 20,  0);
  _outtext ("Frequency Graph of File :  ");
  _settextposition ( 20, 30);
  _outtext (filename);

  _settextposition ( 21,  0);
  _outtext ("Segment Length in Milliseconds :  ");
  _settextposition ( 21, 34);
  sprintf ( number, "%-10ld", aud_header.msec );
  _outtext (number);

  _settextposition ( 23, 0);
  _outtext ("Instructions");

  _settextposition ( 24,  5);
  _outtext (" LEFT Arrow shifts display BACKWARD 20 milliseconds ");

  _settextposition ( 25,  5);
  _outtext (" RIGHT Arrow shifts display FORWARD 20 milliseconds ");

  _settextposition ( 26,  5);
  _outtext (" MINUS key shifts display BACKWARD 80 milliseconds ");

  _settextposition ( 27,  5);
  _outtext (" PLUS key shifts display FORWARD 80 milliseconds ");

  _settextposition ( 29,  5);
  _outtext (" ESC key QUITS, return to DOS");

  }

//*************************************************************
// Function : plot
//
// Description :
//    Calls draw_grid () and plots the data every time the
// segment position changes.
//*************************************************************
int plot ( BYTE *data )

{
  int pos;

  draw_grid ( );


  _setcolor ( 12 );

  _moveto ( OL, OB - *data++ );

  for ( pos=OL+1; pos<=OR; pos++ ) {
    _lineto ( pos, OB - *data++ );
  }

  return ( 0 );
}


//*************************************************************
// Function : plot_at
//
// Description :
//    Reads the required data from the file, calls plot () to
// draw the grid and data, then displays the grid coordinates
// every time the segment position is changed.
//*************************************************************
int plot_at ( int fh, unsigned long msec )

  {
  unsigned long   seek_pos = HDSIZE;
  char            number[12];
  int             read_cnt;

  seek_pos += ( msec / 20 ) * 157;
  lseek ( fh, seek_pos, 0 );

  read_cnt = read ( fh, adata, ADSIZE );
  if ( read_cnt < ADSIZE ) {
    if ( read_cnt ) {
      do {
        int small_read;
        adata[read_cnt++] = 0x80;
        small_read = read ( fh, &adata[read_cnt], ADSIZE - read_cnt );
        if ( small_read ) {
          read_cnt += small_read;
        } else {
          break;
        }
      } while ( read_cnt < ADSIZE );
    }
  }

  plot ( adata );

  /*************************/
  /* Print the grid coords */
  /*************************/
  sprintf ( number, "%-10ld", msec );
  _settextposition ( 18, 1 );
  _outtext ( number );

  sprintf ( number, "%-10ld", msec+20 );
  _settextposition ( 18, 20 );
  _outtext ( number );

  sprintf ( number, "%-10ld", msec+40 );
  _settextposition ( 18, 40 );
  _outtext ( number );

  sprintf ( number, "%-10ld", msec+60 );
  _settextposition ( 18, 60 );
  _outtext ( number );

  sprintf ( number, "%10ld", msec+80 );
  _settextposition ( 18, 70 );
  _outtext ( number );

  _settextposition ( 19, 25);
  _outtext ("Segment Position in Milliseconds");

  return ( 0 );
  }


//*************************************************************
// Function : main
//
// Description :
//      Checks the input parms to determine if a valid M-Motion
// audio file is specified.  If so, the instructions are
// displayed, and the first part of the audio segment is graphed.
//      The remainder of the function is a loop allowing the
// user to press keys to change the position in the segment to
// be displayed.
//*************************************************************
main(argc, argv)
  int argc;
  char *argv[];

  {
  int            fh;
  int            c;
  int            quit = 0;
  unsigned long  msec = 0;

  /* Are there enough parameters? */
  if (argc < 2)
     {
     printf ("GRAPHAUD : \n");
     printf ("   Displays the frequency data in a MMotion Digitized Audio File\n");
     printf ("   You can move forward and backward in the file.\n\n");
     printf ("Provide the name of the desired audio file to GRAPHAUD by typing\n");
     printf ("   GRAPHAUD audiofilename\n");
     }

  /* Open the file */
  if ( (fh = open ( argv[1], O_RDONLY, O_BINARY ) ) != -1 )
     {
     strcpy (filename, argv[1]);

     /* Read in the header.  This program ONLY uses MMotion Audio files
        that have the audio segment as the first data in the file */
     read ( fh, (char *)&aud_header, HDSIZE );

     /* Ensure that it contains an MMOTION Audio segment */
     if (!strncmp (aud_header.name, "PCM08M08", 8))
        {

        _setvideomode ( _VRES16COLOR );

        draw_instructions ();

        do {
           if ( !kbhit ( ) ) {
             plot_at ( fh, msec );
             }

           c = getch ( );

           if ( !c ) {
             c = getch ( );
             switch ( c )
                {
                case 75:    // Left Arrow : LEFT BY 20 milliseconds
                  if ( msec >= 20 ) msec -= 20;
                  break;

                case 77:    // Right Arrow : RIGHT BY 20 milliseconds
                  if ( msec <= aud_header.msec - 60 ) msec += 20;
                  break;
                }
             }
           else
             {
             switch ( c )
                {
                case '-':   // Minus : LEFT BY 80 milliseconds
                  if ( msec >= 80 ) msec -= 80;
                  break;

                case '+':   // Plus : RIGHT BY 20 milliseconds
                  if ( msec <= aud_header.msec - 140 ) msec += 80;
                  break;

                case 0x1B:
                  quit = 1;
                  break;
                }
             }
           } while ( !quit ); // DO-WHILE wait for ESC char

        _setvideomode ( _DEFAULTMODE );
        }
     else
        printf (" Sorry, the file you requested is NOT an MMotion Audio File\n");

     close ( fh );
     }
  else {
     printf (" Sorry, the file you requested to display could not be opened\n");
     printf ("    Please check to see if it exists, and try again.\n");
     }

  return ( 0 );
  }
