//*************************************************************************
// Copyright IBM Corporation 1989, 1992
/**************************************************************************
*
*  File:  MCALLS.C
*
*  DESCRIPTION:
*     M-Control Program 'C' function call interface examples.
*
*     These functions are provided for your study and use.  They encompass
*  most of the actions that can be performed by the M-Control Program
*  on the video image and digitized audio.  Two functions for starting and
*  stopping a video player are also included.
*     These functions can be used as the basis for your own multimedia
*  function library, or can be used as supplied.
*     There are 4 sample programs also included that use the MCALLS functions.
*  These are:
*       AR.C          Digitized Audio Record
*       AP.C          Digitized Audio Play sample
*       AVCVIDEO.C    Displays a 640x480 256 color Audio Video Connection
*                     image in video.
*       SHOWVID.C     Provides a test set of functions on the MMotion Video
*                     adapter.  Various video levels, characteristics, and
*                     video image capture/restore are included.
*
*  Compilation Instructions:
*      1) Compile MCALLS.C with IBM C/2 version 1.1 using the following parms
*
*              cl -c /AL /G2s /Zp mcalls.c
*
*         Do NOT select/use ANSI compatibility for your compiler.
*
*  CHANGES:
*   01.01  09-09-91  New function cpaud_Wait()
*
*
***************************************************************************
*
*  Misc Functions:
*     cp_snd
*     cp_rcv
*     cp_wake_handler
*     cp_loaded
*     cp_init
*     cp_exit
*     cp_purge_que
*     cp_YUVtoRGB
*
*  Video Control Functions:             Audio Control Functions:
*     cpvid_Reset                          cpaud_Reset
*     cpvid_Control                        cpaud_Follow
*     cpvid_Source                         cpaud_Inputs
*     cpvid_VideoOnly                      cpaud_Outputs
*     cpvid_Transparency                   cpaud_ReadStatus
*     cpvid_BlackWhite
*     cpvid_ReadStatus                     cpaud_Volume
*     cpvid_ReadImageParms                 cpaud_DefVolume
*     cpvid_ConfigInputs
*     cpvid_TranspColor                    cpaud_Balance
*     cpvid_RestoreStatus                  cpaud_DefBalance
*     cpvid_SetCoordinates                 cpaud_Bass
*     cpvid_SetFullscreenCoord             cpaud_DefBass
*     cpvid_VideoProtect                   cpaud_Treble
*     cpvid_VideoProtectAll                cpaud_DefTreble
*     cpvid_GraphicProtect                 cpaud_Fade
*     cpvid_GraphicProtectAll              cpaud_DefFade
*     cpvid_OptimizeVid                    cpaud_BufferInit
*     cpvid_Brightness                     cpaud_BufferActivate
*     cpvid_DefBrightness                  cpaud_BufferClose
*     cpvid_Contrast                       cpaud_OpenPlay
*     cpvid_DefContrast
*     cpvid_Color                          cpaud_OpenRecord
*     cpvid_DefColor                       cpaud_Close
*     cpvid_Tint
*     cpvid_DefTint                        cpaud_AhandleStatus
*     cpvid_Fade                           cpaud_RecordVolume
*     cpvid_DefFade
*     cpvid_Sharpness                      cpaud_Record
*     cpvid_DefSharpness                   cpaud_Monitor
*     cpvid_Freeze
*     cpvid_Capture                        cpaud_Mute
*     cpvid_CaptureBuffer
*     cpvid_CaptureFile                    cpaud_Stop
*     cpvid_Restore                        cpaud_Play
*     cpvid_RestoreBuffer
*     cpvid_RestoreFile                    cpaud_PlaySegment
*     cpvid_Copy                           cpaud_PlayCurrent
*     cpvid_Clear                          cpaud_PlayMulti
*     cpvid_ClearAllBlack
*     cpvid_Show256Pic
*     cpvid_MultiVid
*
*
****************************************************************************/

/****************************************************************************/
/* complier library headers                                                 */
/****************************************************************************/
#include <dos.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>

/**********************************************/
/* M-Control Program specific header          */
/**********************************************/
#include "mcalls.h"

/*****************************************************************************
*   M-Control Program Global and Control Structures
*   (required for 7F interface)
*****************************************************************************/

struct {                    /* M-Control Program Global Variable Structure*/
  USHORT avisndst;            /* Global Send Status Variable                */
  USHORT avirecst;            /* Global Receive Status Variable             */
  USHORT avista0;             /* Global Destination Status Variable         */
  USHORT aviver;              /* IBM M-Control Program Version Number       */
  USHORT avidcode;            /* IBM Infowindow compatibility               */
  USHORT avipcode;            /* IBM Infowindow compatibility               */
} near cp_globals;

struct {                      /* M-Control Program Control Block Structure  */
  USHORT   dir;               /* AVISND or AVIREC                           */
  PVOID  dbuff;               /* Data Buffer Address                        */
  PVOID  globals;             /* Global Variable Address                    */
} near cp_cntrl;       /* note must be in default data segment!!            */

/* Set defines for each destination status variable                         */
#define ERROR_HAS_MSG      ( cp_globals.avista0 & 0x8000 )
#define SEND_ERR           ( cp_globals.avisndst )
#define SYSTEM_HAS_MSG    (( cp_globals.avista0 & 0x4000 ) || SEND_ERR )
#define VIDEO_HAS_MSG     (( cp_globals.avista0 & 0x2000 ) || SEND_ERR )
#define POINTER_HAS_MSG   (( cp_globals.avista0 & 0x0400 ) || SEND_ERR )
#define VDP1_HAS_MSG      (( cp_globals.avista0 & 0x0200 ) || SEND_ERR )
#define VDP2_HAS_MSG      (( cp_globals.avista0 & 0x0100 ) || SEND_ERR )
#define TIME_HAS_MSG      (( cp_globals.avista0 & 0x0080 ) || SEND_ERR )
#define AUDIO_HAS_MSG     (( cp_globals.avista0 & 0x0040 ) || SEND_ERR )

#define READLENGTH  (UINT)20000

/*****************************************************************************
*  internal variables
*****************************************************************************/
static BOOL      Q = FALSE;
static BOOL      play_resp = FALSE;
static CP_RCVBUF rcvbuf;

/*****************************************************************************
*  Audio Variables
*****************************************************************************/
static BOOL     play_response = FALSE;
static ULONG    record_msec = 0L;
static BOOL     fader_response = FALSE;

/* Report Audio Change status variables */
BOOL            cpaud_in_record       = FALSE;
USHORT          cpaud_in_play         = FALSE;
BYTE            cpaud_repchange_ahand = 0;
BYTE            cpaud_repchange_seg   = 0;
ULONG           cpaud_repchange_pos   = 0L;

/*****************************************************************************
*  Video Control Variables
*****************************************************************************/
BOOL     in_video_fade = FALSE;


/*****************************************************************************/
/*****************************************************************************/
/*                  Control program access commands                          */
/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************
*  FUNCTION: cp_snd()
*
*  DESCRIPTION:  Send command to M-Control Program
*
*  INPUT:
*     cmdbuff : Ptr to command data structure (e.g. CP_CMD, CP_LEVELS, etc)
*
*  RETURNS:
*     Send status AVISNDST
*
*****************************************************************************/

USHORT cp_snd ( PVOID cmdbuff )
{
  USHORT far *p;
  union   REGS myregs;
  struct SREGS segregs;

  cp_cntrl.dir = 0;                  /* send */
  cp_cntrl.dbuff = cmdbuff;
  cp_cntrl.globals = &cp_globals;

  myregs.x.ax = 0x0107;
  p = (USHORT *)&cp_cntrl;
  myregs.x.dx = FP_OFF (p);
  segregs.ds = FP_SEG (p);
  int86x (0x7F, &myregs, &myregs, &segregs);

  return ( cp_globals.avisndst );
}

/*****************************************************************************
*  FUNCTION: cp_rcv()
*
*  DESCRIPTION:  Receive response from M-Control Program
*
*  INPUT:
*     Dest : The type of message to expect/desire a response from
*
*  RETURNS:  0 = Successful
*           >0 = Error:  Reponse Data Byte 1
*           <0 = Error:  -AVIRECST
*
*****************************************************************************/
SHORT cp_rcv ( BYTE dest )
{
  USHORT far *p;
  union REGS myregs;
  struct SREGS segregs;

  rcvbuf.count = sizeof ( rcvbuf );
  rcvbuf.dest = dest;

  cp_cntrl.dir = 1;            /* receive */
  cp_cntrl.dbuff = &rcvbuf;
  cp_cntrl.globals = &cp_globals;

  myregs.x.ax = 0x0107;
  p = (USHORT *)&cp_cntrl;
  myregs.x.dx = FP_OFF (p);
  segregs.ds  = FP_SEG (p);
  int86x (0x7F, &myregs, &myregs, &segregs);

  if ( cp_globals.avirecst == 0 )        /* If response returned in buffer */
     return ( (SHORT)rcvbuf.data[0] );   /* Return Response Byte 1 */
  else                                   /* Else return -AVIRECST */
     return( -( cp_globals.avirecst ) );
}

/*****************************************************************************
*  FUNCTION: cp_wake_handler()
*
*  DESCRIPTION:
*       Set up wake up handler for M-Control Program.  All responses are
*  passed through this function.  Some are queued, so that the application
*  or MCALLS functions can access the response data by using cp_rcv.
*       Commands that are sent and the response IS NOT evaluated (such as
*  audio and video fade) are NOT queued.
*
*****************************************************************************/
//
// A bug in MS C 5.1 generates incorrect code for this function.  If you
// use MS C 5.1, use the alternate function declaration.
// For IBM C/2 version and MS C 6.0 version
static VOID far cdecl _loadds _saveregs cp_wake_handler (CP_RCVBUF *response,
                                                         USHORT len )
// For MS C 5.1 version
//static VOID far cdecl _loadds _saveregs cp_wake_handler ( SHORT dummy,
//                                     CP_RCVBUF *response, USHORT len )
{
  /* this routine will be called each time a response is queued */

  /******************/
  /* VIDEO Commands */
  /******************/
  if ( response->dest == VIDEO_DEST )
     {
     switch ( response->id )
        {
        case VID_FADE_ID:
          in_video_fade = FALSE;
          /* don't que this response - since we don't wait for completion */
          response->count = 0;
          break;
        default:
          break;
        }
     }

  /****************************/
  /* DIGITIZED AUDIO Commands */
  /****************************/
  else if ( response->dest == AUDIO_DEST )
     {
     switch ( response->cmd )
        {
        case AUD_FADE_CMD:
           /* don't que it, no one will read it */
           response->count = 0;
           fader_response = TRUE;
           break;

        case AUD_STOP_CMD:
           if ( response->data[0] )  /* ERROR? */
             {}
           else
             {
             if ( cpaud_in_record )
               {
               /* grab the milliseconds of this recording */
               record_msec = *(ULONG *)(&response->data[5]);
               }

             cpaud_in_record = FALSE;
             cpaud_in_play = FALSE;
             }

           /* If this is a SYSTEM ORIGINATED STOP
              don't que an automatic response    */
           if (response->id != AUD_STOP_ID)
              response->count = 0;

           break;

        case AUD_PLAY_CMD:
        case AUD_PLAY_MULTIPLE_CMD:
           if ( Q )
             {
             Q = FALSE;
             response->count = 0;
             }
           play_resp = TRUE;
           break;

        case AUD_REPORT_CHANGE_CMD:
           response->count = 0;
           if ( !response->data[0] )
             {
             cpaud_repchange_ahand = response->data[1];
             cpaud_repchange_seg   = response->data[2];
             cpaud_repchange_pos   = *(ULONG *)&response->data[3];
             }
           break;

        default:
          break;
        }
     }
  return;
}


/*****************************************************************************
*  Function: cp_loaded
*
*  Description:
*       Checks if M-Control Program is loaded.  Looks at interrupt 0x7F
*  to see if a interrupt handler has been installed.  0x7F MUST BE ACTIVE
*  for multimedia functions to operate.  Serious errors will occur if the
*  M-Control Program is not loaded.
*
*  Returns:
*       TRUE  if M-Control is active
*       FALSE if error - M-Control has NOT been loaded.
*****************************************************************************/
BOOL cp_loaded ( )
{
  /* confirm that int 7F points to something */
  if ( *(PVOID  *)( 0x7FL * 4L ) )
    return ( TRUE );
  else
    return ( FALSE );
}


/*****************************************************************************
*  FUNCTION: cp_init()
*
*  DESCRIPTION:  Initalize the M-Control Program
*
*  INPUT:  None
*
*  RETURNS:  0 = Successful
*           !0 = Pointer to Receive Buffer from M-Control Program
*
*****************************************************************************/

typedef struct _SYS_WAKE_UP {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  USHORT        segment;    /* Address of wake-up routine */
  USHORT        offset;
} SYS_WAKE_UP;


CP_FRCVBUF  cp_init ( )
{

  USHORT *usp;
  PVOID   fp;

  static CP_CMD          snd_buf;
  static SYS_WAKE_UP     sys_wake_up;

  snd_buf.count = sizeof (CP_CMD);
  snd_buf.id    = ID1;
  snd_buf.dest  = SYSTEM_DEST;
  snd_buf.cmd   = SOFT_START_CMD;

  /***************************/
  /* attempt to soft restart */
  /***************************/
  if ( !cp_snd ( &snd_buf ) )
     {
     while ( !SYSTEM_HAS_MSG );
     if ( cp_rcv ( SYSTEM_DEST ) )
        return ( &rcvbuf );
     }

  else
     /*****************************/
     /* Send stop/reset and start */
     /*****************************/
     {
     snd_buf.cmd   = STOP_RESET_CMD;

     cp_snd ( &snd_buf );

     while ( !SYSTEM_HAS_MSG );
     if ( cp_rcv ( SYSTEM_DEST ) )
        return ( &rcvbuf );

     snd_buf.cmd = START_CMD;

     cp_snd ( &snd_buf );


     while ( !SYSTEM_HAS_MSG );
     if ( cp_rcv ( SYSTEM_DEST ) )
        return ( &rcvbuf );
     }


/*cp_purge_que ( );*/ /* Remove any outstanding messages from the message que */

  /***************************/
  /* set our wake-up address */
  /***************************/
  usp = (USHORT *)&sys_wake_up.segment;
  fp = (PVOID) cp_wake_handler;
  usp[0] = FP_SEG ( fp );
  usp[1] = FP_OFF ( fp );

  sys_wake_up.count = sizeof (SYS_WAKE_UP);
  sys_wake_up.id    = ID1;
  sys_wake_up.dest  = SYSTEM_DEST;
  sys_wake_up.cmd   = WAKE_UP_CMD;

  cp_snd ( &sys_wake_up );

  while ( !SYSTEM_HAS_MSG );
  if ( cp_rcv ( SYSTEM_DEST ) )
     return ( &rcvbuf );

  /***********************************************************/
  /* Read the configuration data                             */
  /***********************************************************/
  snd_buf.count = sizeof (CP_CMD);
  snd_buf.cmd   = RD_SYS_CONFIG_CMD;

  cp_snd ( &snd_buf );

  while ( !SYSTEM_HAS_MSG );
  cp_rcv ( SYSTEM_DEST );

#ifdef VDP_SAMPLE_ONLY
This section is for your review and study only - video player
support is not included in the example due to the error checking
and recovery requirements.


  /*********************************/
  /* Spin up and check VDP players */
  /*********************************/
  if ( !rcvbuf.data[0] )     /* no error on get_config command */
     {
     vdp1 = rcvbuf.data[3];
     vdp2 = rcvbuf.data[4];

     vdp_still_frame.count = sizeof (VDP_STILL_FRAME);
     vdp_still_frame.id    = ID1;
     vdp_still_frame.cmd   = VDP_STILL_FRAME_CMD;
     vdp_still_frame.frame = 0xFFFF;

     snd_buf.count = sizeof (CP_CMD);
     snd_buf.cmd   = VDP_INIT_CMD;

     /********************/
     /* test for player1 */
     /********************/
     if ( vdp1 & 0x80 )
        {
        /********************************************************/
        /* Check if Player 1 is initialized - try a still frame */
        /********************************************************/
        vdp_still_frame.dest  = VDP1_DEST;

        cp_snd ( &vdp_still_frame );

        while ( !VDP1_HAS_MSG );
        cp_rcv ( VDP1_DEST );

        /*******************************************/
        /* Attempt spin up if an error is returned */
        /*******************************************/
        if ( rcvbuf.data[0] )
           {
           snd_buf.dest  = VDP1_DEST;

           cp_snd ( &snd_buf );

           while ( !VDP1_HAS_MSG );
           if ( cp_rcv ( VDP1_DEST ) )
              return ( &rcvbuf );
           }
        }

     /********************/
     /* test for player2 */
     /********************/
     if ( vdp2 & 0x80 )
        {
        /********************************************************/
        /* Check if Player 2 is initialized - try a still frame */
        /********************************************************/
        vdp_still_frame.dest  = VDP2_DEST;

        cp_snd ( &vdp_still_frame );

        while ( !VDP2_HAS_MSG );
        cp_rcv ( VDP2_DEST );

        /*******************************************/
        /* Attempt spin up if an error is returned */
        /*******************************************/
        if ( rcvbuf.data[0] )
           {
           snd_buf.dest  = VDP2_DEST;

           cp_snd ( &snd_buf );

           while ( !VDP2_HAS_MSG );
           if ( cp_rcv ( VDP2_DEST ) )
              return ( &rcvbuf );
           }
        }
     }
  else
     return ( &rcvbuf );
#endif

  return ( 0L );    /* No error occurred */
}




/*****************************************************************************
* Function: cp_purge_que()
*
* Description:
*    Remove any outstanding messages from the message queue
*
* Input Parms : NONE
*
* Returns : VOID
*
*****************************************************************************/

VOID  cp_purge_que ( )
{
  while ( ERROR_HAS_MSG )
     cp_rcv ( ERROR_DEST );

  while ( SYSTEM_HAS_MSG )
     cp_rcv ( SYSTEM_DEST );

  while ( VIDEO_HAS_MSG )
     cp_rcv ( VIDEO_DEST );

  while ( POINTER_HAS_MSG )
     cp_rcv ( POINTER_DEST );

  while ( VDP1_HAS_MSG )
     cp_rcv ( VDP1_DEST );

  while ( VDP2_HAS_MSG )
     cp_rcv ( VDP2_DEST );

  while ( TIME_HAS_MSG )
     cp_rcv ( TIME_DEST );

  while ( AUDIO_HAS_MSG )
     cp_rcv ( AUDIO_DEST );

}


/*****************************************************************************
* Function: cp_exit()
*
* Description:
*    Exit M-Control Program
*
* Input Parms : None
*
* Returns:  0 = Successful
*          !0 = Pointer to Receive Buffer from M-Control Program
*
*****************************************************************************/
PBYTE cp_exit ( )
{
  CP_CMD snd_buf;


/*cp_purge_que ( );*//* Remove any outstanding messages from the message que */

  /* reset the wake address                                                  */

  snd_buf.count = sizeof (CP_CMD);
  snd_buf.id    = ID1;
  snd_buf.dest  = SYSTEM_DEST;
  snd_buf.cmd   = WAKE_UP_CMD;

  cp_snd ( &snd_buf );
  while ( !SYSTEM_HAS_MSG );
  if ( cp_rcv ( SYSTEM_DEST ) )
     return ( (PBYTE)&rcvbuf );

  /* soft stop                                                               */

  snd_buf.cmd   = SOFT_STOP_CMD;

  cp_snd ( &snd_buf );
  while ( !SYSTEM_HAS_MSG );
  if ( cp_rcv ( SYSTEM_DEST ) )
     return ( (PBYTE)&rcvbuf );

  return ( 0L );       /* No error occurred                                  */
}


/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
/*              CONVERSION FUNCTIONS                                         */
/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/


/*****************************************************************************
*  Function: cp_YUVtoRGB
*
*  DESCRIPTION: Convert three 16 bit WORDs representing 4 pels YUV to
*                       twelve BYTEs representing 4 pels RGB.
*
*  Input Parms:
*              YUVWord1 : The 48 bits representing 4 pels YUV.
*              YUVWord2      12 bits per pel, 8 bits Y and 4 bits of
*              YUVWord3      the shared U or V data.
*              *RGB1    : Ptr to one pel RGB structure (three BYTEs).
*              *RGB2    : Ptr to one pel RGB structure (three BYTEs).
*              *RGB3    : Ptr to one pel RGB structure (three BYTEs).
*              *RGB4    : Ptr to one pel RGB structure (three BYTEs).
*              Sat      : Percentage of Saturation of colors
*                         (Suggested Value: 80).
*
*  Returns: None
*****************************************************************************/
VOID cp_YUVtoRGB  (UINT YUVWord1, UINT YUVWord2, UINT YUVWord3,
                   RGBStruct *RGB1, RGBStruct *RGB2, RGBStruct *RGB3,
                   RGBStruct *RGB4, float Sat)
{
  BYTE  Y1, Y2, Y3, Y4;      // Unsigned byte (0-255), Y component of each pel
  SCHAR U1, V1;              // Signed byte (-127 to 127) shared U and V

  Y1 = (BYTE)((YUVWord1 & 0x0FE0)  >> 4);
  Y2 = (BYTE)((YUVWord2 & 0xFE00)  >> 8);
  Y3 = (BYTE)(((YUVWord2 & 0x000F) << 4) +
             ((YUVWord3 & 0xE000)  >> 12));
  Y4 = (BYTE)(YUVWord3 & 0x00FE);

  U1 = (SCHAR)((int)(((YUVWord1 & 0xC000) >> 8) +
                     ((YUVWord1 & 0x000C) << 2) +
                      (YUVWord2 & 0x00C0  >> 4) +
                     ((YUVWord3 & 0x0800) >> 10)));

  V1 = (SCHAR)((int)(((YUVWord1 & 0x3000) >> 6) +
                     ((YUVWord1 & 0x0003) << 4) +
                      (YUVWord2 & 0x0030  >> 2) +
                     ((YUVWord3 & 0x0200) >> 8)));

  ConvertYUVtoRGB (Y1, U1, V1, RGB1, Sat);
  ConvertYUVtoRGB (Y2, U1, V1, RGB2, Sat);
  ConvertYUVtoRGB (Y3, U1, V1, RGB3, Sat);
  ConvertYUVtoRGB (Y4, U1, V1, RGB4, Sat);
}


/*****************************************************************************
*  Function: ConvertYUVtoRGB
*
*  DESCRIPTION: Function used by cp_YUVtoRGB
*
*  Input Parms: Y          : Unsigned byte (0-255)     LUMINACE.
*               U and V    : Signed byte (-127 to 127) SHARED COLOR.
*               *RGB       : Ptr to RGB structure.
*               SATURATION : Saturation of colors (Suggested Value: 80% ).
*  Returns: None
*****************************************************************************/
VOID ConvertYUVtoRGB (BYTE Y, SCHAR U, SCHAR V, RGBStruct *RGB,
                      float SATURATION)
{
  double TR, TG, TB;
  double TU, TV;

  /* Change SATURATION into a percentage */
  SATURATION = SATURATION / 100.0;

  /* Convert back to RGB */
  TV = (((double)V * 179.0) / 127.0);
  TR = (TV * SATURATION) + (double)Y;
  if (TR > 255.0) RGB->Red = 255;
  else if (TR < 0.0) RGB->Red = 0;
  else RGB->Red = (BYTE)TR;

  TU = (((double)U * 227.0) / 127.0);
  TB = (TU * SATURATION) + (double)Y;
  if (TB > 255.0) RGB->Blue = 255;
  else if (TB < 0.0) RGB->Blue = 0;
  else RGB->Blue = (BYTE)TB;

  TG = (((double)Y * 170.0) - (TR * 51.0) - (TB * 19.0)) / 100.0;

  if (TG > 255.0) RGB->Green = 255;
  else if (TG < 0.0) RGB->Green = 0;
  else RGB->Green = (BYTE)TG;
}


/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
/*              VIDEO FUNCTIONS                                              */
/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/


/*****************************************************************************
*  Function: cpvid_Reset
*
*  Command Used: Video Reset (0x00)
*
*  Input Parms: None
*
*  Returns:
*            0 = Successful
*           >0 = Error:  Reponse Data Byte 1
*           <0 = Error:  -AVIRECST
*****************************************************************************/
CP_FRCVBUF   cpvid_Reset ( )
{
  CP_CMD snd_buf;

  snd_buf.count = sizeof (CP_CMD);
  snd_buf.id    = ID1;
  snd_buf.dest  = VIDEO_DEST;
  snd_buf.cmd   = VID_RESET_CMD;

  cp_snd ( &snd_buf );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpvid_Control
*
*  Command Used: Video Control (0x01)
*
*  Description: Allows both CONTROL and SOURCE bytes to be specified
*
*  Input Parms :
*       Control : The bit combination of BW, Transparency & video desired
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
typedef struct _VID_CONTROL {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          control;            /* Color Index Format                    */
  BYTE          source;
} VID_CONTROL;

CP_FRCVBUF   cpvid_Control (BYTE Control)
{
  VID_CONTROL vid_control;

                        /* DON'T include source byte in structure */
  vid_control.count   = sizeof (VID_CONTROL) - 1;
  vid_control.id      = ID1;
  vid_control.dest    = VIDEO_DEST;
  vid_control.cmd     = VID_CONTROL_CMD;
  vid_control.control = Control;

  cp_snd ( &vid_control );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpvid_Source
*
*  Command Used: Video Control (0x01)
*
*  Input Parms :
*       Source  : The bit combination of sources desired enabled/disabled
*                 See Prog Guide pp 15-9
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF   cpvid_Source (BYTE Source)
{
  VID_CONTROL vid_source;
  PREAD_VIDEO_STATUS state;

  // Get the short-version of the status info
  state = cpvid_ReadStatus (FALSE);

  vid_source.count   = sizeof (VID_CONTROL);
  vid_source.id      = ID1;
  vid_source.dest    = VIDEO_DEST;
  vid_source.cmd     = VID_CONTROL_CMD;
  vid_source.control = state->state_data.control1;
  vid_source.source  = Source;

  cp_snd ( &vid_source );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}

/*****************************************************************************
*  Function: cpvid_VideoOnly
*
*  Command Used: Video Control (0x01)
*
*  Input Parms :
*       VidOnly : TRUE if ONLY video is desired - no graphics
*                 FALSE if graphics and video is desired
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF   cpvid_VideoOnly (BOOL VidOnly)
{
  CP_CMD_N_DATA1 video_only;
  PREAD_VIDEO_STATUS state;

  // Get the short-version of the status info
  state = cpvid_ReadStatus (FALSE);

  video_only.count     = sizeof (CP_CMD_N_DATA1);
  video_only.id        = ID1;
  video_only.dest      = VIDEO_DEST;
  video_only.cmd       = VID_CONTROL_CMD;

  // Effect ONLY the video/graphics bit in the control byte
  if (VidOnly)
     video_only.cmd_byte1 = state->state_data.control1 | 0x08;
  else
     video_only.cmd_byte1 = state->state_data.control1 & 0xF7;

  cp_snd ( &video_only );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpvid_Transparency
*
*  Command Used: Video Control (0x01)
*
*  Input Parms :
*       TranspOn : TRUE if transparency is desired
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF   cpvid_Transparency (BOOL TranspOn)
{
  CP_CMD_N_DATA1 vid_transp;
  PREAD_VIDEO_STATUS state;

  // Get the short-version of the status info
  state = cpvid_ReadStatus (FALSE);

  vid_transp.count     = sizeof (CP_CMD_N_DATA1);
  vid_transp.id        = ID1;
  vid_transp.dest      = VIDEO_DEST;
  vid_transp.cmd       = VID_CONTROL_CMD;

  // Effect ONLY the transparency bit in the control byte
  if (TranspOn)
     vid_transp.cmd_byte1 = state->state_data.control1 | 0x10;
  else
     // Turn off BOTH transparency and Video Only bits
     vid_transp.cmd_byte1 = state->state_data.control1 & 0xE8;

  cp_snd ( &vid_transp );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpvid_BlackWhite
*
*  Command Used: Video Control (0x01)
*
*  Input Parms :
*       BlackWhiteON : TRUE if black/white mode is desired
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_BlackWhite    (BOOL BlackWhiteON)
{
  CP_CMD_N_DATA1 vid_bw;
  PREAD_VIDEO_STATUS state;

  // Get the short-version of the status info
  state = cpvid_ReadStatus (FALSE);

  vid_bw.count     = sizeof (CP_CMD_N_DATA1);
  vid_bw.id        = ID1;
  vid_bw.dest      = VIDEO_DEST;
  vid_bw.cmd       = VID_CONTROL_CMD;

  // Effect ONLY the black-white bit in the control byte
  if (BlackWhiteON)
     vid_bw.cmd_byte1 = state->state_data.control1 | 0x02;
  else
     vid_bw.cmd_byte1 = state->state_data.control1 & 0xFD;

  cp_snd ( &vid_bw );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpvid_ReadStatus
*
*  Command Used: Read Video Status (0x02)
*
*  Input Parms :
*       extended_flag : TRUE if the extended video status data is desired
*
*  Returns:
*       ptr to video status buffer
*       Caller must check Response Byte 1 for error
*****************************************************************************/
PREAD_VIDEO_STATUS cpvid_ReadStatus (BOOL extended_flag)
{
  CP_CMD_N_DATA1 vid_readstatus;

  vid_readstatus.id    = ID1;
  vid_readstatus.dest  = VIDEO_DEST;
  vid_readstatus.cmd   = VID_READ_STATUS_CMD;

  if ( extended_flag )      /* If Extended info requested                  */
     {
     vid_readstatus.count     = sizeof (CP_CMD_N_DATA1);
     vid_readstatus.cmd_byte1 = 0x01;
     }
  else                        /* No Extended info requested                  */
     vid_readstatus.count = sizeof (CP_CMD);

  cp_snd ( &vid_readstatus );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( (PREAD_VIDEO_STATUS)&rcvbuf.data[0] );
}


/*****************************************************************************
*  Function: cpvid_TranspColor
*
*  Command Used: Transparent Color (0x03)
*
*  Input Parms :
*       All256   : TRUE if all 256 colors are to be transparent
*                : FALSE if only a single color is to be transparent
*       ColorNum : single color to be used, if applicable
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
typedef struct _VID_TRANSPCOL {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          All256_flag;         /* Color Index Format                    */
  BYTE          colornum;
  BYTE          reserved;
} VID_TRANSPCOL;

CP_FRCVBUF  cpvid_TranspColor (BYTE All256, BYTE ColorNum)
{
  VID_TRANSPCOL vid_transpcolor;

  vid_transpcolor.count       = sizeof (VID_TRANSPCOL);
  vid_transpcolor.id          = ID1;
  vid_transpcolor.dest        = VIDEO_DEST;
  vid_transpcolor.cmd         = VID_TRANS_COLOR_CMD;

  /* Control bit 1 = 1 if ALL 256 colors are to be transparent               */
  vid_transpcolor.All256_flag = (BYTE)(All256? 0x01 : 0x00);
  vid_transpcolor.colornum    = ColorNum;
  vid_transpcolor.reserved    = 0;


  cp_snd ( &vid_transpcolor );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_RestoreStatus
*
*  Command Used: Restore Video Status (0x04)
*
*  Input Parms :
*       status_data : a pointer to the video status data structure to be used
*                     Video status will be set to values in this structure
*                     See Prog Guide pp 15-22 to 15-27
*  Returns:
*       ptr to response buffer
*****************************************************************************/
typedef struct _VID_RESTORE_STATUS {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          status_data[36];
} VID_RESTORE_STATUS;

CP_FRCVBUF  cpvid_RestoreStatus (PVIDEO_STATE StatusData)
{
  VID_RESTORE_STATUS vid_restorestatus;


  vid_restorestatus.count = sizeof (VID_RESTORE_STATUS);
  vid_restorestatus.id    = ID1;
  vid_restorestatus.dest  = VIDEO_DEST;
  vid_restorestatus.cmd   = VID_RESTORE_STATUS_CMD;

  /* Control bit 1 = 1 if ALL 256 colors are to be transparent               */
  memcpy ( &vid_restorestatus.status_data[0], StatusData, 36 );

  cp_snd ( &vid_restorestatus );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_ReadImageParms
*
*  Command Used: Read Image Parms (0x05)
*
*  Input Parms:  NONE
*
*  Returns:
*       ImageParmsStruct : Ptr to structure to hold the Image Parmameter data
*                          See Prog Guide pp 15-29 and 15-30
*
*****************************************************************************/
PVID_IMAGE_PARMS cpvid_ReadImageParms ()
{
  CP_CMD readimageparms;

  readimageparms.count = sizeof (CP_CMD);
  readimageparms.id    = 0x01;
  readimageparms.dest  = 0x02;
  readimageparms.cmd   = VID_READ_IMG_PARMS_CMD;

  cp_snd ( &readimageparms );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ((PVID_IMAGE_PARMS) &rcvbuf.data[0]);
}

/*****************************************************************************
*  Function: cpvid_ConfigInputs
*
*  Command Used: Configure Video Inputs (0x06)
*
*  Input Parms :
*       The TYPE and AUDIO STATUS selections for each of the 3 video
*       sources.  These are described in detail in the Prog Guide, pp 15-33,
*       15-34.
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
typedef struct _VID_CNFG_INPUTS {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          V1type;
  BYTE          V1audio;
  BYTE          V2type;
  BYTE          V2audio;
  BYTE          V3type;
  BYTE          V3audio;
} VID_CNFG_INPUTS;

CP_FRCVBUF cpvid_ConfigInputs  (BYTE V1Type, BYTE V1Audio,
                                BYTE V2Type, BYTE V2Audio,
                                BYTE V3Type, BYTE V3Audio)
{
  VID_CNFG_INPUTS cnfg_inputs;

  cnfg_inputs.count   = (BYTE)sizeof (VID_CNFG_INPUTS);
  cnfg_inputs.id      = 0x01;
  cnfg_inputs.dest    = VIDEO_DEST;
  cnfg_inputs.cmd     = VID_CONFIG_INPUTS_CMD;
  cnfg_inputs.V1type  = V1Type;
  cnfg_inputs.V1audio = V1Audio;
  cnfg_inputs.V2type  = V2Type;
  cnfg_inputs.V2audio = V2Audio;
  cnfg_inputs.V3type  = V3Type;
  cnfg_inputs.V3audio = V3Audio;

  cp_snd (&cnfg_inputs);
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  /* return the response */
  return ( &rcvbuf );
}



/*****************************************************************************
*  Function: cpvid_SetCoordinates
*
*  Command Used: Set Video Coordinates (0x07)
*
*  Input Parms :
*       The rectangular regions that video is to be obtain from and shown at.
*
*       All parameters are described in the Prog Guide, pp 15-39 & 15-40
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
typedef struct _VID_COORD {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  SHORT         SourceHorg;
  SHORT         SourceVorg;
  USHORT        SourceHlen;
  USHORT        SourceVlen;
  SHORT         DestHorg;
  SHORT         DestVorg;
  USHORT        DestHlen;
  USHORT        DestVlen;
} VID_COORD;

CP_FRCVBUF  cpvid_SetCoordinates  (SHORT  SourceHorg, SHORT  SourceVorg,
                                   USHORT SourceHlen, USHORT SourceVlen,
                                   SHORT  DestHorg,   SHORT  DestVorg,
                                   USHORT DestHlen,   USHORT DestVlen)
{
  VID_COORD vid_coords;


  vid_coords.count      = sizeof (VID_COORD);
  vid_coords.id         = ID1;
  vid_coords.dest       = VIDEO_DEST;
  vid_coords.cmd        = VID_SET_COORDINATES_CMD;
  vid_coords.SourceHorg = SourceHorg;
  vid_coords.SourceVorg = SourceVorg;
  vid_coords.SourceHlen = SourceHlen;
  vid_coords.SourceVlen = SourceVlen;
  vid_coords.DestHorg   = DestHorg;
  vid_coords.DestVorg   = DestVorg;
  vid_coords.DestHlen   = DestHlen;
  vid_coords.DestVlen   = DestVlen;

  cp_snd ( &vid_coords );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_SetFullscreenCoord
*
*  Command Used: Set Video Coordinates (0x07)
*
*  Description:
*       Sets video coordinate to the encompass the entire screen.
*
*  Input Parms : NONE
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_SetFullscreenCoord ( )
{
  CP_CMD vid_coords;

  vid_coords.count = sizeof (CP_CMD);
  vid_coords.id    = ID1;
  vid_coords.dest  = VIDEO_DEST;
  vid_coords.cmd   = VID_SET_COORDINATES_CMD;

  cp_snd ( &vid_coords );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_VideoProtect
*
*  Command Used: Define Video Protect (0x08)
*
*  Parms :
*       VidProtectFlag = TRUE if video protect is to be turned ON
*       Rectangular region to be affected.  Described in Prog Guide
*       pp 15-42, 15-43.
*****************************************************************************/
typedef struct _VID_PROTECT {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          flag;
  SHORT         DestHorg;
  SHORT         DestVorg;
  USHORT        DestHlen;
  USHORT        DestVlen;
} VID_PROTECT;

CP_FRCVBUF  cpvid_VideoProtect  (BOOL VidProtectFlag,
                            SHORT  DestHorg, SHORT  DestVorg,
                            USHORT DestHlen, USHORT DestVlen)
{
  VID_PROTECT vid_vprot;


  vid_vprot.count    = sizeof (VID_PROTECT);
  vid_vprot.id       = ID1;
  vid_vprot.dest     = VIDEO_DEST;
  vid_vprot.cmd      = VID_VIDEO_PROTECT_CMD;
  vid_vprot.flag     = (BYTE)( VidProtectFlag ? 0x02 : 0x01);
  vid_vprot.DestHorg = DestHorg;
  vid_vprot.DestVorg = DestVorg;
  vid_vprot.DestHlen = DestHlen;
  vid_vprot.DestVlen = DestVlen;

  cp_snd ( &vid_vprot );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_VideoProtectAll
*
*  Command Used: Define Video Protect (0x08)
*
*  Input Parms :
*       VidProtectFlag : TRUE if ENTIRE screen is to have video protect
*                        FALSE if video protect is to be OFF for entire screen
*****************************************************************************/
CP_FRCVBUF  cpvid_VideoProtectAll (BOOL VidProtectFlag)
{
  CP_CMD_N_DATA1 vid_vprot;

  vid_vprot.count     = sizeof (CP_CMD_N_DATA1);
  vid_vprot.id        = ID1;
  vid_vprot.dest      = VIDEO_DEST;
  vid_vprot.cmd       = VID_VIDEO_PROTECT_CMD;
  vid_vprot.cmd_byte1 = (BYTE)( VidProtectFlag ? 0x02 : 0x01);

  cp_snd ( &vid_vprot );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_GraphicProtect
*
*  Command Used: Define Graphic Protect (0x09)
*
*  Input Parms:
*       GrafProtectFlag = TRUE if graphic protect is to be turned ON.
*       Rectangular region to be affected.  Described in Prog Guide
*       pp 15-45, 15-46.
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_GraphicProtect (BOOL GrafProtectFlag,
                                  SHORT  DestHorg, SHORT  DestVorg,
                                  USHORT DestHlen, USHORT DestVlen)
{
  VID_PROTECT vid_gprot;


  vid_gprot.count    = sizeof (VID_PROTECT);
  vid_gprot.id       = ID1;
  vid_gprot.dest     = VIDEO_DEST;
  vid_gprot.cmd      = VID_GRAPHIC_PROTECT_CMD;
  vid_gprot.flag     = (BYTE)( GrafProtectFlag ? 0x10 : 0x08);
  vid_gprot.DestHorg = DestHorg;
  vid_gprot.DestVorg = DestVorg;
  vid_gprot.DestHlen = DestHlen;
  vid_gprot.DestVlen = DestVlen;

  cp_snd ( &vid_gprot );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_GraphicProtectAll
*
*  Command Used: Define Graphic Protect (0x09)
*
*  Input Parms:
*       GrafProtectFlag : TRUE if ENTIRE screen is to have graphic protect
*                         FALSE for graphic protect to be OFF for entire screen
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_GraphicProtectAll (BOOL GrafProtectFlag)
{
  CP_CMD_N_DATA1 vid_gprot;

  vid_gprot.count     = sizeof (CP_CMD_N_DATA1);
  vid_gprot.id        = ID1;
  vid_gprot.dest      = VIDEO_DEST;
  vid_gprot.cmd       = VID_GRAPHIC_PROTECT_CMD;
  vid_gprot.cmd_byte1 = (BYTE)( GrafProtectFlag ? 0x10 : 0x08);

  cp_snd ( &vid_gprot );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_OptimizeVid
*
*  Command Used: Optimize Video (0x0A)
*
*  Description:
*       Permits setting of the scan method and video algorithm to be used.
*
*  Input Parms:
*       ScanMethod    : Both are described in Prog Guide pp 15-48, 15-49
*       Algorithm
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
typedef struct _VID_OPTIMIZE {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          scan_method;
  BYTE          algorithm;
} VID_OPTIMIZE;

CP_FRCVBUF  cpvid_OptimizeVid  (BYTE ScanMethod, BYTE Algorithm)
{
  VID_OPTIMIZE vid_optimize;

  /* Scanning Method :                                                       */
  /*    0x01 = Progressive                                                   */
  /*    0x02 = Scan Line Doubling                                            */
  /*    0x04 = Single Field                                                  */
  /* Algorithm :                                                             */
  /*    0x01 = A                                                             */
  /*    0x02 = B                                                             */
  /*    0x04 = C                                                             */

  vid_optimize.count       = sizeof (VID_OPTIMIZE);
  vid_optimize.id          = ID1;
  vid_optimize.dest        = VIDEO_DEST;
  vid_optimize.cmd         = VID_OPTIMIZE_CMD;
  vid_optimize.scan_method = ScanMethod;
  vid_optimize.algorithm   = Algorithm;

  cp_snd ( &vid_optimize );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}

/*****************************************************************************
*  Function: cpvid_Brightness
*
*  Command Used: Adjust Brightness (0x0B)
*
*  Input Parms:
*       MakeDefault : TRUE if this level is to be the new default
*       Level       : New value for this attribute
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_Brightness (BOOL MakeDefault, BYTE Level)
{
  CP_LEVELS vid_bright;

  vid_bright.count        = sizeof (CP_LEVELS);
  vid_bright.id           = ID1;
  vid_bright.dest         = VIDEO_DEST;
  vid_bright.cmd          = VID_BRIGHTNESS_CMD;
  vid_bright.default_flag = (BYTE)( MakeDefault ? 0x01 : 0x00);
  vid_bright.level        = Level;

  cp_snd ( &vid_bright );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}

/*****************************************************************************
*  Function: cpvid_DefBrightness
*
*  Command Used: Adjust Brightness (0x0B)
*
*  Input Parms: NONE
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_DefBrightness ()
{
  CP_CMD vid_bright;

  vid_bright.count        = sizeof (CP_CMD);
  vid_bright.id           = ID1;
  vid_bright.dest         = VIDEO_DEST;
  vid_bright.cmd          = VID_BRIGHTNESS_CMD;

  cp_snd ( &vid_bright );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_Contrast
*
*  Command Used: Adjust Contrast (0x0C)
*
*  Input Parms:
*       MakeDefault : TRUE if this level is to be the new default
*       Level       : New value for this attribute
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_Contrast (BOOL MakeDefault, BYTE Level)
{
  CP_LEVELS vid_contrast;


  vid_contrast.count        = sizeof (CP_LEVELS);
  vid_contrast.id           = ID1;
  vid_contrast.dest         = VIDEO_DEST;
  vid_contrast.cmd          = VID_CONTRAST_CMD;
  vid_contrast.default_flag = (BYTE)( MakeDefault ? 0x01 : 0x00);
  vid_contrast.level        = Level;

  cp_snd ( &vid_contrast );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}

/*****************************************************************************
*  Function: cpvid_DefContrast
*
*  Command Used: Adjust Contrast (0x0C)
*
*  Input Parms: NONE
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_DefContrast ()
{
  CP_CMD vid_contrast;

  vid_contrast.count        = sizeof (CP_CMD);
  vid_contrast.id           = ID1;
  vid_contrast.dest         = VIDEO_DEST;
  vid_contrast.cmd          = VID_CONTRAST_CMD;

  cp_snd ( &vid_contrast );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_Color
*
*  Command Used: Adjust Color (0x0D)
*
*  Input Parms:
*       MakeDefault : TRUE if this level is to be the new default
*       Level       : New value for this attribute
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_Color (BOOL MakeDefault, BYTE Level)
{
  CP_LEVELS vid_color;


  vid_color.count        = sizeof (CP_LEVELS);
  vid_color.id           = ID1;
  vid_color.dest         = VIDEO_DEST;
  vid_color.cmd          = VID_COLOR_CMD;
  vid_color.default_flag = (BYTE)( MakeDefault ? 0x01 : 0x00);
  vid_color.level        = Level;

  cp_snd ( &vid_color );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpvid_DefColor
*
*  Command Used: Adjust Color (0x0D)
*
*  Input Parms: NONE
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_DefColor ()
{
  CP_CMD vid_color;

  vid_color.count        = sizeof (CP_CMD);
  vid_color.id           = ID1;
  vid_color.dest         = VIDEO_DEST;
  vid_color.cmd          = VID_COLOR_CMD;

  cp_snd ( &vid_color );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}

/*****************************************************************************
*  Function: cpvid_Tint
*
*  Command Used: Adjust Tint (0x0E)
*
*  Input Parms:
*       MakeDefault : TRUE if this level is to be the new default
*       Level       : New value for this attribute
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_Tint (BOOL MakeDefault, char Level)
{
  CP_LEVELS vid_tint;


  vid_tint.count        = sizeof (CP_LEVELS);
  vid_tint.id           = ID1;
  vid_tint.dest         = VIDEO_DEST;
  vid_tint.cmd          = VID_TINT_CMD;
  vid_tint.default_flag = (BYTE)( MakeDefault ? 0x01 : 0x00);
  vid_tint.level        = Level;   /* -100 to +100 (Blue to Green)         */

  cp_snd ( &vid_tint );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}

/*****************************************************************************
*  Function: cpvid_DefTint
*
*  Command Used: Adjust Tint (0x0E)
*
*  Input Parms: NONE
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_DefTint ()
{
  CP_CMD vid_tint;

  vid_tint.count        = sizeof (CP_CMD);
  vid_tint.id           = ID1;
  vid_tint.dest         = VIDEO_DEST;
  vid_tint.cmd          = VID_TINT_CMD;

  cp_snd ( &vid_tint );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}

/*****************************************************************************
*  Function: cpvid_Fade
*
*  Command Used: Adjust Video Fade (0x0F)
*
*  Description:
*       Performs a video fade to a precentage of the current brightness.
*       This does NOT effect the current brightness SETTING however.
*       If a subsequent FADE is performed up to 100%, then the current
*          brightness is restored.
*
*  Input Parms:
*       Level        : Fade level to proceed TO, 0-100%
*       TimeInTenths : Number of tenths of seconds over which to perform fade
*
*  Returns:
*     VOID
*     This function does not wait for the Fade to complete before returning.
*****************************************************************************/
typedef struct _VID_FADE {
  USHORT          count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          level;
  BYTE          time_tenths;
} VID_FADE;

void  cpvid_Fade (BYTE Level, BYTE TimeInTenths)
{
  VID_FADE vid_fade;


  vid_fade.count       = sizeof (VID_FADE);
  vid_fade.id          = VID_FADE_ID;
  vid_fade.dest        = VIDEO_DEST;
  vid_fade.cmd         = VID_FADE_CMD;
  vid_fade.level       = Level;
  vid_fade.time_tenths = TimeInTenths;

  in_video_fade = TRUE;
  cp_snd ( &vid_fade );

  return;
}

/*****************************************************************************
*  Function: cpvid_DefFade
*
*  Command Used: Adjust Fade (0x0F)
*
*  Input Parms: NONE
*
*  Returns:
*       VOID
*****************************************************************************/
void  cpvid_DefFade ()
{
  CP_CMD vid_fade;

  vid_fade.count        = sizeof (CP_CMD);
  vid_fade.id           = VID_FADE_ID;
  vid_fade.dest         = VIDEO_DEST;
  vid_fade.cmd          = VID_FADE_CMD;

  in_video_fade = TRUE;
  cp_snd ( &vid_fade );

  return;
}


/*****************************************************************************
*  Function: cpvid_Sharpness
*
*  Command Used: Adjust Sharpness (0x10)
*
*  Input Parms:
*       MakeDefault : TRUE if this level is to be the new default
*       Level       : New value for this attribute
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_Sharpness (BOOL MakeDefault, BYTE Level)
{
  CP_LEVELS vid_sharpness;


  vid_sharpness.count        = sizeof (CP_LEVELS);
  vid_sharpness.id           = ID1;
  vid_sharpness.dest         = VIDEO_DEST;
  vid_sharpness.cmd          = VID_SHARPNESS_CMD;
  vid_sharpness.default_flag = (BYTE)( MakeDefault ? 0x01 : 0x00);
  vid_sharpness.level        = Level;

  cp_snd ( &vid_sharpness );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}

/*****************************************************************************
*  Function: cpvid_DefSharpness
*
*  Command Used: Adjust Sharpness (0x10)
*
*  Input Parms: NONE
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_DefSharpness ()
{
  CP_CMD vid_sharp;

  vid_sharp.count        = sizeof (CP_CMD);
  vid_sharp.id           = ID1;
  vid_sharp.dest         = VIDEO_DEST;
  vid_sharp.cmd          = VID_SHARPNESS_CMD;

  cp_snd ( &vid_sharp );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_Freeze
*
*  Command Used: Freeze Video (0x11)
*
*  Input Parms:
*       FreezeOn : TRUE if video freeze is to be turned ON.
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpvid_Freeze (BOOL FreezeOn)
{
  CP_CMD_N_DATA1 vid_freeze;

  vid_freeze.count     = sizeof (CP_CMD_N_DATA1);
  vid_freeze.id        = ID1;
  vid_freeze.dest      = VIDEO_DEST;
  vid_freeze.cmd       = VID_FREEZE_CMD;
  vid_freeze.cmd_byte1 = (BYTE)( FreezeOn ? 0x01 : 0x00);

  cp_snd ( &vid_freeze );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_Capture
*
*  Command Used: Capture Video (0x12)
*
*  Description:
*       Captures the current video image, from the area specified, into
*       the file handle provided.
*
*  Input Parms:
*       Control data : video settings
*       Rectangular source region to be captured
*       Buffer       : buffer to be used.
*       Filehandle, Fileoffset : handle and location in file to place the
*                                image data
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
#define BSIZE   (size_t) 65000

typedef struct _VID_CAPTURE {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          control;
  USHORT        SourceHorg;
  USHORT        SourceVorg;
  USHORT        SourceHlen;
  USHORT        SourceVlen;
  PBYTE         buffer;
  ULONG         bufferlen;
  HFILE         filehandle;
  ULONG         fileoffset;
} VID_CAPTURE;

CP_FRCVBUF cpvid_Capture (BYTE Control,
                     USHORT SourceHorg, USHORT SourceVorg,
                     USHORT SourceHlen, USHORT SourceVlen,
                     PBYTE Buffer, ULONG Bufferlen,
                     HFILE Filehandle, ULONG Fileoffset)
{
  VID_CAPTURE cap;

  cap.count      = sizeof (VID_CAPTURE);
  cap.id         = 0x01;
  cap.dest       = 0x02;
  cap.cmd        = VID_CAPTURE_CMD;
  cap.control    = Control;
  cap.SourceHorg = SourceHorg;
  cap.SourceVorg = SourceVorg;
  cap.SourceHlen = SourceHlen;
  cap.SourceVlen = SourceVlen;
  cap.buffer     = Buffer;
  cap.bufferlen  = Bufferlen;
  cap.filehandle = Filehandle;
  cap.fileoffset = Fileoffset;

  cp_snd ( &cap );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpvid_CaptureBuffer
*
*  Command Used: Capture Video (0x12)
*
*  Description:
*       Captures the current video image, from the area specified, into
*       a user-defined buffer (Does not send file handle bytes).
*       User must allocate memory for the buffer!
*
*  Input Parms:
*       Control data : video settings
*       Rectangular source region to be captured
*       Buffer       : buffer to be used.
*
*  Returns:
*       ptr to response buffer
*****************************************************************************/
typedef struct _VID_CAPTURE_BUF {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          control;
  USHORT        SourceHorg;
  USHORT        SourceVorg;
  USHORT        SourceHlen;
  USHORT        SourceVlen;
  PBYTE         buffer;
  ULONG         bufferlen;
} VID_CAPTURE_BUF;

CP_FRCVBUF cpvid_CaptureBuffer (BYTE Control,
                                USHORT SourceHorg, USHORT SourceVorg,
                                USHORT SourceHlen, USHORT SourceVlen,
                                PBYTE Buffer, ULONG Bufferlen)
{
  VID_CAPTURE_BUF cap;

  cap.count      = sizeof (VID_CAPTURE_BUF);
  cap.id         = 0x01;
  cap.dest       = 0x02;
  cap.cmd        = VID_CAPTURE_CMD;
  cap.control    = Control;
  cap.SourceHorg = SourceHorg;
  cap.SourceVorg = SourceVorg;
  cap.SourceHlen = SourceHlen;
  cap.SourceVlen = SourceVlen;
  cap.buffer     = Buffer;
  cap.bufferlen  = Bufferlen;

  cp_snd ( &cap );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpvid_CaptureFile
*
*  Command Used: NONE - utility function
*
*  Description:
*     Captures a video image to a file by calling cpvid_Capture().
*
*  Input Parms:
*       Filename : ascii string holding the NAME of the file to
*                  hold the captured image data.
*
*  Returns:
*       FALSE (0) if open error
*       ptr to response buffer in all other cases
*****************************************************************************/
#define BSIZE   (size_t) 65000

CP_FRCVBUF cpvid_CaptureFile (PBYTE Filename)

{
  // capture the whole video screen into the given filename

  PBYTE   buffer;
  ULONG   bufferlen;
  HFILE   filehandle;
  PVID_IMAGE_PARMS vidata;

  // open up the file for writing
  filehandle = open ( Filename, O_BINARY | O_RDWR | O_CREAT | O_TRUNC,
                      S_IWRITE | S_IREAD  );
  if ( filehandle == -1 ) {
    return ( FALSE );
    }

  // allocate a buffer to speed up the restore
  if ( buffer = malloc ( BSIZE ) ) {
    bufferlen = (ULONG)BSIZE;
    }
  else {
    bufferlen = 0;
    }

  vidata = cpvid_ReadImageParms ();

  return (cpvid_Capture ( 0,
               0 - vidata->LBorder, 0 - vidata->TBorder,
               vidata->LBorder + vidata->ShownBufHlen + vidata->RBorder,
               vidata->TBorder + vidata->ShownBufVlen + vidata->BBorder,
               buffer, bufferlen,
               filehandle, 0L ));
}


/*****************************************************************************
*  Function: cpvid_Restore
*
*  Command Used: Restore Video (0x13)
*
*  Input Parms:
*       Control data           : video settings
*       Rectangular region to be restored
*       Buffer                 : buffer to be used.
*       DataHorg, DataVord     : Location on screen to place the video image
*       Filehandle, Fileoffset : handle and location in file to place the
*                                image data
*
*  Returns:
*       Ptr to video response buffer
*****************************************************************************/
typedef struct _VID_RESTORE {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          control;
  SHORT         DestHorg;
  SHORT         DestVorg;
  USHORT        DestHlen;
  USHORT        DestVlen;
  PBYTE         Vbuffer;
  ULONG         Vbufferlen;
  SHORT         DataHorg;
  SHORT         DataVorg;
  SHORT         file_handle;
  ULONG         file_offset;
} VID_RESTORE;


CP_FRCVBUF  cpvid_Restore (BYTE    control,
                      SHORT   DestHorg,    SHORT DestVorg,
                      USHORT  DestHlen,    USHORT DestVlen,
                      PBYTE   Vbuffer,     ULONG Vbufferlen,
                      SHORT   DataHorg,    SHORT DataVorg,
                      SHORT   Filehandle,  ULONG Fileoffset)
{
  VID_RESTORE vid_restore;


  vid_restore.count       = sizeof (VID_RESTORE);
  vid_restore.id          = ID1;
  vid_restore.dest        = VIDEO_DEST;
  vid_restore.cmd         = VID_RESTORE_CMD;
  vid_restore.control     = control;
  vid_restore.DestHorg    = DestHorg;
  vid_restore.DestVorg    = DestVorg;
  vid_restore.DestHlen    = DestHlen;
  vid_restore.DestVlen    = DestVlen;
  vid_restore.Vbuffer     = Vbuffer;
  vid_restore.Vbufferlen  = Vbufferlen;
  vid_restore.DataHorg    = DataHorg;
  vid_restore.DataVorg    = DataVorg;
  vid_restore.file_handle = Filehandle;
  vid_restore.file_offset = Fileoffset;

  cp_snd ( &vid_restore );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_RestoreBuffer
*
*  Command Used: Restore Video (0x13)
*
*  Description:
*       Restores the video image from the user-defined buffer
*       (Does not send file handle bytes).
*       User must allocate memory for the buffer!
*
*  Input Parms:
*       Control data           : video settings
*       Rectangular region to be restored
*       Buffer                 : buffer to be used.
*       DataHorg, DataVord     : Location on screen to place the video image
*
*  Returns:
*       Ptr to video response buffer
*****************************************************************************/
typedef struct _VID_RESTORE_BUF {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          control;
  SHORT         DestHorg;
  SHORT         DestVorg;
  USHORT        DestHlen;
  USHORT        DestVlen;
  PBYTE         Vbuffer;
  ULONG         Vbufferlen;
  SHORT         DataHorg;
  SHORT         DataVorg;
} VID_RESTORE_BUF;


CP_FRCVBUF  cpvid_RestoreBuffer  (BYTE    control,
                                  SHORT   DestHorg,    SHORT DestVorg,
                                  USHORT  DestHlen,    USHORT DestVlen,
                                  PBYTE   Vbuffer,     ULONG Vbufferlen,
                                  SHORT   DataHorg,    SHORT DataVorg)
{
  VID_RESTORE_BUF vid_restore;


  vid_restore.count       = sizeof (VID_RESTORE_BUF);
  vid_restore.id          = ID1;
  vid_restore.dest        = VIDEO_DEST;
  vid_restore.cmd         = VID_RESTORE_CMD;
  vid_restore.control     = control;
  vid_restore.DestHorg    = DestHorg;
  vid_restore.DestVorg    = DestVorg;
  vid_restore.DestHlen    = DestHlen;
  vid_restore.DestVlen    = DestVlen;
  vid_restore.Vbuffer     = Vbuffer;
  vid_restore.Vbufferlen  = Vbufferlen;
  vid_restore.DataHorg    = DataHorg;
  vid_restore.DataVorg    = DataVorg;

  cp_snd ( &vid_restore );

  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}
/*****************************************************************************
*  Function: cpvid_RestoreFile
*
*  Command Used: NONE - Utility function
*
*  Description:
*     Restores a video image from a file by calling cpvid_Restore ().
*
*  Input Parms:
*       Filename : ascii string containing name of file to be displayed
*
*  Returns:
*       FALSE (0) if a file open error
*       ptr to response buffer otherwise
*****************************************************************************/
typedef struct _VID_HEADER {
  char          id[8];     /* "YUV12C"  */
  USHORT        x;
  USHORT        y;
  USHORT        xlen;
  USHORT        ylen;
} VID_HEADER;

#define BSIZE   (size_t) 65000

CP_FRCVBUF  cpvid_RestoreFile ( PBYTE Filename )
{
  PBYTE   buffer;
  ULONG   bufferlen;
  SHORT     filehandle;
  VID_HEADER vheader;
  CP_FRCVBUF rspbuf_ptr;

  /* open up the file for reading */
  filehandle = open ( Filename, O_BINARY | O_RDONLY );
  if ( filehandle == -1 )
     return ( FALSE );


  /* read the VID_HEADER in */
  read ( filehandle, (PBYTE)&vheader, sizeof ( vheader ) );

  /* allocate a buffer to speed up the restore  */
  if ( buffer = malloc ( BSIZE ) )
     bufferlen = (ULONG)BSIZE;
  else
     bufferlen = 0;

  /* Turn on Graphic Protect, off Video Protect */
  rspbuf_ptr = cpvid_Restore (0x09, vheader.x, vheader.y,
                              vheader.xlen, vheader.ylen,
                              buffer, bufferlen,
                              0, 0, filehandle, 0L );

  /* if the buffer request did not fail, free it */
  if ( buffer )
     free ( buffer );

  close ( filehandle );

  return (rspbuf_ptr);
}


/*****************************************************************************
*  Function: cpvid_Copy
*
*  Command Used: Copy Video Region (0x14)
*
*  Input Parms:
*       Control : video settings
*       Source region to copy from
*       Destination location to copy to
*
*  Returns:
*       Ptr to video response buffer
*****************************************************************************/
typedef struct _VID_COPY {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          control;
  USHORT        SourceHorg;
  USHORT        SourceVorg;
  USHORT        SourceHlen;
  USHORT        SourceVlen;
  USHORT        DestHorg;
  USHORT        DestVorg;
} VID_COPY;

CP_FRCVBUF  cpvid_Copy  (BYTE control,
                    SHORT  SourceHorg, SHORT  SourceVorg,
                    USHORT SourceHlen, USHORT SourceVlen,
                    SHORT  DestHorg,   SHORT  DestVorg)
{
  VID_COPY vid_copy;

  vid_copy.count      = sizeof (VID_COPY);
  vid_copy.id         = ID1;
  vid_copy.dest       = VIDEO_DEST;
  vid_copy.cmd        = VID_COPY_CMD;
  vid_copy.control    = control;
  vid_copy.SourceHorg = SourceHorg;
  vid_copy.SourceVorg = SourceVorg;
  vid_copy.SourceHlen = SourceHlen;
  vid_copy.SourceVlen = SourceVlen;
  vid_copy.DestHorg   = DestHorg;
  vid_copy.DestVorg   = DestVorg;

  cp_snd ( &vid_copy );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************
*  Function: cpvid_Clear
*
*  Command Used: Clear Video (0x15)
*
*  Description :
*       This command can be used to perform a gradual change, or "spread"
*  of colors by having color set 1 NOT EQUAL to color set 2.
*
*  Input Parms:
*       Control     : video settings
*       Dest values : Region to be cleared
*       Color set 1 : Color to begin clearing to
*       Color set 2 : Color to finish clearing to
*
*  Returns:
*       Ptr to video response buffer
*****************************************************************************/
typedef struct _VID_CLEAR {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          control;
  USHORT        DestHorg;
  USHORT        DestVorg;
  USHORT        DestHlen;
  USHORT        DestVlen;
  BYTE          Y_red_1;
  BYTE          U_green_1;
  BYTE          V_blue_1;
  BYTE          Y_red_2;
  BYTE          U_green_2;
  BYTE          V_blue_2;
} VID_CLEAR;

CP_FRCVBUF  cpvid_Clear (BYTE control,
                    SHORT  DestHorg, SHORT  DestVorg,
                    USHORT DestHlen, USHORT DestVlen,
                    BYTE Red_1, BYTE Green_1,  BYTE Blue_1,
                    BYTE Red_2, BYTE Green_2,  BYTE Blue_2)
{
  VID_CLEAR vid_clear;

  vid_clear.count     = sizeof (VID_CLEAR);
  vid_clear.id        = ID1;
  vid_clear.dest      = VIDEO_DEST;
  vid_clear.cmd       = VID_CLEAR_CMD;
  vid_clear.control   = control;
  vid_clear.DestHorg  = DestHorg;
  vid_clear.DestVorg  = DestVorg;
  vid_clear.DestHlen  = DestHlen;
  vid_clear.DestVlen  = DestVlen;
  vid_clear.Y_red_1   = Red_1;
  vid_clear.U_green_1 = Green_1;
  vid_clear.V_blue_1  = Blue_1;
  vid_clear.Y_red_2   = Red_2;
  vid_clear.U_green_2 = Green_2;
  vid_clear.V_blue_2  = Blue_2;

  cp_snd ( &vid_clear );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}



/*****************************************************************************
*  Function: cpvid_Show256Pic
*
*  Command Used: Show 256 Pic (0x16)
*
*  Description:
*       Loads 256 color VGA format data into be video buffer by converting
*       the pels into YUV format.
*
*  Input Parms:
*       control         : video settings
*       color table     : ptr to 768 byte RGB color array (each RGB=0-63)
*       pic_ptr         : ptr to pel data, each pel 0-255, representing
*                         a color table value
*       lengths         : total bytes in buffer and bytes per LINE
*       Dests           : rectangular region to show the picture in
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
typedef struct _SHOW_PIC {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          control;
  PBYTE         color_table;
  PBYTE         pic_ptr;
  ULONG         total_byte_len;
  USHORT        line_len;
  USHORT        DestHorg;
  USHORT        DestVorg;
  USHORT        DestHlen;
  USHORT        DestVlen;
} SHOW_PIC;


CP_FRCVBUF  cpvid_Show256Pic (BYTE  control,  PBYTE color_table, PBYTE pic_ptr,
                              ULONG total_byte_len, USHORT  line_len,
                              SHORT  DestHorg,  SHORT  DestVorg,
                              USHORT DestHlen,  USHORT DestVlen)
{
  SHOW_PIC picdata;

  picdata.count          = sizeof (SHOW_PIC);
  picdata.id             = ID1;
  picdata.dest           = VIDEO_DEST;
  picdata.cmd            = VID_SHOW_256PIC_CMD;
  picdata.control        = control;
  picdata.color_table    = color_table;
  picdata.pic_ptr        = pic_ptr;
  picdata.total_byte_len = total_byte_len;
  picdata.line_len       = line_len;
  picdata.DestHorg       = DestHorg;
  picdata.DestVorg       = DestVorg;
  picdata.DestHlen       = DestHlen;
  picdata.DestVlen       = DestVlen;

  cp_snd ( &picdata );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}

/*****************************************************************************
*  Function: cpvid_ClearAllBlack
*
*  Command Used: NONE - Utility function
*
*  Description:
*     Clears the entire video buffer to BLACK by calling cpvid_Clear ().
*
*  Input Parms: NONE
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF cpvid_ClearAllBlack ( )
{
  return ( cpvid_Clear ( 0x09, -8, -8, 1024, 500, 0, 0, 0, 0, 0, 0) );

}


/*****************************************************************************
*  Function: cpvid_MultiVid
*
*  Command Used: Multi Video (0x17)
*
*  Description:
*       Allows you to show motion video from one or more video sources at
*       different locations on the screen at the same time.
*
*       NOTE: Be sure to issue a 'cpvid_Reset()' before exiting your
*             program to avoid a system lockup.
*
*  Input Parms:
*       entries         : number of multi video array entries ( 1 - 255 )
*       video table     : far ptr to multi video array ( long int format )
*                         ( see page 15-84 in prog. guide for description )
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
typedef struct _MULTI_VID {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          entries;
  PVOID         video_table;
} MULTI_VID;


CP_FRCVBUF  cpvid_MultiVid (BYTE  entries,  PVOID video_table)

{
  MULTI_VID multivid;

  multivid.count          = sizeof (MULTI_VID);
  multivid.id             = ID1;
  multivid.dest           = VIDEO_DEST;
  multivid.cmd            = VID_MULTI_VID_CMD;
  multivid.entries        = entries;
  multivid.video_table    = video_table;

  cp_snd ( &multivid );
  while ( !VIDEO_HAS_MSG );
  cp_rcv ( VIDEO_DEST );

  return ( &rcvbuf );

}


/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
/*              AUDIO FUNCTIONS                                              */
/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/

/*****************************************************************************
*  Function: cpaud_Reset
*
*  Command Used: Audio Reset (0x00)
*
*  Input Parms: NONE
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_Reset ( )
{
  CP_CMD aud_reset;

  aud_reset.count   = (BYTE)sizeof (CP_CMD);
  aud_reset.id      = 0x01;
  aud_reset.dest    = AUDIO_DEST;
  aud_reset.cmd     = AUD_RESET_CMD;

  cp_snd ( &aud_reset);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_Follow
*
*  Command Used: Audio Control (0x01)
*
*  Input Parms:
*       FollowOn = TRUE if the audio is to follow the video
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
typedef struct _AUDIO_CONTROL {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          mute;
  BYTE          inputs;
  BYTE          outputs;
} AUDIO_CONTROL;

CP_FRCVBUF  cpaud_Follow (BOOL FollowOn)
{
  PREAD_AUDIO_STATUS status;
  AUDIO_CONTROL aud_follow;

  status = cpaud_ReadStatus ( );

  aud_follow.count   = (BYTE)sizeof (AUDIO_CONTROL);
  aud_follow.id      = 0x01;
  aud_follow.dest    = AUDIO_DEST;
  aud_follow.cmd     = AUD_CONTROL_CMD;
  aud_follow.mute    = status->state_data.control1;

  if (FollowOn)
     // Turn off high bit
     aud_follow.inputs  = status->state_data.control2 & 0x7F;
  else
     // Turn on high bit
     aud_follow.inputs  = status->state_data.control2 | 0x80;
  aud_follow.outputs = status->state_data.control3;

  cp_snd (&aud_follow);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_Mute
*
*  Command Used: Audio Control (0x01)
*
*  Input Parms:
*       MuteVal : Mute on/off/line only
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_Mute (BYTE MuteVal)
                      /* val = 0   no mute                                   */
                      /* val = 1   mute input and output                     */
                      /* val = 2   mute line input only                      */
{
  AUDIO_CONTROL aud_control;

  // DON'T send the inputs/outputs bytes.  They won't be changed.
  aud_control.count   = (BYTE) sizeof (AUDIO_CONTROL) - 2;
  aud_control.id      = 0x01;
  aud_control.dest    = AUDIO_DEST;
  aud_control.cmd     = AUD_CONTROL_CMD;
  aud_control.mute    = MuteVal;

  cp_snd ( &aud_control);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return (&rcvbuf);
}



/*****************************************************************************
*  Function: cpaud_Inputs
*
*  Command Used: Audio Control (0x01)
*
*  Input Parms:
*       Inputs : various settings.  Described in Prog Guide pp 18-7
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_Inputs (BYTE Inputs)
                        /* OR the following bit values together to create val*/
                        /* 0x80  = Don't follow Video                        */
                        /* 0x40  = Enable Right Channel                      */
                        /* 0x20  = Enable Left Channel                       */
                        /* 0x02  = Enable Audio input 2                      */
                        /* 0x01  = Enable Audio input 1                      */
{
  PREAD_AUDIO_STATUS status;
  AUDIO_CONTROL aud_control;

  status = cpaud_ReadStatus ( );

  aud_control.count   = (BYTE)sizeof (AUDIO_CONTROL);
  aud_control.id      = 0x01;
  aud_control.dest    = AUDIO_DEST;
  aud_control.cmd     = AUD_CONTROL_CMD;
  aud_control.mute    = status->state_data.control1;
  aud_control.inputs  = Inputs;
  aud_control.outputs = status->state_data.control3;

  cp_snd ( &aud_control);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_Outputs
*
*  Command Used: Audio Control (0x01)
*
*  Input Parms:
*       Outputs : PS/2 speaker/Mono/Stereo
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_Outputs (BYTE Outputs)
                         /* OR the following bit values together to create val*/
                         /* 0x02 = Activate the PS/2 Speaker                 */
                         /* 0x01 = Monophonic speaker output                 */
{
  PREAD_AUDIO_STATUS status;
  AUDIO_CONTROL aud_control;

  status = cpaud_ReadStatus ( );

  aud_control.count   = (BYTE)sizeof (AUDIO_CONTROL);
  aud_control.id      = 0x01;
  aud_control.dest    = AUDIO_DEST;
  aud_control.cmd     = AUD_CONTROL_CMD;
  aud_control.mute    = status->state_data.control1;
  aud_control.inputs  = status->state_data.control2;
  aud_control.outputs = Outputs;

  cp_snd ( &aud_control);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_ReadStatus
*
*  Command Used: Read Audio Status (0x02)
*
*  Input Parms: NONE
*
*  Returns:
*       Ptr to audio status in response buffer
*       Caller must check Response Byte 1 for error
*****************************************************************************/
PREAD_AUDIO_STATUS cpaud_ReadStatus ( )
{
  CP_CMD aud_readstatus;

  aud_readstatus.count   = (BYTE)sizeof (CP_CMD);
  aud_readstatus.id      = 0x01;
  aud_readstatus.dest    = AUDIO_DEST;
  aud_readstatus.cmd     = AUD_READ_STATUS_CMD;

  cp_snd ( &aud_readstatus);

  /* Wait for response from this command  */
  cpaud_Wait ( aud_readstatus.dest, aud_readstatus.cmd );             /*01.01*/

  /* return the response */
  return ( (PREAD_AUDIO_STATUS)&rcvbuf.data[0]);
}

/*****************************************************************************
*  Function: cpaud_RestoreStatus
*
*  Command Used: Restore Audio Status (0x03)
*
*  Input Parms:
*       Ptr to an audio status buffer, (as returned from cpaud_ReadStatus)
*
*  Returns:
*       Ptr to audio response buffer
*****************************************************************************/
CP_FRCVBUF cpaud_RestoreStatus (PAUDIO_STATE StateData)
{
  RESTORE_AUD_STATE aud_restorestatus;

  aud_restorestatus.count   = (BYTE)sizeof (RESTORE_AUD_STATE);
  aud_restorestatus.id      = 0x01;
  aud_restorestatus.dest    = AUDIO_DEST;
  aud_restorestatus.cmd     = AUD_RESTORE_STATUS_CMD;
  memcpy ((char *)&aud_restorestatus.state_data,
          (char *)StateData,
          (UINT) sizeof (AUDIO_STATE));

  cp_snd ( &aud_restorestatus);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_ConfigInputs
*
*  Command Used: Configure External Audio Inputs (0x04)
*
*  Input Parms:
*       BOOL Flags  : One for each of the input sources. TRUE if to be used.
*       BYTE levels : value 0-100% for each level.  Only used if corresponding
*                     ON flag is TRUE.
*
*  Returns:
*       Ptr to audio response buffer
*****************************************************************************/
typedef struct _AUD_CNFG_INPUTS {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          extinput1L;
  BYTE          extinput1R;
  BYTE          extinput2L;
  BYTE          extinput2R;
} AUD_CNFG_INPUTS;

CP_FRCVBUF cpaud_ConfigInputs  (BOOL L1On, BYTE L1Level,
                                BOOL R1On, BYTE R1Level,
                                BOOL L2On, BYTE L2Level,
                                BOOL R2On, BYTE R2Level)
{
  AUD_CNFG_INPUTS cnfg_inputs;

  cnfg_inputs.count       = (BYTE)sizeof (AUD_CNFG_INPUTS);
  cnfg_inputs.id          = 0x01;
  cnfg_inputs.dest        = AUDIO_DEST;
  cnfg_inputs.cmd         = AUD_CONFIG_INPUTS_CMD;
  cnfg_inputs.extinput1L  = L1Level + (L1On? 0x80 : 0);
  cnfg_inputs.extinput1R  = R1Level + (R1On? 0x80 : 0);
  cnfg_inputs.extinput2L  = L2Level + (L2On? 0x80 : 0);
  cnfg_inputs.extinput2R  = R2Level + (R2On? 0x80 : 0);


  cp_snd ( &cnfg_inputs);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_Volume
*
*  Command Used: Adjust Main Volume (0x05)
*
*  Input Parms:
*       MakeDefault : TRUE if this level is to be the new default
*       Level       : New value for this attribute
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/

CP_FRCVBUF  cpaud_Volume (BOOL MakeDefault, BYTE Level)
{
  CP_LEVELS aud_volume;

  aud_volume.count        = (BYTE)sizeof (CP_LEVELS);
  aud_volume.id           = 0x01;
  aud_volume.dest         = AUDIO_DEST;
  aud_volume.cmd          = AUD_MAIN_VOLUME_CMD;
  aud_volume.default_flag = (BYTE)(MakeDefault? 1 : 0);
  aud_volume.level        = Level;

  cp_snd ( &aud_volume);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_DefVolume
*
*  Command Used: Adjust Main Volume (0x05)
*
*  Input Parms: None
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/

CP_FRCVBUF  cpaud_DefVolume ()
{
  CP_CMD aud_volume;

  aud_volume.count        = (BYTE)sizeof (CP_CMD);
  aud_volume.id           = 0x01;
  aud_volume.dest         = AUDIO_DEST;
  aud_volume.cmd          = AUD_MAIN_VOLUME_CMD;

  cp_snd ( &aud_volume);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}



/*****************************************************************************
*  Function: cpaud_Balance
*
*  Command Used: Adjust Balance (0x06)
*
*  Input Parms:
*       MakeDefault : TRUE if this level is to be the new default
*       Level       : New value for this attribute
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_Balance (BOOL MakeDefault,  BYTE Level)
{
  CP_LEVELS aud_balance;

  aud_balance.count        = (BYTE)sizeof (CP_LEVELS);
  aud_balance.id           = 0x01;
  aud_balance.dest         = AUDIO_DEST;
  aud_balance.cmd          = AUD_BALANCE_CMD;
  aud_balance.default_flag = (BYTE)(MakeDefault? 1 : 0);
  aud_balance.level        = Level;

  cp_snd ( &aud_balance);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_DefBalance
*
*  Command Used: Adjust Balance (0x06)
*
*  Input Parms: NONE
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/

CP_FRCVBUF  cpaud_DefBalance ()
{
  CP_CMD aud_balance;

  aud_balance.count        = (BYTE)sizeof (CP_CMD);
  aud_balance.id           = 0x01;
  aud_balance.dest         = AUDIO_DEST;
  aud_balance.cmd          = AUD_BALANCE_CMD;

  cp_snd ( &aud_balance);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_Bass
*
*  Command Used: Adjust Bass (0x07)
*
*  Input Parms:
*       MakeDefault : TRUE if this level is to be the new default
*       Level       : New value for this attribute
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_Bass  (BOOL MakeDefault,  BYTE Level)
{
  CP_LEVELS aud_bass;

  aud_bass.count        = (BYTE)sizeof (CP_LEVELS);
  aud_bass.id           = 0x01;
  aud_bass.dest         = AUDIO_DEST;
  aud_bass.cmd          = AUD_BASS_CMD;
  aud_bass.default_flag = (BYTE)(MakeDefault? 1 : 0);
  aud_bass.level        = Level;

  cp_snd ( &aud_bass);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_DefBass
*
*  Command Used: Adjust Bass (0x07)
*
*  Input Parms: NONE
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/

CP_FRCVBUF  cpaud_DefBass ()
{
  CP_CMD aud_bass;

  aud_bass.count        = (BYTE)sizeof (CP_CMD);
  aud_bass.id           = 0x01;
  aud_bass.dest         = AUDIO_DEST;
  aud_bass.cmd          = AUD_BASS_CMD;

  cp_snd ( &aud_bass);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_Treble
*
*  Command Used: Adjust Treble (0x08)
*
*  Input Parms:
*       MakeDefault : TRUE if this level is to be the new default
*       Level       : New value for this attribute
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_Treble (BOOL MakeDefault,  BYTE Level)
{
  CP_LEVELS aud_treble;

  aud_treble.count        = (BYTE)sizeof (CP_LEVELS);
  aud_treble.id           = 0x01;
  aud_treble.dest         = AUDIO_DEST;
  aud_treble.cmd          = AUD_TREBLE_CMD;
  aud_treble.default_flag = (BYTE)(MakeDefault? 1 : 0);
  aud_treble.level        = Level;

  cp_snd ( &aud_treble);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_DefTreble
*
*  Command Used: Adjust Treble (0x08)
*
*  Input Parms: NONE
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/

CP_FRCVBUF  cpaud_DefTreble ()
{
  CP_CMD aud_treble;

  aud_treble.count        = (BYTE)sizeof (CP_CMD);
  aud_treble.id           = 0x01;
  aud_treble.dest         = AUDIO_DEST;
  aud_treble.cmd          = AUD_TREBLE_CMD;

  cp_snd ( &aud_treble);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_Fade
*
*  Command Used: Adjust Fade (0x09)
*
*  Input Parms:
*       MakeDefault : TRUE if this level is to be the new default
*       Level       : New value for this attribute
*
*  Returns: NONE
*****************************************************************************/
typedef struct _AUD_FADE {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          level;
  BYTE          time_tenths;
} AUD_FADE;
void cpaud_Fade (BYTE Level, BYTE Time_in_Tenths)
{
  AUD_FADE aud_fade;

  aud_fade.count       = (BYTE)sizeof (AUD_FADE);
  aud_fade.id          = 0x01;
  aud_fade.dest        = AUDIO_DEST;
  aud_fade.cmd         = AUD_FADE_CMD;
  aud_fade.level       = Level;
  aud_fade.time_tenths = Time_in_Tenths;

  cp_snd ( &aud_fade );

  return;
}

/*****************************************************************************
*  Function: cpaud_DefFade
*
*  Command Used: Adjust Fade (0x09)
*
*  Input Parms: NONE
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/

void  cpaud_DefFade ()
{
  CP_CMD aud_fade;

  aud_fade.count        = (BYTE)sizeof (CP_CMD);
  aud_fade.id           = 0x01;
  aud_fade.dest         = AUDIO_DEST;
  aud_fade.cmd          = AUD_FADE_CMD;

  cp_snd ( &aud_fade);

  /* return the response */
  return;
}



/*****************************************************************************
*  Function: cpaud_BufferInit
*
*  Command Used: Init Audio Buffer (0x0A)
*
*  Description:
*       Initializes a buffer for use as an Audio Buffer for the MControl
*     Program.  THIS MUST BE PERFORMED BEFORE CALLING cpaud_BufferActivate ().
*
*  Input Parms:
*       Buffer          : Ptr to buffer to init
*       BufferLen       : Length of buffer in bytes
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
typedef struct _AUDIO_BUFF_INIT {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  PVOID         buffer;
  USHORT        bufflen;
} AUDIO_BUFF_INIT;


CP_FRCVBUF  cpaud_BufferInit (PVOID Buffer, USHORT BufferLen)
{
  AUDIO_BUFF_INIT buff_init;

  buff_init.count   = (BYTE)sizeof (AUDIO_BUFF_INIT);
  buff_init.id      = 0x01;
  buff_init.dest    = AUDIO_DEST;
  buff_init.cmd     = AUD_INIT_BUFFER_CMD;
  buff_init.buffer  = Buffer;
  buff_init.bufflen = BufferLen;

  cp_snd ( &buff_init );
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_BufferActivate
*
*  Command Used: Activate Audio Buffer (0x0B)
*
*  Description:
*       Activates an audio buffer for use with recording or playback.
*     Your application program allocates the buffer, then passes the
*     ptr to the buffer to the Activate function with registers it
*     with the M-Control Program.
*       This buffer will become the CURRENT audio buffer, and must be closed
*     later.
*
*     NOTE!! : CALL cpaud_BufferInit () BEFORE YOU CALL THIS FUNCTION
*
*  Input Parms:
*       Buffer  : Ptr to buffer to activate
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
typedef struct _AUDIO_BUFF_ACTIVE {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  PVOID         buffer;
} AUDIO_BUFF_ACTIVE;


CP_FRCVBUF  cpaud_BufferActivate (PVOID Buffer)
{
  AUDIO_BUFF_ACTIVE buff_active;

  buff_active.count   = (BYTE)sizeof (AUDIO_BUFF_ACTIVE);
  buff_active.id      = 0x01;
  buff_active.dest    = AUDIO_DEST;
  buff_active.cmd     = AUD_ACTIVATE_BUFFER_CMD;
  buff_active.buffer  = Buffer;

  cp_snd ( &buff_active );
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_BufferClose
*
*  Command Used: Activate Audio Buffer (0x0B)
*
*  Description:
*       Closes, or deactivates, the current audio buffer.
*
*  Input Parms:
*       Buffer  : Ptr to buffer to deactivate (close)
*
*  Returns:
*       0L : Error - no buffer, audio handles still open, etc
*       !0 : Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_BufferClose ( )
{
  CP_FRCVBUF  rc;
  CP_CMD buff_active;

  rc = cpaud_AhandleStatus ( );

  /* ERRORS : Misc (NO BUFFER)  or  OPEN AUDIO HANDLES */
  if ((rc->data[0]) || (rc->data[1]))
     {
     return (0L);
     }

  buff_active.count   = (BYTE)sizeof (CP_CMD);
  buff_active.id      = 0x01;
  buff_active.dest    = AUDIO_DEST;
  buff_active.cmd     = AUD_ACTIVATE_BUFFER_CMD;

  cp_snd ( &buff_active );
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_OpenRecord
*
*  Command Used: Open Record Handle   (0x0C)
*
*  Description:
*     Open an audio file for recording.  You may specify either the beginning
*     or an offset into the file.
*
*  Input Parms:
*       Filename        : ascii string holding file name
*       Offset          : 0L - beginning of file
*                         !0 - position in file to record at
*       Ahand           : audio handle assigned in this function
*
*  Returns:
*       Ptr to response buffer
*
*       Note : Special non-saved recording is possible with use of
*              Filename = 0L.  See Prog Guide pp 18-33 and 18-35.
*****************************************************************************/
typedef struct _AUDIO_OPEN_REC_AHAND {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  PBYTE         filename;
  BYTE          type;
  ULONG         offset;
} AUDIO_OPEN_REC_AHAND;


 CP_FRCVBUF  cpaud_OpenRecord (char *Filename, ULONG Offset, PBYTE Ahand)
/* CP_FRCVBUF  cpaud_OpenRecord (char *Filename, ULONG Offset, BYTE *Ahand)*/
{
  AUDIO_OPEN_REC_AHAND open_rec_ahand;

  open_rec_ahand.count    = (BYTE)sizeof (AUDIO_OPEN_REC_AHAND);
  open_rec_ahand.id       = 0x01;
  open_rec_ahand.dest     = AUDIO_DEST;
  open_rec_ahand.cmd      = AUD_OPEN_REC_HANDLE_CMD;
  open_rec_ahand.type     = 255;
  open_rec_ahand.filename = Filename;
  open_rec_ahand.offset   = Offset;

  cp_snd ( &open_rec_ahand );

  /* Wait for response from this command  */
  if ( cpaud_Wait ( open_rec_ahand.dest, open_rec_ahand.cmd ) )       /*01.01*/
  {                                                                   /*01.01*/
     if ( rcvbuf.data[0] )                                            /*01.01*/
        *Ahand = 0;                                                   /*01.01*/
     else                                                             /*01.01*/
        *Ahand = rcvbuf.data[3];                                      /*01.01*/
  }                                                                   /*01.01*/
  else                                                                /*01.01*/
     *Ahand = 0;                                                      /*01.01*/

  /* return the response */
  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_MakeSegmentList
*
*  Command Used: NONE - utility function
*
*  Description:
*      Opens and evaluates the specifed file to find any audio segments.
*  A audio segment array is allocated if any segments are found.  Each
*  segment number is entered into the array for use with the AUDIO PLAY or
*  AUDIO PLAY MULTIPLE commands.
*
*  Input Parms:
*       Ptr to ascii string for filename to be searched.
*       ADDRESS of a Ptr to an EMPTY segment list
*
*  Changed Parms:
*       Ptr to segment list is assigned a memory block that has be
*       filled with segment data found in the file.
*
*  Returns:
*       0 : if not an audio file, no segments, or memory allocation error
*      !0 : number of audio segments found in file
*****************************************************************************/
BYTE cpaud_MakeSegmentList (char *Filename, AUDIO_SEGMENTS **SegmentArray)
{
  int    done = FALSE,
         ReadRequired,
         rc,
         cursegment = 0,
         filehdl;
  long   curlocation,
         bytesleft;
  ULONG  file_offsets[256];     /* temporary segment array */
  char  *SortBuf,
        *CurBufPtr,
        *resultptr;
  UINT   bytesread;
  AUDIO_HEADER AudHeader;
  AUDIO_SEGMENTS *SegPtr;

  /************************************************/
  /* Attempt to open the file, start at beginning */
  /************************************************/
  filehdl = open (Filename, O_RDONLY | O_BINARY, S_IREAD );

  if (filehdl == 0xFFFF)
    return (0);

  SortBuf = (char *)calloc((UINT)READLENGTH,1);

  if (!SortBuf)
    return(0);

  /* Initialize offset array to empty */
  memset (file_offsets, 0, 256*sizeof (ULONG));

  resultptr = 0L;
  bytesleft = 0L;

  lseek ( filehdl, 0L, SEEK_SET );
  ReadRequired = TRUE;

  /***********************************************/
  /* Loop looking for audio segments in the file */
  /***********************************************/
  while (!done)
     {
     if (ReadRequired)
        {
        bytesread = read(filehdl, SortBuf, READLENGTH);
        if (!bytesread || (bytesread == 0xFFFF))
           {
           done = TRUE;
           break;
           }
        CurBufPtr = SortBuf;
        ReadRequired = FALSE;
        }

     /*************************************************************/
     /* SCAN to find audio header - it may not be sequential      */
     /*************************************************************/
     resultptr = memchr (CurBufPtr, 'P',
                         (bytesread - (UINT)(CurBufPtr - SortBuf)));

     if (resultptr)
        {
        /***********************************************************/
        /*  Check to see if entire header is in the buffer.  If   */
        /*  not, refill buffer starting with address of result.   */
        /***********************************************************/
        if (((long)SortBuf + (long)bytesread) >=
                     ((long)resultptr + sizeof (AUDIO_HEADER)))
           {
           memcpy (&AudHeader, resultptr, sizeof (AUDIO_HEADER));
           if (!strncmp (AudHeader.name, "PCM08M08",8))
              {
              curlocation = lseek (filehdl, 0L, SEEK_CUR);
              curlocation = curlocation - (ULONG)bytesread +
                            (long)(resultptr - SortBuf);
              file_offsets[cursegment++] = curlocation;
              curlocation = lseek(filehdl,curlocation + AudHeader.length,
                                  SEEK_SET);
              ReadRequired = TRUE;
              }

           /***********************************************************/
           /*  Stuff in header didn't match.  Refill buffer. Starting */
           /*  at resultptr + 1.                                      */
           /***********************************************************/
           else
              CurBufPtr = resultptr + 1;
           }

        /**********************************************************/
        /*  Whole header not in buffer.  Reposition file pointer   */
        /*  and read into buffer starting with the P.              */
        /**********************************************************/
        else
           {
           bytesleft = (0L - ((long)bytesread - (long)(resultptr - SortBuf)));
           curlocation = lseek(filehdl,bytesleft, SEEK_CUR);
           ReadRequired = TRUE;
           }
        }
     else
        /* Did NOT find any 'P' chars - read another buffer full */
        ReadRequired = TRUE;

     /* We CANNOT handle more than 255 entries */
     if (cursegment >= 255)
        done = TRUE;

     }

  /* Free memory and the file handle */
  free(SortBuf);
  close (filehdl);

  /* Set up the permanent segment array for the user */
  if (cursegment)
     {
     /* allocate memory for the permanent segment array */
     /*    MUST have 1 ULONG position, 1 ULONG reserved for each segment */
     SegPtr = (AUDIO_SEGMENTS *)calloc ((UINT)cursegment,
                                      (UINT)sizeof (AUDIO_SEGMENTS));

     /* Was memory allocated? */
     if (SegPtr)
        {
        /* copy the offsets into the permanent array.  Add the reserved ULONG */
        for (rc=0; rc < cursegment; rc++)
           {
           SegPtr[rc].offset     = file_offsets[rc];       /* Position */
           SegPtr[rc].for_cp_use = 0L;                     /* Reserved */
           }

        *SegmentArray = SegPtr;
        }

     /* If not, then an error occured */
     else
        cursegment = 0;
     }

  return ((BYTE)cursegment);
}

/*****************************************************************************
*  Function: cpaud_OpenPlay
*
*  Command Used: Open Play Handle   (0x0D)
*
*  Description:
*     Open an audio file for playback, checks the number of audio segments,
*  obtains an ahand.
*     If the calling passes in the SegmentArray variable with a value of
*  0L, then it is assumed that the file is to be scanned for the audio segments
*  it contains.  The function cpaud_MakeSegmentList performs the file check to
*  determine the number of audio segments.  It will also allocate the memory
*  to hold the audio segment array, placing the pointer to the structure in
*  the variable SegmentArray.  The variable NumSegments will be set to the
*  number of segments found in the file.
*     If the SegmentArray variable has a value, and is therefore ALREADY
*  allocated and filled in, then no scan is performed and no variables are
*  changed.
*
*  NOTE : Calling application MUST SET SegmentArray = 0L if the AUTOSCAN
*         feature is desired.
*
*
*  Input Parms:
*    Filename     : ptr to ASCII string holding name of file
*    Ahand        : ptr that WILL BE ASSIGNED the ahandle.
*    SegmentArray : ADDRESS of the ptr to the segment array
*                   If VALUE of ptr = 0L it WILL BE ALLOCATED AND
*                   ASSIGNED by cpaud_MakeSegmentList.
*    NumSegments  : ptr to a BYTE variable holding number of segments in file
*                   WILL BE ASSIGNED if SegmentArray was = 0L.
*  Returns:
*       Ptr to response buffer
*
*  Examples:
*    To have MCALLS scan the file to build the segment array for you
*       ...
*       AUDIO_SEGMENTS *audsegs;
*       BYTE            numsegs;
*       BYTE            ahand1;
*       ...
*       audsegs = 0L;
*       cpaud_OpenPlay ("SONG1.AUD", &ahand1, &audsegs, &numsegs);
*
*    To use your own segment array
*       ...
*       AUDIO_SEGMENTS audsegs[10];
*       BYTE           numsegs;
*       BYTE           ahand1;
*       ...
*       audsegs[0].offset = 0L;
*       audsegs[1].offset = 10000L;
*       numsegs = 2;
*       cpaud_OpenPlay ("SONG2.AUD", &ahand1, &audsegs, &numsegs);
*
*****************************************************************************/
typedef struct _AUDIO_OPEN_PLAY_AHAND {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  PBYTE         filename;
  BYTE          numsegments;
  PVOID         segmentarray;
} AUDIO_OPEN_PLAY_AHAND;

CP_FRCVBUF  cpaud_OpenPlay (char *Filename, PBYTE Ahand,
/* CP_FRCVBUF  cpaud_OpenPlay (char *Filename, BYTE *Ahand,*/
                            AUDIO_SEGMENTS **SegmentArray, BYTE *NumSegments)
{
  AUDIO_OPEN_PLAY_AHAND open_play_ahand;

  /* Is the segment array already allocated and filled? */
  if (!*SegmentArray)
     /* Determine the number and location of the audio file segments */
     *NumSegments = cpaud_MakeSegmentList (Filename, SegmentArray);

  /* if there were NO segments, then an error : file unopenable,
     not audio file, etc */
  if (!*NumSegments)
     {
     *Ahand = 0;
     return (0L);
     }

  open_play_ahand.count         = (BYTE)sizeof (AUDIO_OPEN_PLAY_AHAND);
  open_play_ahand.id            = 0x01;
  open_play_ahand.dest          = AUDIO_DEST;
  open_play_ahand.cmd           = AUD_OPEN_PLAY_HANDLE_CMD;
  open_play_ahand.filename      = Filename;
  open_play_ahand.numsegments   = *NumSegments;
  open_play_ahand.segmentarray  = (PVOID)*SegmentArray;

  cp_snd ( &open_play_ahand );

  /* Wait for response from this command  */
  if ( cpaud_Wait ( open_play_ahand.dest, open_play_ahand.cmd ) )     /*01.01*/
  {                                                                   /*01.01*/
     if ( rcvbuf.data[0] )                                            /*01.01*/
        /* ERROR - don't return a handle */
        *Ahand = 0;                                                   /*01.01*/
     else                                                             /*01.01*/
        *Ahand = rcvbuf.data[3];                                      /*01.01*/
  }                                                                   /*01.01*/
  else                                                                /*01.01*/
     /* ERROR - don't return a handle */
     *Ahand = 0;                                                      /*01.01*/

  /* return the response */
  return ( &rcvbuf );
}



/*****************************************************************************
*  Function: cpaud_Close
*
*  Command Used: Close Audio Handle   (0x0E)
*
*  Description:
*       Closes the specified audio handle, and frees the associated
*  segment array (if one exists).
*
*  Input Parms:
*       Ahand        : Audio handle to be closed.
*       SegmentArray : Ptr to the segment array for this handle.
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
typedef struct _AUDIO_CLOSE_AHAND {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          ahand;
} AUDIO_CLOSE_AHAND;

CP_FRCVBUF  cpaud_Close (BYTE Ahand, AUDIO_SEGMENTS **SegmentArray)
{
  AUDIO_CLOSE_AHAND close_ahand;

  close_ahand.count    = (BYTE)sizeof (AUDIO_CLOSE_AHAND);
  close_ahand.id       = 0x01;
  close_ahand.dest     = AUDIO_DEST;
  close_ahand.cmd      = AUD_CLOSE_HANDLE_CMD;
  close_ahand.ahand = Ahand;

  cp_snd ( &close_ahand );

  /* Wait for response from this command  */
  cpaud_Wait ( close_ahand.dest, close_ahand.cmd );                   /*01.01*/

  /* deallocate the segment array information */
  if (SegmentArray)
     free (SegmentArray);

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_AhandleStatus
*
*  Command Used: Check Handle Status   (0x0F)
*
*  Description:
*       Returns information associated with this audio handle: the filename,
*       number of audio segments, record or playback, etc.
*
*  Input Parms: NONE
*
*  Returns:
*       Ptr to response buffer
*
*       Format and values are described in the Prog Guide pp 18-44.
*****************************************************************************/
CP_FRCVBUF  cpaud_AhandleStatus ( )
{
  CP_CMD status_ahand;

  status_ahand.count    = (BYTE)sizeof (CP_CMD);
  status_ahand.id       = 0x01;
  status_ahand.dest     = AUDIO_DEST;
  status_ahand.cmd      = AUD_HANDLE_STATUS_CMD;

  cp_snd ( &status_ahand );

  /* Wait for response from this command  */
  cpaud_Wait ( status_ahand.dest, status_ahand.cmd );                 /*01.01*/

  /* return the response */
  return ( &rcvbuf );
}


/****************************************************************************
*  Function: cpaud_RecordVolume
*
*  Command Used: Adjust Record Volume  (0x10)
*
*  Description:
*       Effects the RECORD volume, not the output volume.  Effectively
*       increases or decreases the input sensitivity gain.
*
*  Input Parms:
*       MakeDefault : TRUE if this level is to be the new default
*       Level       : New value for this attribute
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
typedef struct _AUDIO_REC_VOLUME {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          control;
  BYTE          level;
} AUDIO_REC_VOLUME;


CP_FRCVBUF  cpaud_RecordVolume (BOOL MakeDefault,  BYTE Level)
{
  CP_LEVELS  rec_vol;

  rec_vol.count         = (BYTE)sizeof (CP_LEVELS);
  rec_vol.id            = 0x01;
  rec_vol.dest          = AUDIO_DEST;
  rec_vol.cmd           = AUD_RECORD_VOLUME_CMD;
  rec_vol.default_flag  = (BYTE)(MakeDefault? 1 : 0);
  rec_vol.level         = Level;

  cp_snd ( &rec_vol );
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_Record
*
*  Command Used: Record Digitized Audio  (0x11)
*
*  Description:
*       Records digitized audio to the file associated with the current
*  record handle from OpenRecord ().  Continues until a STOP.
*
*  Input Parms:
*       Ahand        : handle from OpenRecord ()
*       Control      : video settings - left OR right channels (NOT BOTH!)
*
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
typedef struct _AUDIO_REC {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          control;
  BYTE          ahand;
  ULONG         msec;
} AUDIO_REC;


CP_FRCVBUF  cpaud_Record (BYTE Ahand, BYTE Control)
{
  AUDIO_REC  record;

                   /* DON'T include data area for milliseconds duration */
  record.count   = (BYTE)sizeof (AUDIO_REC) - sizeof (ULONG);
  record.id      = 0x01;
  record.dest    = AUDIO_DEST;
  record.cmd     = AUD_RECORD_CMD;
  record.control = Control;
  record.ahand   = Ahand;

  cp_snd ( &record );
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  if ( !rcvbuf.data[0] )
     cpaud_in_record = TRUE;

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_RecordTimed
*
*  Command Used: Record Digitized Audio  (0x11)
*
*  Description:
*       Records digitized audio to the file associated with the current
*  record handle from OpenRecord ().
*
*  Input Parms:
*       Ahand        : handle from OpenRecord ()
*       Control      : video settings - left OR right channels (NOT BOTH!)
*       milliseconds : duration to record
*
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/

CP_FRCVBUF  cpaud_RecordTimed (BYTE Ahand, BYTE Control, ULONG Milliseconds)
{
  AUDIO_REC  record;

  record.count   = (BYTE)sizeof (AUDIO_REC);
  record.id      = 0x01;
  record.dest    = AUDIO_DEST;
  record.cmd     = AUD_RECORD_CMD;
  record.control = Control;
  record.ahand   = Ahand;
  record.msec    = Milliseconds;

  cp_snd ( &record );
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  if ( !rcvbuf.data[0] )
     cpaud_in_record = TRUE;

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_RecordSource
*
*  Command Used: Record Digitized Audio  (0x11)
*
*  Description:
*       Changes the audio source for a RECORD IN PROGRESS.
*
*  Input Parms:
*       Control      : video setting - LEFT OR RIGHT channels (NOT BOTH!)
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/

CP_FRCVBUF  cpaud_RecordSource (BYTE Control)
{
  CP_CMD_N_DATA1  recordsource;

  recordsource.count     = (BYTE)sizeof (CP_CMD_N_DATA1);
  recordsource.id        = 0x01;
  recordsource.dest      = AUDIO_DEST;
  recordsource.cmd       = AUD_RECORD_CMD;
  recordsource.cmd_byte1 = Control;

  cp_snd ( &recordsource );
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  if ( !rcvbuf.data[0] )
     cpaud_in_record = TRUE;

  /* return the response */
  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_Monitor
*
*  Command Used: Monitor Digitizing Level     (0x12)
*
*  Description:
*
*  Input Parms:
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_Monitor (AUDIO_MONITOR *MonitorBuff)
{
  CP_CMD monitor;

  monitor.count    = (BYTE)sizeof (CP_CMD);
  monitor.id       = 0x01;
  monitor.dest     = AUDIO_DEST;
  monitor.cmd      = AUD_MONITOR_DIGI_LVL_CMD;

  cp_snd ( &monitor );

  /* Wait for response from this command  */
  cpaud_Wait ( monitor.dest, monitor.cmd );                           /*01.01*/

  *MonitorBuff = *(AUDIO_MONITOR *)(&rcvbuf.data[1]);

  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_PlayVolume
*
*  Command Used: Adjust Digital Playback Volume (0x13)
*
*  Description:
*
*  Input Parms:
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_PlayVolume (BOOL MakeDefault, BYTE Level)
{
  CP_LEVELS aud_playvol;

  aud_playvol.count        = (BYTE)sizeof (CP_LEVELS);
  aud_playvol.id           = 0x01;
  aud_playvol.dest         = AUDIO_DEST;
  aud_playvol.cmd          = AUD_DIGI_PLAY_VOLUME_CMD;
  aud_playvol.default_flag = (BYTE)(MakeDefault? 1 : 0);
  aud_playvol.level        = Level;

  cp_snd ( &aud_playvol);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_DefPlayVolume
*
*  Command Used: Adjust Digital Playback Volume (0x13)
*
*  Input Parms: NONE
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/

CP_FRCVBUF  cpaud_DefPlayVolume ()
{
  CP_CMD aud_playvol;

  aud_playvol.count        = (BYTE)sizeof (CP_CMD);
  aud_playvol.id           = 0x01;
  aud_playvol.dest         = AUDIO_DEST;
  aud_playvol.cmd          = AUD_DIGI_PLAY_VOLUME_CMD;

  cp_snd ( &aud_playvol);
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function: cpaud_Play
*
*  Command Used: Play Digitized Audio (0x14)
*
*  Description :
*       Allows playing ANY segment, for ANY length of time, as well as
*       control over the control bits.
*
*  Input Parms:
*       Control         : video settings
*       Ahand           : audio handle to use
*       Segment         : number of the segment to play
*       Start,Stop      : Start and Stop times (milliseconds) in the segment
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
typedef struct _AUDIO_PLAY {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          control;
  BYTE          ahand;
  BYTE          subhand;
  ULONG         start_msec;
  ULONG         stop_msec;
} AUDIO_PLAY;

CP_FRCVBUF  cpaud_Play (BYTE Control, BYTE Ahand, BYTE Segment,
                        ULONG Start, ULONG Stop)
{
  AUDIO_PLAY aud_play;

  aud_play.count      = (BYTE)sizeof (AUDIO_PLAY);
  aud_play.id         = AUD_PLAY_ID;
  aud_play.dest       = AUDIO_DEST;
  aud_play.cmd        = AUD_PLAY_CMD;
  aud_play.control    = Control;
  aud_play.ahand      = Ahand;
  aud_play.subhand    = Segment;
  aud_play.start_msec = Start;
  aud_play.stop_msec  = Stop;

  cp_snd ( &aud_play );

  if ( Q )
     return ( 0L );

  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  if ( !rcvbuf.data[0] )
     cpaud_in_play = TRUE;

  cpaud_repchange_ahand = 0;
  cpaud_repchange_seg   = 0;
  cpaud_repchange_pos   = 0L;

  /* return the response */
  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_PlaySegment
*
*  Command Used: Play Digitized Audio (0x14)
*
*  Description :
*       Plays a specifed audio segment, starting at the beginning of the
*       audio segment.
*
*  Input Parms:
*       Control         : video settings
*       Ahand           : audio handle to use
*       Segment         : number of the segment to play
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_PlaySegment (BYTE Control, BYTE Ahand, BYTE Segment)
{
  AUDIO_PLAY aud_play;

  /* use the AUDIO_PLAY struct, but don't send the TIMES */
  aud_play.count      = (BYTE)sizeof (AUDIO_PLAY) - (sizeof (ULONG) * 2);
  aud_play.id         = AUD_PLAY_ID;
  aud_play.dest       = AUDIO_DEST;
  aud_play.cmd        = AUD_PLAY_CMD;
  aud_play.control    = Control;
  aud_play.ahand      = Ahand;
  aud_play.subhand    = Segment;

  cp_snd ( &aud_play );

  if ( Q )
     return ( 0L );

  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  if ( !rcvbuf.data[0] )
     cpaud_in_play = TRUE;

  cpaud_repchange_ahand = 0;
  cpaud_repchange_seg   = 0;
  cpaud_repchange_pos   = 0L;

  /* return the response */
  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_PlayCurrent
*
*  Command Used: Play Digitized Audio (0x14)
*
*  Description :
*       Plays the CURRENT audio segment, starting at the current position.
*
*  Input Parms:
*       Control         : video settings
*       Ahand           : audio handle to use
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_PlayCurrent (BYTE Control, BYTE Ahand)
{
  AUDIO_PLAY aud_play;

  /* use the AUDIO_PLAY struct, but don't send the SEGMENT or TIMES */
  aud_play.count      = (BYTE)sizeof (AUDIO_PLAY) - 1 - (sizeof (ULONG) * 2);
  aud_play.id         = AUD_PLAY_ID;
  aud_play.dest       = AUDIO_DEST;
  aud_play.cmd        = AUD_PLAY_CMD;
  aud_play.control    = Control;
  aud_play.ahand      = Ahand;

  cp_snd ( &aud_play );

  if ( Q )
     return ( 0L );

  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  if ( !rcvbuf.data[0] )
     cpaud_in_play = TRUE;

  cpaud_repchange_ahand = 0;
  cpaud_repchange_seg   = 0;
  cpaud_repchange_pos   = 0L;

  /* return the response */
  return ( &rcvbuf );
}

/*****************************************************************************
*  Function: cpaud_PlayMulti
*
*  Command Used: Play Multiple Audio Entries (0x15)
*
*  Description:
*       Plays the set of audio segments specified in the PlayArray
*  audio segment array.  These may be One-Byte or Ten-Byte format
*  array entries.
*
*  Input Parms:
*       Control         : video settings
*       Entries         : number of segments to be played from the array
*       Ahand           : !0 = audio handle
*                       : 0L = NO HANDLE - Array is in TEN byte format
*       PlayArray       : Array or 1 or 10 byte segment entries.
*
*       Details about the control bits, segment array, 1 and 10 byte entries
*       are in the Prog Guide pp 18-61 to 18-66
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
typedef struct _AUDIO_MULTIPLAY {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          control;
  BYTE          entries;
  BYTE          ahand;
  PVOID         array;
} AUDIO_MULTIPLAY;

CP_FRCVBUF  cpaud_PlayMulti (BYTE Control, BYTE Entries, BYTE Ahand,
                             PVOID PlayArray)
{
  AUDIO_MULTIPLAY multi_play;

  multi_play.count      = (BYTE)sizeof (AUDIO_MULTIPLAY);
  multi_play.id         = AUD_PLAY_ID;
  multi_play.dest       = AUDIO_DEST;
  multi_play.cmd        = AUD_PLAY_MULTIPLE_CMD;
  multi_play.control    = Control;
  multi_play.entries    = Entries;
  multi_play.ahand      = Ahand;
  multi_play.array      = PlayArray;

  cp_snd (&multi_play );

  if ( Q )
     return ( 0L );

  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  if ( !rcvbuf.data[0] )
     cpaud_in_play = TRUE;

  cpaud_repchange_ahand = 0;
  cpaud_repchange_seg   = 0;
  cpaud_repchange_pos   = 0L;

  return ( &rcvbuf );         /* return the response                         */
}

/*****************************************************************************
*  Function:  cpaud_Stop
*
*  Command Used: Stop Audio Play/Record (0x16)
*
*  Input Parms: NONE
*
*  Returns:
*       Ptr to response buffer
*****************************************************************************/
CP_FRCVBUF  cpaud_Stop ( )
{
  CP_CMD aud_stop;

  aud_stop.count = (BYTE)sizeof (CP_CMD);
  aud_stop.id    = AUD_STOP_ID;
  aud_stop.dest  = AUDIO_DEST;
  aud_stop.cmd   = AUD_STOP_CMD;

  cp_snd ( &aud_stop );
  while ( !AUDIO_HAS_MSG );
  cp_rcv ( AUDIO_DEST );

  /* return the response */
  return ( &rcvbuf );
}


/*****************************************************************************
*  Function:  cpaud_ReportChange
*
*  Command Used: Report Audio Change (0x17)
*
*  Input Parms: NONE
*
*  Returns: NONE
*****************************************************************************/
void  cpaud_ReportChange ( )
{
  CP_CMD aud_stop;

  aud_stop.count = (BYTE)sizeof (CP_CMD);
  aud_stop.id    = 0x01;
  aud_stop.dest  = AUDIO_DEST;
  aud_stop.cmd   = AUD_REPORT_CHANGE_CMD;

  cp_snd ( &aud_stop );

  /* Response data is stored in VARIABLES :
        cpaud_repchange_ahand : audio handle
        cpaud_repchange_seg   : segment number
        cpaud_repchange_pos   : current position

     Since this function does NOT provide a response until/unless
     a play or record is in progress.  */
}


/*****************************************************************************
*  Function:  cpaud_Wait
*
*  Command Used: Wait for audio response
*
*  Input Parms: Destination
*               Command
*
*  Returns: True or False
*****************************************************************************/
BOOL  cpaud_Wait ( BYTE dest, BYTE cmd )                              /*01.01*/
{                                                                     /*01.01*/
  SHORT  i;                                                           /*01.01*/

  for( i = 0; i <= 10; i++ )                                          /*01.01*/
  {                                                                   /*01.01*/
     while ( !AUDIO_HAS_MSG );                                        /*01.01*/
     cp_rcv ( AUDIO_DEST );                                           /*01.01*/
                                                                      /*01.01*/
     if ( ( rcvbuf.dest  == dest ) && ( rcvbuf.cmd  == cmd ) )        /*01.01*/
        return ( TRUE );                                              /*01.01*/
  }                                                                   /*01.01*/
  return ( FALSE );                                                   /*01.01*/
}                                                                     /*01.01*/

