
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBCmd.c          (VSB Command Processing)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains functions that process COMMANDs, ACKs,
//          and RESPONSEs for the VSB UI.  It also contains functions
//          that initiate commands back to the MVPC.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#define INCL_WIN

#include <os2.h>

#include "mvpc.h"       // Include for the MVPC object
#include "vsb.h"        // Public  includes
#include "vsb2.h"       // Private includes


//**************************************************************************<f>
// Func: ProcessCOMMAND                                Author: Steve Henson
//
// Desc: Processes COMMANDS from the MVPC
//
// Retn: Returns through WinDefWindowProc ()
//
//*****************************************************************************
MRESULT ProcessCOMMAND (HWND hwnd, MPARAM mp2)
   {
   PVSBDATA pvsbdata;
   PMVPCCMD pmvpccmd;

   // address this instance's data area
   pvsbdata = WinQueryWindowPtr (hwnd, VSB_WWOFF_PVSB);

   // Convert mp2 to the MVPCCMD ptr:
   pmvpccmd = (PMVPCCMD)PVOIDFROMMP (mp2);

   // switch on the specific command
   switch (pmvpccmd->command.usCommand)
      {
      case VSB_CMD_ADVISEUSER: // App wants to know about user interactions
         pvsbdata->hwndAdviseUser = HWNDFROMMP (pmvpccmd->data.mpCmdParam1);
         break;

      case VSB_CMD_SETPOINTERS: // set pointers
         pvsbdata->hptrActive = HWNDFROMMP (pmvpccmd->data.mpCmdParam1);
         pvsbdata->hptrBusy   = HWNDFROMMP (pmvpccmd->data.mpCmdParam2);
         break;

      case VSB_CMD_SHOWLOG:
         LogShow (hwnd, SHORT1FROMMP (pmvpccmd->data.mpCmdParam1));
         break;
      }

   return ((MRESULT)0);
   }


//**************************************************************************<f>
// Func: ProcessACK                                   Author: Steve Henson
//
// Desc: This function processes any ACK messages received by the VSB.
//       For instance, ACKs from the Cmd...() functions listed below.
//
//*****************************************************************************

MRESULT ProcessACK (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
   {
   PMVPCCMD pmvpccmd;
   USHORT   usCommand;
   USHORT   usReturnCode;
   USHORT   usResponse;
   PVSBDATA pvsbdata;

   pmvpccmd = (PMVPCCMD)mp2;           // set up addressing to parameter block

   usCommand    = pmvpccmd->command.usCommand;     // get original command
   usResponse   = pmvpccmd->command.usResponse;    // get response code
   usReturnCode = pmvpccmd->command.usReturnCode;  // get return code

   pvsbdata = WinQueryWindowPtr (hwnd, VSB_WWOFF_PVSB);  // address instance data

   if (usResponse == MRESP_NORESPONSEPENDING)
      {
      MVPCDestroyCmd (pmvpccmd, (HWND) mp1, msg);
      }
   }


//**************************************************************************<f>
// Func: ProcessRESP                                  Author: Steve Henson
//
// Desc: This function processes any RESPONSEs received by the VSB.
//       For instance, some of the Cmd...() functions listed below.
//
//*****************************************************************************

MRESULT ProcessRESP (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
   {
   PMVPCCMD  pmvpccmd;
   USHORT    usCommand;
   USHORT    usReturnCode;
   USHORT    usResponse;
   PVSBDATA  pvsbdata;

   pmvpccmd = (PMVPCCMD) mp2;          // set up addressing to parameter block

   usCommand    = pmvpccmd->command.usCommand;     // get original command
   usResponse   = pmvpccmd->command.usResponse;    // get response code
   usReturnCode = pmvpccmd->command.usReturnCode;  // get return code

   pvsbdata = WinQueryWindowPtr (hwnd, VSB_WWOFF_PVSB);  // get allocated buffer

   MVPCDestroyCmd (pmvpccmd, (HWND) mp1, msg);
   }


//**************************************************************************<f>
// Func: CmdGetToken                                  Author: Steve Henson
//
// Desc: This function sends an ACTIVATE command to the MVPC to allow
//       it to capture the FOCUS.
//
//*****************************************************************************

BOOL CmdGetToken (HWND hwndMVPC, HWND hwndVSB)
   {
   PMVPCCMD pmvpccmd;               // Ptr to the MVPC Command structure
   BOOL     fReturn = FALSE;        // Return code for this function

   if (MVPCCreateCmd (&pmvpccmd, MVPC_CMD_ACTIVATE))
      {
      pmvpccmd->data.mpCmdParam1 = (MPARAM) TRUE;
      WinPostMsg (hwndMVPC, msgMVPC_COMMAND, hwndVSB, MPFROMP (pmvpccmd));
      fReturn = TRUE;
      }

   return (fReturn);
   }


//**************************************************************************<f>
// Func: CmdSeek                                      Author: Steve Henson
//
// Desc: This function sends a STILLFRAME command to the MVPC.
//
//*****************************************************************************

BOOL CmdSeek (HWND hwndMVPC, HWND hwndVSB, LONG lFrame)
   {
   PMVPCCMD pmvpccmd;               // Ptr to the MVPC Command structure
   BOOL     fReturn = FALSE;        // Return code for this function

   if (MVPCCreateCmd (&pmvpccmd, MVPC_CMD_STILLFRAME))
      {
      pmvpccmd->data.mpCmdParam1 = MPFROMLONG(lFrame);
      WinPostMsg (hwndMVPC, msgMVPC_COMMAND, hwndVSB, MPFROMP (pmvpccmd));
      fReturn = TRUE;
      }

   return (fReturn);
   }


//**************************************************************************<f>
// Func: CmdStep                                      Author: Steve Henson
//
// Desc: This function sends a STEPFORWARD or STEPBACKWARD command to the
//       MVPC object.
//
//*****************************************************************************

BOOL CmdStep (HWND hwndMVPC, HWND hwndVSB, USHORT usDirection)
   {
   PMVPCCMD pmvpccmd;               // Ptr to the MVPC Command structure
   BOOL     fReturn = FALSE;        // Return code for this function

   if (MVPCCreateCmd (&pmvpccmd, (usDirection == FORWARD) ? MVPC_CMD_STEPFORWARD
                                                          : MVPC_CMD_STEPBACKWARD))
      {
      WinPostMsg (hwndMVPC, msgMVPC_COMMAND, hwndVSB, MPFROMP (pmvpccmd));
      fReturn = TRUE;
      }

   return (fReturn);
   }


//**************************************************************************<f>
// Func: CmdPlayFromTo                                Author: Steve Henson
//
// Desc: This function sends a PLAYFROMTO command to the MVPC object.
//
//*****************************************************************************

BOOL CmdPlayFromTo (HWND hwndMVPC, HWND hwndVSB, LONG lFrom, LONG lTo, BOOL fNoReMap)
   {
   BOOL     fReturn = FALSE;        // Return code for this function
   PMVPCCMD pmvpccmd;               // Ptr to the MVPC Command structure

   if (MVPCCreateCmd (&pmvpccmd, MVPC_CMD_PLAYFROMTO))
      {
      pmvpccmd->data.mpCmdParam1 = MPFROMLONG(lFrom);
      pmvpccmd->data.mpCmdParam2 = MPFROMLONG(lTo);
      pmvpccmd->data.mpCmdParam3 = MPFROM2SHORT ((SHORT)fNoReMap,0);
      WinPostMsg (hwndMVPC, msgMVPC_COMMAND, hwndVSB, MPFROMP (pmvpccmd));
      fReturn = TRUE;
      }

   return (fReturn);
   }


//**************************************************************************<f>
// Func: CmdStop                                      Author: Steve Henson
//
// Desc: This function sends a STOP command to the MVPC object.
//
//*****************************************************************************

BOOL CmdStop (HWND hwndMVPC, HWND hwndVSB)
   {
   PMVPCCMD pmvpccmd;               // Ptr to the MVPC Command structure
   BOOL     fReturn = FALSE;        // Return code for this function

   if (MVPCCreateCmd (&pmvpccmd, MVPC_CMD_STOP))
      {
      WinPostMsg (hwndMVPC, msgMVPC_COMMAND, hwndVSB, MPFROMP (pmvpccmd));
      fReturn = TRUE;
      }

   return (fReturn);
   }


//**************************************************************************<f>
// Func: CmdInit                                      Author: Daryl Warinner
//
// Desc: This function sends a INIT command to the MVPC object.
//
//*****************************************************************************

BOOL CmdInit (HWND hwndMVPC, HWND hwndVSB)
   {
   PMVPCCMD pmvpccmd;               // Ptr to the MVPC Command structure
   BOOL     fReturn = FALSE;        // Return code for this function

   if (MVPCCreateCmd (&pmvpccmd, MVPC_CMD_INIT))
      {
      WinPostMsg (hwndMVPC, msgMVPC_COMMAND, hwndVSB, MPFROMP (pmvpccmd));
      fReturn = TRUE;
      }

   return (fReturn);
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
