
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBLog.c          (VSB Log Support)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains the functions and variables necessary
//          for supporting logging to the VSB DLL.  This logging
//          capability may prove useful for other Socketable User Interfaces
//          developed for the Multimedia Videodisc Player Control Object.
//
//          See VSBLOG.H for DEFINEs, TYPEDEFs, and FUNCTION PROTOTYPES.
//          See VSB.RC for Log Messages.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#define INCL_DOSSEMAPHORES    // DosFSRamSemRequest (), DosFSRamSemClear ()
#define INCL_WIN

#include <os2.h>
#include <stdio.h>      // sprintf (), vsprintf ()
#include <stdlib.h>     // ultoa ()
#include <string.h>     // strcat (), strnset (), memset ()
#include <stdarg.h>     // va_start (), va_end (), va_list

#include "mvpc.h"       // Include for the MVPC object
#include "vsb.h"        // Public  includes
#include "vsb2.h"       // Private includes


//*****************************************************************************
// Global Variables used by this module only
//*****************************************************************************

static DOSFSRSEM  semLog;
static ULONG      ulLine;
static char       szLogMask [MAX_LEN_LOGMASK];
static char       szTopOfLog [MAX_LEN_LOGMESSAGE];
static char       szMarkMessage [MAX_LEN_LOGMESSAGE];


//**************************************************************************<f>
// Func: LogLock                                      Author: Daryl Warinner
//
// Desc: This function locks and unlocks the Log Semaphore to prevent
//       multiple instances of VSB Logs from accessing/changing the
//       global string buffers.  It also protects sprintf and other
//       non-reentrant functions from reentrancy.
//
// Retn: VOID
//
//*****************************************************************************

VOID LogLock (BOOL fLock)
   {

   if (fLock)
      DosFSRamSemRequest (&semLog, SEM_INDEFINITE_WAIT);
   else // if (!fLock)
      DosFSRamSemClear (&semLog);
   }


//**************************************************************************<f>
// Func: LogInitialize                                Author: Daryl Warinner
//
// Desc: This function initializes the log semaphore and loads some
//       required strings from the resource file.
//
//       NOTE: Throughout this module, I lock the semaphore before
//             performing string manipulations.  The string buffers are
//             shared between all instances of VSBs, and it is necessary
//             to prevent multiple VSBs from altering the strings
//             simultaneously.  This also prevents re-entrancy to sprintf
//             and other functions that are not re-entrant.
//
// Retn: VOID
//
//*****************************************************************************

VOID LogInitialize (HAB hab)
   {

   if (!semLog.cb)
      semLog.cb = sizeof (DOSFSRSEM);     // Initialize the semaphore

   LogLock (TRUE);
   if (!szLogMask[0])
      WinLoadString (hab, hmodDLL, IDS_LOG_MSGMASK,
                     sizeof (szLogMask),
                     szLogMask);

   if (!szTopOfLog[0])
      WinLoadString (hab, hmodDLL, IDS_LM_TOPOFLOG,
                     sizeof (szTopOfLog),
                     szTopOfLog);
   LogLock (FALSE);
   }


//**************************************************************************<f>
// Func: LogTerminate                                 Author: Daryl Warinner
//
// Desc: This function is called when the VSB is terminating.  It does
//       nothing now, but it may in the future....
//
// Retn: VOID
//
//*****************************************************************************

VOID LogTerminate (HAB hab)
   {
   // Nothing for now....
   }


//**************************************************************************<f>
// Func: LogCreate                                    Author: Daryl Warinner
//
// Desc: This function creates an instance of a log window for a VSB.
//       It allocates a LOGINFO structure and saves a pointer to it in
//       the VSBs Window Word data structure.  It also subclasses the
//       "real" scroll bar to trap Right Button Double Clicks.
//
// Retn: VOID
//
//*****************************************************************************

VOID LogCreate (HWND hwndVSB)
   {
   PVSBDATA          pvsbdata;
   SEL               selLogInfo;
   PLOGINFO          pLogInfo;
   LOGCREATEPARAMS   logcreateparams;

   pvsbdata = WinQueryWindowPtr (hwndVSB, VSB_WWOFF_PVSB);

   // Allocate the LogInfo structure and attach it to the Window word struct
   if (!DosAllocSeg (sizeof (LOGINFO), &selLogInfo, SEG_NONSHARED))
      {
      pLogInfo = (PLOGINFO)(MAKEP (selLogInfo, 0));
      pvsbdata->pLogInfo = pLogInfo;

      // Clear the structure
      memset (pLogInfo, 0, sizeof (LOGINFO));

      logcreateparams.cbData   = sizeof (logcreateparams);
      logcreateparams.usType   = NULL;
      logcreateparams.hab      = WinQueryAnchorBlock (hwndVSB);
      logcreateparams.hwndVSB  = hwndVSB;
      logcreateparams.usPlayer = pvsbdata->usPlayer;

      pLogInfo->hwndVSBLOG = WinLoadDlg (HWND_DESKTOP,
                                         HWND_DESKTOP,
                                         VSBLogDlgProc,
                                         hmodDLL,
                                         IDD_VSBLOG,
                                         &logcreateparams);

      pLogInfo->hwndLOGBOX = WinWindowFromID (pLogInfo->hwndVSBLOG, IDC_LOGBOX);

      pLogInfo->lf.fsAudio         = 1;
      pLogInfo->lf.fsBusy          = 1;
      pLogInfo->lf.fsFrameChange   = 1;
      pLogInfo->lf.fsFrameRange    = 1;
      pLogInfo->lf.fsInfoDisplay   = 1;
      pLogInfo->lf.fsLoaded        = 1;
      pLogInfo->lf.fsMotion        = 1;
      pLogInfo->lf.fsPlayerError   = 1;
      pLogInfo->lf.fsSeek          = 1;
      pLogInfo->lf.fsSpeed         = 1;
      pLogInfo->lf.fsStepIncrement = 1;
      pLogInfo->lf.fsTokenChange   = 1;
      }
   else
      pvsbdata->pLogInfo = (PLOGINFO) NULL;
   }


//**************************************************************************<f>
// Func: LogDestroy                                   Author: Daryl Warinner
//
// Desc: This function is called when the VSB is being destroyed.  It
//       frees the LOGINFO structure and removes the references to it
//       from the VSBs window word data structure.  It also UN-subclasses
//       the "real" scroll bar.
//
// Retn: VOID
//
//*****************************************************************************

VOID LogDestroy (HWND hwndVSB)
   {
   PVSBDATA    pvsbdata;
   PLOGINFO    pLogInfo;

   pvsbdata = WinQueryWindowPtr (hwndVSB, VSB_WWOFF_PVSB);
   if (pLogInfo = pvsbdata->pLogInfo)
      {
      WinDestroyWindow (pLogInfo->hwndVSBLOG);
      pLogInfo->hwndVSBLOG = (HWND) NULL;
      pLogInfo->hwndLOGBOX = (HWND) NULL;

      DosFreeSeg (SELECTOROF (pLogInfo));
      pvsbdata->pLogInfo = (PLOGINFO) NULL;
      }
   }


//**************************************************************************<f>
// Func: LogShow                                      Author: Daryl Warinner
//
// Desc: This function shows or hides the Log Window.
//
//****************************************************************************

VOID LogShow (HWND hwnd, BOOL fShow)
   {
   PVSBDATA    pvsbdata;
   PLOGINFO    pLogInfo;

   pvsbdata = WinQueryWindowPtr (hwnd, VSB_WWOFF_PVSB);
   if (pLogInfo = pvsbdata->pLogInfo)
      {
      WinShowWindow (pLogInfo->hwndVSBLOG, fShow);
      if (fShow)
         WinSetFocus (HWND_DESKTOP, pLogInfo->hwndVSBLOG);
      else
         WinSetFocus (HWND_DESKTOP, pvsbdata->hwndOwner);
      }

   return;
   }


//**************************************************************************<f>
// Func: LogFrame                                  Author: Daryl Warinner
//
// Desc: This function places a frame number into the log box.
//
// Retn: VOID
//
//*****************************************************************************

VOID LogFrame (HWND hwndVSB, LONG lFrame)
   {
   static char szFrame [MAX_LEN_FRAME];
   PVSBDATA    pvsbdata;
   PLOGINFO    pLogInfo;

   pvsbdata = WinQueryWindowPtr (hwndVSB, VSB_WWOFF_PVSB);
   if (pLogInfo = pvsbdata->pLogInfo)
      {
      LogLock (TRUE);                           // sprintf is NOT re-entrant.

      sprintf (szFrame, "%ld", lFrame);
      WinSetDlgItemText (pLogInfo->hwndVSBLOG, IDC_FRAME, szFrame);

      LogLock (FALSE);
      }
   }


//**************************************************************************<f>
// Func: LogMessage                                Author: Daryl Warinner
//
// Desc: This function places a log message into the log list box.
//       It supports a variable number of parameters, and loads the
//       actual string from the Resource file.  To add log messages,
//       you must:
//
//          1) Define the string in VSB.RC
//          2) Define the string ID in VSBLOG.H
//          3) Place a call to LogMessage () in the code.
//
// Retn: VOID
//
//*****************************************************************************

VOID cdecl LogMessage (HWND hwndVSB, USHORT usLogMsgID, ...)
   {
   static char szLoadString [MAX_LEN_LOADSTRING];
   static char szLogMessage [MAX_LEN_LOGMESSAGE];
   static char szLineNumber [MAX_LEN_LINESTRING];
   static char szLogEntry [MAX_LEN_LOGENTRY];
   va_list     pArguments;
   PVSBDATA    pvsbdata;
   PLOGINFO    pLogInfo;
   SHORT       itemindex;
   HAB         hab;
   PSZ         pszLine = szLineNumber;

   // Retrieve the Window Word data structure
   pvsbdata = WinQueryWindowPtr (hwndVSB, VSB_WWOFF_PVSB);
   if (pLogInfo = pvsbdata->pLogInfo)
      {
      LogLock (TRUE);                           // vsprintf is NOT re-entrant.

      // Get the Anchor Block handle for this window
      hab = WinQueryAnchorBlock (hwndVSB);

      // Load the message string from the resource file
      WinLoadString (hab, hmodDLL, usLogMsgID, sizeof (szLoadString), szLoadString);

      // Copy the message (w/ arguments) into a temporary buffer
      va_start (pArguments, usLogMsgID);
      vsprintf (szLogMessage, szLoadString, pArguments);
      va_end (pArguments);

      // Convert the current line number into an ASCII string
      ulLine++;
      ulLine &= 0x0000FFFFL;                       // Lop off at 65535
      pszLine = ultoa (ulLine, szLineNumber, 10);

      // Copy the line number and message string into the buffer
      sprintf (szLogEntry, szLogMask, pszLine, szLogMessage);

      // Insert the buffered string into the list box
      itemindex = (SHORT) WinSendMsg (pLogInfo->hwndLOGBOX,
                                      LM_INSERTITEM,
                                      (MPARAM) LIT_END,
                                      szLogEntry);

      // Make sure the string is visible
      WinSendMsg (pLogInfo->hwndLOGBOX, LM_SETTOPINDEX,
                  (MPARAM) itemindex, (MPARAM) NULL);

      if (itemindex > MAX_LOGMESSAGES)
         {
         WinSendMsg (pLogInfo->hwndLOGBOX,
                     LM_DELETEITEM,
                     (MPARAM) 0,
                     (MPARAM) NULL);
         }
      LogLock (FALSE);
      }
   }


//**************************************************************************<f>
// Func: VSBLogDlgProc                                Author: Daryl Warinner
//
// Parm: HWND     hwnd;    -- The user interface window handle
//       USHORT   msg;     -- The message
//       MPARAM   mp1;     -- message specific
//       MPARAM   mp2;     -- message specific
//
// Desc: This is the window procedure for the VSBLOG modeless dialog box.
//
//*****************************************************************************

MRESULT EXPENTRY VSBLogDlgProc (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
   {
   static char       szTitle [MAX_LEN_LOGTITLE];
   static char       szLoadString [MAX_LEN_LOADSTRING];
   HWND              hwndLOGBOX;
   SHORT             itemindex;
   PLOGCREATEPARAMS  plogcreateparams;
   USHORT            usReturn;

   switch (msg)
      {
      //-----------------------------------------------------------------------
      case WM_INITDLG:
         plogcreateparams = (PLOGCREATEPARAMS) PVOIDFROMMP (mp2);

         LogLock (TRUE);
         // Load the TITLE
         WinLoadString (plogcreateparams->hab,
                        hmodDLL,
                        IDS_LOG_TITLE,
                        sizeof (szTitle),
                        szTitle);

         WinLoadString (plogcreateparams->hab,
                        hmodDLL,
                        (IDS_LOG_TITLE + plogcreateparams->usPlayer),
                        sizeof (szLoadString),
                        szLoadString);

         strcat (szTitle, szLoadString);

         WinSetWindowText (hwnd, szTitle);

         // Initialize the Log List Box
         hwndLOGBOX = WinWindowFromID (hwnd, IDC_LOGBOX);
         WinSendMsg (hwndLOGBOX, LM_DELETEALL, NULL, NULL);

         itemindex = (SHORT) WinSendMsg (hwndLOGBOX,
                                         LM_INSERTITEM,
                                         (MPARAM) LIT_END,
                                         szTopOfLog);
         WinSendMsg (hwndLOGBOX, LM_SETTOPINDEX, (MPARAM) itemindex, NULL);
         LogLock (FALSE);

         return ((MRESULT) 0);
         break;

      //-----------------------------------------------------------------------
      case WM_CONTROL:
         //--------------------------------------------------------------------
         // Currently, this code doesn't do anything.  But it is provided
         //    in case you want to add something....
         //--------------------------------------------------------------------
         switch (SHORT1FROMMP (mp1))            // Control ID
            {
            case IDC_LOGBOX:
               switch (SHORT2FROMMP (mp1))      // Notification Code
                  {
                  default:
                     return (WinDefDlgProc (hwnd, msg, mp1, mp2));
                     break;
                  }
               break;

            default:
               return (WinDefDlgProc (hwnd, msg, mp1, mp2));
               break;
            }

         return ((MRESULT) 0);
         break;

      //-----------------------------------------------------------------------
      case WM_COMMAND:
         hwndLOGBOX = WinWindowFromID (hwnd, IDC_LOGBOX);
         switch (COMMANDMSG (&msg)->cmd)        // Control ID
            {
            case IDC_OK:
               // Hide the window
               WinShowWindow (hwnd, FALSE);
               WinSetFocus (HWND_DESKTOP, WinQueryWindow (hwnd, QW_OWNER, FALSE));
               break;

            case IDC_MARK:
               // Bring up and process the mark dialog box
               LogLock (TRUE);
               usReturn = WinDlgBox (HWND_DESKTOP,
                                     hwnd,
                                     VSBLogMarkDlgProc,
                                     hmodDLL,
                                     IDD_VSBLOGMARK,
                                     NULL);
               if (usReturn == IDC_MARKADD)
                  {
                  itemindex = (SHORT) WinSendMsg (hwndLOGBOX,
                                                  LM_INSERTITEM,
                                                  (MPARAM) LIT_END,
                                                  szMarkMessage);
                  WinSendMsg (hwndLOGBOX, LM_SETTOPINDEX, (MPARAM) itemindex, NULL);
                  }

               // Clear the message string
               strnset (szMarkMessage, 0, sizeof (szMarkMessage));
               LogLock (FALSE);
               break;

            case IDC_CLEAR:
               // Re-initialize the Log List Box
               LogLock (TRUE);
               WinSendMsg (hwndLOGBOX, LM_DELETEALL, NULL, NULL);
               ulLine = 0L;
               LogLock (FALSE);
               break;
            }

         return ((MRESULT) 0);
         break;

      //-----------------------------------------------------------------------
      default:
         return (WinDefDlgProc (hwnd, msg, mp1, mp2));
         break;
      }
   }


//**************************************************************************<f>
// Func: VSBLogMarkDlgProc                            Author: Daryl Warinner
//
// Parm: HWND     hwnd;    -- The user interface window handle
//       USHORT   msg;     -- The message
//       MPARAM   mp1;     -- message specific
//       MPARAM   mp2;     -- message specific
//
// Desc: This window procedure processes the Mark string dialog box.
//
//*****************************************************************************

MRESULT EXPENTRY VSBLogMarkDlgProc (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
   {

   switch (msg)
      {
      //-----------------------------------------------------------------------
      case WM_INITDLG:
         WinSendDlgItemMsg (hwnd,
                            IDC_EDITBOX,
                            EM_SETTEXTLIMIT,
                            MPFROM2SHORT (MAX_LEN_LOGMESSAGE, 0),
                            NULL);
         return (FALSE);
         break;

      //-----------------------------------------------------------------------
      case WM_CONTROL:
         //--------------------------------------------------------------------
         // Currently, this code doesn't do anything.  But it is provided
         //    in case you want to add something....
         //--------------------------------------------------------------------
         switch (SHORT1FROMMP (mp1))            // Control ID
            {
            case IDC_EDITBOX:
               switch (SHORT2FROMMP (mp1))      // Notification Code
                  {
                  default:
                     return (WinDefDlgProc (hwnd, msg, mp1, mp2));
                     break;
                  }
               break;

            default:
               return (WinDefDlgProc (hwnd, msg, mp1, mp2));
               break;
            }

         return ((MRESULT) 0);
         break;

      //-----------------------------------------------------------------------
      case WM_COMMAND:
         switch (COMMANDMSG (&msg)->cmd)        // Control ID
            {
            case IDC_MARKADD:
               // Get the string (we were already locked)
               WinQueryWindowText (WinWindowFromID (hwnd, IDC_EDITBOX),
                                   MAX_LEN_LOGMESSAGE,
                                   szMarkMessage);
               // Fall Through

            case IDC_MARKCANCEL:
               // Kill the dialog box
               WinDismissDlg (hwnd, (COMMANDMSG (&msg)->cmd));
               break;
            }

         return ((MRESULT) 0);
         break;

      //-----------------------------------------------------------------------
      default:
         return (WinDefDlgProc (hwnd, msg, mp1, mp2));
         break;
      }
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
