/*********************************************************************/
/*                                                                    */
/* (C) Copyright IBM Corporation 1992                                 */
/*                                                                    */
/*  MODULE:       MISC.C                                              */
/*                                                                    */
/*  VERSION:      Digital Audio Sample Application for the            */
/*                M-Control Program /2   Version 2.0                  */
/*                using Microsoft Windows 3.0 ( TM )                  */
/*                                                                    */
/*  FUNCTIONS:    AddRecordBuffer()                                   */
/*                AudioSeek()                                         */
/*                CloseAll()                                          */
/*                ResetRiffHeader()                                   */
/*                SendNextBuffer()                                    */
/*                SetTitle()                                          */
/*                SetPosition()                                       */
/*                StartTimer()                                        */
/*                StopTimmer()                                        */
/*                                                                    */
/*  DESCRIPTION:  Miscellaneous processes used by multiple modules    */
/*                                                                    */
/*  M-MOTION FUNCTIONS USED:                                          */
/*                waveInAddBuffer                                     */
/*                waveInClose                                         */
/*                waveInGetErrorText                                  */
/*                waveInPrepareHeader                                 */
/*                waveInReset                                         */
/*                waveOutClose                                        */
/*                waveOutGetErrorText                                 */
/*                waveOutPrepareHeader                                */
/*                waveOutReset                                        */
/*                waveOutWrite                                        */
/**********************************************************************/
#include  "AUDIO.H"                    // sample appliaction header file

/**********************************************************************/
/* Function:     AddRecordBuffer()                                    */
/*                                                                    */
/* Called From:  MainWinProc()                                        */
/*                                                                    */
/* Calls to:     None                                                 */
/*                                                                    */
/* Description:  For Recording....                                    */
/*               Allocate & lock memory for wave header (WaveHdr)     */
/*               Prepare buffer for input                             */
/*               Sends buffer to input device                         */
/*                                                                    */
/**********************************************************************/
VOID AddRecordBuffer()
{
 char      Text[80];
 char      ErrMsgText[120];
 WORD      rc;                          // return code
 char FAR  *BufferPtr;
 HANDLE    WaveHdrHandle;
 LPWAVEHDR WaveHdrPtr;

 WaveHdrHandle = GlobalAlloc( GMEM_MOVEABLE | GMEM_SHARE,
                              sizeof( WAVEHDR ) + BUFFSIZE );
 WaveHdrPtr   = ( LPWAVEHDR ) GlobalLock( WaveHdrHandle );

 BufferPtr = ( char far * ) WaveHdrPtr + sizeof( WAVEHDR );

 WaveHdrPtr->lpData   = ( LPSTR ) BufferPtr;
 WaveHdrPtr->dwBufferLength  = ( DWORD ) BUFFSIZE;
 WaveHdrPtr->dwBytesRecorded = 0L;
 WaveHdrPtr->dwFlags  = 0L;      /* flag used to tell when end of buffer */
 WaveHdrPtr->lpNext   = NULL;
 WaveHdrPtr->reserved = NULL;

 /* prepare waveform input data block */

 rc = waveInPrepareHeader( hWaveIn, ( LPWAVEHDR ) WaveHdrPtr,
                           sizeof( WAVEHDR ) );
 if  ( rc )
    { waveInGetErrorText( rc, ( LPSTR ) Text, sizeof( Text ) );
      LoadString ( hInst, IDS_WAVEINPREPHDR, ErrMsgText, STRBUFLEN );
      MessageBox( hwndFrame, Text, ErrMsgText,
                  MB_OK | MB_APPLMODAL | MB_ICONEXCLAMATION );
      return;
    }

 rc = waveInAddBuffer( hWaveIn, ( LPWAVEHDR ) WaveHdrPtr, sizeof( WAVEHDR ) );
 if  ( rc )
    { waveInGetErrorText( rc, ( LPSTR ) Text, sizeof( Text ) );
      LoadString ( hInst, IDS_WAVEINADDBUFF, ErrMsgText, STRBUFLEN );
      MessageBox( hwndFrame, Text, ErrMsgText,
                  MB_OK | MB_APPLMODAL | MB_ICONEXCLAMATION );
      GlobalUnlock( WaveHdrHandle );         // Free the Resources
      GlobalFree( WaveHdrHandle );
    }

 return;
 }


/**********************************************************************/
/* Function:     AudioSeek()                                          */
/*                                                                    */
/* Called From:  MainWinProc()                                        */
/*               GetWaveFormat()                                      */
/*                                                                    */
/* Calls to:     SetPosition()                                        */
/*                                                                    */
/* Description:  Seek to the beginning of the audio file              */
/**********************************************************************/
VOID AudioSeek()
{
 if  ( WaveFile != NOFILE )
    { if  ( hWaveOut )
         { Playing = FALSE;
           waveOutReset( hWaveOut );
           }
      lseek( WaveFile, Position + RiffHdrLength, SEEK_SET );
      SetPosition();
      }
 return;
 }


/**********************************************************************/
/* Function:     CloseAll()                                           */
/*                                                                    */
/* Called From:  MainWinProc()                                        */
/*               GetWaveFormat()                                      */
/*               ButtonsProc()                                        */
/*                                                                    */
/* Calls to:     SetTitle()                                           */
/*                                                                    */
/* Description:  Close the opened file,                               */
/*               Reset the wave input and output devices              */
/*               Reset the window caption                             */
/**********************************************************************/
VOID CloseAll()
{
 if ( WaveFile != NOFILE )              // IF the  File is Open
    { close( WaveFile );                //    Close It
      WaveFile = -1;
    }

 if  ( hWaveOut )                       // IF an Open Output Device
    { waveOutReset( hWaveOut );         //    Make Sure It's Reset
      waveOutClose( hWaveOut );
      hWaveOut = NULL;
    }

 if  ( hWaveIn )                        // IF an Open Input Device
    { waveInReset( hWaveIn );           //    Make Sure It's Reset
    waveInClose( hWaveIn );             //    Close It
    hWaveIn = NULL;
    }

 FileName[0] = 0;
 FileExist   = FALSE;
 OldOpenFile = FALSE;
 Recording   = FALSE;
 SetTitle();
 return;
 }

/**********************************************************************/
/* Function:     ResetRiffHeader()                                    */
/*                                                                    */
/* Called From:  MainWinProc()                                        */
/*                                                                    */
/* Calls to:     None                                                 */
/*                                                                    */
/* Description:  Reset the RIFF header length and data chunk fields   */
/*               Determine the length of the wave file                */
/*               Seek to beginning of file                            */
/*               Read the wave file                                   */
/*               Set the length of the RIFFHEADER in the RIFFHEADER   */
/*               Set the length of the data chunk in the RIFFHEADER   */
/*               Seek to the beginning of the wave file               */
/*               Write the RiffHeader in the wave file                */
/*                                                                    */
/**********************************************************************/
VOID ResetRiffHeader()
{
 LONG CurrentPosition;                  // Current position in the file
 LONG FileSize;                         // size of RIFF file

 CurrentPosition = lseek( WaveFile, 0L, SEEK_CUR );
 FileSize = lseek( WaveFile, 0L, SEEK_END );

 // Check if file size is odd, if so, pad with a zero byte to
 // word align for RIFF file compatibility.

 if (FileSize & 0x1L)
    {
    char zero = 0;
    // Data length is odd - pad data with extra zero byte and increment
    write( WaveFile, &zero, 1);
    }

 lseek( WaveFile, 0L, SEEK_SET );
 read( WaveFile, ( char * ) &RiffHeader, sizeof( RiffHeader ) );

 RiffHeader.RiffLength = FileSize - ( sizeof ( RiffHeader.RiffName ) +
                                      sizeof ( RiffHeader.RiffLength ) );

 RiffHeader.DataChunkLength = FileSize - ( sizeof( RiffHeader ) );

 lseek( WaveFile, 0L, SEEK_SET );
 write( WaveFile, ( char * ) &RiffHeader, sizeof( RiffHeader ) );
 lseek( WaveFile, CurrentPosition, SEEK_SET );
 return;
 }


/**********************************************************************/
/* Function:     SendNextBuffer()                                     */
/*                                                                    */
/* Called From:  MainWinProc()                                        */
/*               Play()                                               */
/*                                                                    */
/* Calls to:     None                                                 */
/*                                                                    */
/* Description:  Allocates next buffer space                          */
/**********************************************************************/
WORD SendNextBuffer()
{
 char      ErrMsgText[120];
 char      Text[80];                    // Error message text
 WORD      rc;                          // Return code value
 WORD      BytesRead;                   // Number of bytes read
 char FAR  *BufferPtr;                  // Pointer to start of buffered area
 HANDLE    WaveHdrHandle;               // Handle to allocated memory
 LPWAVEHDR WaveHdrPtr;                  // Pointer to header area

 /*  Check for the output device */

 if  ( hWaveOut == NULL )
      return( 0 );

 /* Allocate & lock moveable/shared memroy for the data block */

 WaveHdrHandle = GlobalAlloc( GMEM_MOVEABLE | GMEM_SHARE,
                              sizeof( WAVEHDR ) + BUFFSIZE );
 WaveHdrPtr    = ( LPWAVEHDR ) GlobalLock( WaveHdrHandle );

 /* Skip the header and point to the buffer */

 BufferPtr = ( char far * ) WaveHdrPtr + sizeof( WAVEHDR );

 /* Read the data from the buffer */

 BytesRead = _lread( WaveFile, BufferPtr, BUFFSIZE );

 /* If at End-Of-File, free the resources */

 if  ( BytesRead <= 0 )
    { GlobalUnlock( WaveHdrHandle );
      GlobalFree( WaveHdrHandle );
      return( -1);
    }

 /* Initialize the wave header */

 WaveHdrPtr->lpData = ( LPSTR ) BufferPtr;
 WaveHdrPtr->dwBufferLength  = BytesRead;
 WaveHdrPtr->dwBytesRecorded = 0L;
 WaveHdrPtr->dwFlags         = 0L;
 WaveHdrPtr->lpNext          = NULL;
 WaveHdrPtr->reserved        = NULL;

 /* If this is the last buffer, set the flag to 1 */
 if  ( BytesRead < BUFFSIZE )
      WaveHdrPtr->dwUser = 1L;
 else
      WaveHdrPtr->dwUser = 0;

 /* Check if at the end of the buffer */

 CurPlayPos = lseek( WaveFile, 0L, SEEK_CUR );
 if (CurPlayPos == EndofFile)
      WaveHdrPtr->dwUser = 1L;

 /* Prepare waveform output data block */

 rc = waveOutPrepareHeader( hWaveOut, ( LPWAVEHDR ) WaveHdrPtr,
                            sizeof( WAVEHDR ) );
 if  ( rc )
    { waveOutGetErrorText( rc, ( LPSTR ) Text, sizeof( Text ));
      LoadString( hInst, IDS_WAVEOUTPREPHDR, ErrMsgText, STRBUFLEN );
      MessageBox( hwndFrame, Text, ErrMsgText,
                  MB_OK | MB_APPLMODAL | MB_ICONEXCLAMATION );
      return( rc );
    }

 /* Write data to output device */

 rc = waveOutWrite( hWaveOut, ( LPWAVEHDR ) WaveHdrPtr, sizeof( WAVEHDR ) );
 if  ( rc )
    {
    waveOutGetErrorText( rc, ( LPSTR ) Text, sizeof( Text ) );
    LoadString ( hInst, IDS_WAVEOUTWRITE, ErrMsgText, STRBUFLEN );
    MessageBox( hwndFrame, Text, ErrMsgText,
                MB_OK | MB_APPLMODAL | MB_ICONEXCLAMATION );
    GlobalUnlock( WaveHdrHandle );      // Free the Resources
    GlobalFree( WaveHdrHandle );
    }

 return( rc );
 }

/**********************************************************************/
/* Function:     SetTitle()                                           */
/*                                                                    */
/* Called From:  MainWinProc()                                        */
/*               NewProc()                                            */
/*               Play()                                               */
/*               GetWaveFormat()                                      */
/*               CloseAll()                                           */
/*                                                                    */
/* Calls to:     None                                                 */
/*                                                                    */
/* Description:  Sets the caption for the parent window title         */
/*               to the opened audio file name.                       */
/**********************************************************************/
VOID SetTitle()
{
 char  WindowText[120];
 char  CurrentName[32];
 WORD  i;

 strcpy( WindowText, TitleText );
 if  ( strlen( FileName ) != 0 )
    { strcat( WindowText, " -- " );
      strcpy( CurrentName, FileName );
      i = 0;
      while  ( ( CurrentName[i] != '.' ) &&           // Strip File Extension
               ( CurrentName[i] != 0 ) )
              i++;
      CurrentName[i] = 0;
      strupr( CurrentName );
      strcat( WindowText, CurrentName );
      }

 SetWindowText( hwndFrame, ( LPSTR ) WindowText );
 InvalidateRect( hwndFrame, NULL, TRUE );
 UpdateWindow( hwndFrame );
 return;
 }

/**********************************************************************/
/* Function:     SetPosition()                                        */
/*                                                                    */
/* Called From:  MainWinProc()                                        */
/*               NewProc()                                            */
/*               Play()                                               */
/*               GetWaveFormat()                                      */
/*               CloseAll()                                           */
/*                                                                    */
/* Calls to:     None                                                 */
/*                                                                    */
/* Description:  Sets the time text in the Wave Audio dialog box      */
/*               in the TIME __:__:__ field                           */
/**********************************************************************/
VOID SetPosition()
{
 char   Text[16];
 LONG   Divider;
 WORD   Minutes;
 WORD   Seconds;
 WORD   MilliSeconds;
 LONG   RealPos;

 RealPos = Position + FileStart;
 Divider = ( LONG ) WaveFormat.wf.nSamplesPerSec;
 if  ( Divider < 1000L )                // Never Allow Divide By Zero!
    return;

 Seconds = ( WORD ) ( RealPos / Divider );
 if  ( Seconds >= 60 )
    { Minutes = Seconds / 60;
      Seconds = Seconds % 60 ;
    }
 else
      Minutes = 0;

 MilliSeconds = (WORD) (( RealPos % Divider ) * 1000L / Divider);

 if  ( Minutes )
    wsprintf( Text, "%1.1d:%2.2d:%3.3d", Minutes, Seconds, MilliSeconds );
 else
    wsprintf( Text, "%2.1d:%3.3d", Seconds, MilliSeconds );
 SetDlgItemText( hwndButtonsDlg, ID_POSITION, Text );

 return;
 }

/**********************************************************************/
/* Function:     StartTimer()                                         */
/*                                                                    */
/* Called From:  ButtonsProc()                                        */
/*               Play()                                               */
/*                                                                    */
/* Calls to:     None                                                 */
/*                                                                    */
/* Description:  Start timer with an interval of 110 milliseconds     */
/**********************************************************************/
VOID StartTimer()
{

 if  ( TimerOn == FALSE )
    { SetTimer( hwndFrame, 1, TIMER_INTERVAL, NULL );
      TimerOn = TRUE;
    }
 return;
 }

/**********************************************************************/
/* Function:     StopTimer()                                          */
/*                                                                    */
/* Called From:  MainWndProc()                                        */
/*               ButtonsProc()                                        */
/*                                                                    */
/* Calls to:     None                                                 */
/*                                                                    */
/* Description:  Stops timer                                          */
/**********************************************************************/
VOID StopTimer()
{
 if  ( TimerOn == TRUE )
    { KillTimer( hwndFrame, 1 );
      TimerOn = FALSE;
    }
 return;
 }
