// WINDOWS version
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBCmd.c          (VSB Command Processing)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains functions that process COMMANDs, ACKs,
//          and RESPONSEs for the VSB UI.  It also contains functions
//          that initiate commands back to the MVPC.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#include <windows.h>

#include "mvpcwin.h"       // Include for the MVPC object
#include "vsbwin.h"        // Public  includes
#include "vsb2win.h"       // Private includes


//**************************************************************************<f>
// Func: ProcessCOMMAND                                Author: Steve Henson
//
// Desc: Processes COMMANDS from the MVPC
//
// Retn: Returns through WinDefWindowProc ()
//
//*****************************************************************************
VOID ProcessCOMMAND (HWND hWnd, DWORD lparam)
   {
   HANDLE    hvsbdata;
   LPVSBDATA lpvsbdata;
   LPMVPCCMD lpmvpccmd;

   // address this instance's data area
   hvsbdata = GetWindowWord (hWnd, VSB_WWOFF_PVSB);
   lpvsbdata = (LPVSBDATA)GlobalLock (hvsbdata);

   // Convert lparam to the MVPCCMD ptr:
   lpmvpccmd = (LPMVPCCMD)GlobalLock (LOWORD (lparam));

   // switch on the specific command
   switch (lpmvpccmd->command.wCommand)
      {
      case VSB_CMD_ADVISEUSER: // App wants to know about user interactions
         lpvsbdata->hWndAdviseUser = (HWND)LOWORD (lpmvpccmd->data.dwCmdParam1);
         break;

      case VSB_CMD_SETPOINTERS: // set pointers
         lpvsbdata->hptrActive = (HWND)LOWORD (lpmvpccmd->data.dwCmdParam1);
         lpvsbdata->hptrBusy   = (HWND)LOWORD (lpmvpccmd->data.dwCmdParam2);
         break;
      }

   GlobalUnlock (LOWORD (lparam));
   GlobalUnlock (hvsbdata);
   }


//**************************************************************************<f>
// Func: ProcessACK                                   Author: Steve Henson
//
// Desc: This function processes any ACK messages received by the VSB.
//       For instance, ACKs from the Cmd...() functions listed below.
//
//*****************************************************************************

VOID ProcessACK (HWND hWnd, WORD msg, WORD wparam, DWORD lparam)
   {
   LPMVPCCMD lpmvpccmd;
   WORD      wResponse;

   lpmvpccmd = (LPMVPCCMD)GlobalLock (LOWORD (lparam));        // set up addressing to parameter block
   wResponse   = lpmvpccmd->command.wResponse;    // get response code
   GlobalUnlock (LOWORD (lparam));

   if (wResponse == MRESP_NORESPONSEPENDING)
      {
      MVPCDestroyCmd (LOWORD (lparam), (HWND)wparam, msg);
      }
   }


//**************************************************************************<f>
// Func: ProcessRESP                                  Author: Steve Henson
//
// Desc: This function processes any RESPONSEs received by the VSB.
//       For instance, some of the Cmd...() functions listed below.
//
//*****************************************************************************

VOID ProcessRESP (HWND hWnd, WORD msg, WORD wparam, DWORD lparam)
   {
   MVPCDestroyCmd (LOWORD (lparam), (HWND)wparam, msg);
   }


//**************************************************************************<f>
// Func: CmdGetToken                                  Author: Steve Henson
//
// Desc: This function sends an ACTIVATE command to the MVPC to allow
//       it to capture the FOCUS.
//
//*****************************************************************************

BOOL CmdGetToken (HWND hWndMVPC, HWND hWndVSB)
   {
   HANDLE    hmvpccmd;
   LPMVPCCMD lpmvpccmd;               // Ptr to the MVPC Command structure
   BOOL      bReturn = FALSE;         // Return code for this function

   if (hmvpccmd = MVPCCreateCmd (MVPC_CMD_ACTIVATE))
      {
      if (lpmvpccmd = (LPMVPCCMD)GlobalLock (hmvpccmd))
         {
         lpmvpccmd->data.dwCmdParam1 = (DWORD)TRUE;
         GlobalUnlock (hmvpccmd);

         PostMessage (hWndMVPC, msgMVPC_COMMAND, hWndVSB, (DWORD)hmvpccmd);
         bReturn = TRUE;
         }
      else
         {
         GlobalFree (hmvpccmd);
         }
      }

   return (bReturn);
   }


//**************************************************************************<f>
// Func: CmdSeek                                      Author: Steve Henson
//
// Desc: This function sends a STILLFRAME command to the MVPC.
//
//*****************************************************************************

BOOL CmdSeek (HWND hWndMVPC, HWND hWndVSB, LONG lFrame)
   {
   HANDLE    hmvpccmd;
   LPMVPCCMD lpmvpccmd;               // Ptr to the MVPC Command structure
   BOOL      bReturn = FALSE;         // Return code for this function

   if (hmvpccmd = MVPCCreateCmd (MVPC_CMD_STILLFRAME))
      {
      if (lpmvpccmd = (LPMVPCCMD)GlobalLock (hmvpccmd))
         {
         lpmvpccmd->data.dwCmdParam1 = (DWORD)lFrame;
         GlobalUnlock (hmvpccmd);

         PostMessage (hWndMVPC, msgMVPC_COMMAND, hWndVSB, (DWORD)hmvpccmd);
         bReturn = TRUE;
         }
      else
         {
         GlobalFree (hmvpccmd);
         }
      }

   return (bReturn);
   }


//**************************************************************************<f>
// Func: CmdStep                                      Author: Steve Henson
//
// Desc: This function sends a STEPFORWARD or STEPBACKWARD command to the
//       MVPC object.
//
//*****************************************************************************

BOOL CmdStep (HWND hWndMVPC, HWND hWndVSB, WORD wDirection)
   {
   HANDLE   hmvpccmd;
   BOOL     bReturn = FALSE;        // Return code for this function

   if (hmvpccmd = MVPCCreateCmd ((wDirection == FORWARD) ?
                                  MVPC_CMD_STEPFORWARD : MVPC_CMD_STEPBACKWARD))
      {
      PostMessage (hWndMVPC, msgMVPC_COMMAND, hWndVSB, (DWORD)hmvpccmd);
      bReturn = TRUE;
      }

   return (bReturn);
   }


//**************************************************************************<f>
// Func: CmdPlayFromTo                                Author: Steve Henson
//
// Desc: This function sends a PLAYFROMTO command to the MVPC object.
//
//*****************************************************************************

BOOL CmdPlayFromTo (HWND hWndMVPC, HWND hWndVSB, LONG lFrom, LONG lTo,
                    BOOL bNoReMap)
   {
   HANDLE    hmvpccmd;
   BOOL      bReturn = FALSE;        // Return code for this function
   LPMVPCCMD lpmvpccmd;               // Ptr to the MVPC Command structure

   if (hmvpccmd = MVPCCreateCmd (MVPC_CMD_PLAYFROMTO))
      {
      if (lpmvpccmd = (LPMVPCCMD)GlobalLock (hmvpccmd))
         {
         lpmvpccmd->data.dwCmdParam1 = (DWORD)lFrom;
         lpmvpccmd->data.dwCmdParam2 = (DWORD)lTo;
         lpmvpccmd->data.dwCmdParam3 = (DWORD)bNoReMap;
         GlobalUnlock (hmvpccmd);

         PostMessage (hWndMVPC, msgMVPC_COMMAND, hWndVSB, (DWORD)hmvpccmd);
         bReturn = TRUE;
         }
      else
         {
         GlobalFree (hmvpccmd);
         }
      }

   return (bReturn);
   }


//**************************************************************************<f>
// Func: CmdStop                                      Author: Steve Henson
//
// Desc: This function sends a STOP command to the MVPC object.
//
//*****************************************************************************

BOOL CmdStop (HWND hWndMVPC, HWND hWndVSB)
   {
   HANDLE   hmvpccmd;
   BOOL     bReturn = FALSE;        // Return code for this function

   if (hmvpccmd = MVPCCreateCmd (MVPC_CMD_STOP))
      {
      PostMessage (hWndMVPC, msgMVPC_COMMAND, hWndVSB, (DWORD)hmvpccmd);
      bReturn = TRUE;
      }

   return (bReturn);
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
