// WINDOWS version
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBFunc.c         (VSB Misc. Functions)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains miscellaneous functions used by the
//          Videodisc Scroll Bar (VSB) DLL.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#include <windows.h>

#include "mvpcwin.h"       // Include for the MVPC object
#include "vsbwin.h"        // Public  includes
#include "vsb2win.h"       // Private includes


//**************************************************************************<f>
// Func: CreateVSBDATA                                Author: Steve Henson
//
// Parm: none
//
// Desc: Allocates and sets to NULL the VSBDATA structure for this
//       User Interface
//
// Retn: NULL  = An error occurred while trying to create the structure
//       !NULL = The Ptr to the VSBDATA structure
//
//*****************************************************************************

LPVSBDATA CreateVSBDATA ()
   {
   HANDLE    hvsbdata;
   LPVSBDATA lpvsbdata = (LPVSBDATA) NULL;   // VSBData Pointer (returned)

   if (hvsbdata = GlobalAlloc (GMEM_FIXED |
                               GMEM_ZEROINIT |
                               GMEM_NODISCARD,
                               sizeof (VSBDATA)))
      {
      if (lpvsbdata = (LPVSBDATA)GlobalLock (hvsbdata)) // lock it down
         {
         lpvsbdata->wData = (WORD)sizeof (VSBDATA);   // Set the size field
         }
      }

   return (lpvsbdata);
   }


//**************************************************************************<f>
// Func: PosSlider                                    Author: Steve Henson
//
// Parm: LONG  lLow;       -- The lower boundary of the frame range
//       LONG  lHigh;      -- The upper boundary of the frame range
//       LONG  lCurrent;   -- The current frame number
//
// Desc: Unfortunately, Scroll bars in WINDOWS are only allowed a range
//       from 1 to 32767 (signed int).  Since we can easily have more
//       than that in a frame range for videodisc, we have to map our
//       frame range into a signed int.  This function performs that
//       mapping.
//
// Retn: The scroll bar position (from 1 to 32767) for the current frame.
//
//*****************************************************************************

WORD PosSlider (LONG lLow, LONG lHigh, LONG lCurrent)
   {
   LONG   lOffset;      // The offset of the current frame from the beginning
   LONG   lRange;       // The range of frames
   WORD   posSlider;    // The position (returned)
   int    sSBRange;

   //--------------------------------------------------------------------------
   // Here's how it works:
   //    the FIRST and LAST positions on the scroll bar are saved for the
   //    Beginning and End of the range.  All other values are mapped to
   //    some intermediate position.  This allows us to step backward from
   //    frame #2 to #1, likewise from 53999 to 54000.
   //--------------------------------------------------------------------------

   lRange = lHigh - lLow;

   if (lCurrent == lLow)
      posSlider = SB_LOW;
   else if (lCurrent == lHigh)
      if (lRange & 0xFFFF8000)   // If range > 32767, then it won't matter
         posSlider = SB_RANGE;
      else
         posSlider = (WORD)lRange;
   else
      {
      lOffset = lCurrent - lLow;
      if (lOffset < 0)
         lOffset = 0;      // Or else we would set a negative position

      if (lRange == 0)
         lRange = 1;       // Or else we would get a DIVIDE BY ZERO ERROR!

      if (lRange & 0xFFFF8000)   // If range > 32767, then it won't matter
         sSBRange = SB_RANGE;
      else
         sSBRange = (int) lRange;

      //-----------------------------------------------------------------------
      // Note: we multiply with a ULONG before dividing to minimize
      //       round-off error.
      //-----------------------------------------------------------------------
      posSlider = (WORD)((((DWORD)lOffset) * sSBRange) / lRange) + SB_LOW;

      //-----------------------------------------------------------------------
      // Error caused by the following round-off should not exceed 2 frames
      //    on each end over a maximum frame range of 108,000 frames (CLV).
      //    Given an SB_RANGE of 32767 (& 108,000 frames):
      //       Position 2 will have 5 frames (frames 2, 3, 4, 5, & 6).
      //       Position 32766 will have 5 frames.
      //       All others will have 3 frames (frames 7, 8, & 9).
      //-----------------------------------------------------------------------
      if (posSlider == SB_LOW)
         posSlider++;
      else if (posSlider == SB_HIGH)
         posSlider--;
      }

   return (posSlider);
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
