// WINDOWS version
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBLog.c          (VSB Log Support)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains the functions and variables necessary
//          for supporting logging to the VSB DLL.  This logging
//          capability may prove useful for other Socketable User Interfaces
//          developed for the Multimedia Videodisc Player Control Object.
//
//          See VSBLOG.H for DEFINEs, TYPEDEFs, and FUNCTION PROTOTYPES.
//          See VSB.RC for Log Messages.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#include <windows.h>

#include <stdlib.h>     // ultoa ()
#include <string.h>     // strnset ()

#include "mvpcwin.h"       // Include for the MVPC object
#include "vsbwin.h"        // Public  includes
#include "vsb2win.h"       // Private includes

///////////////////////////////////////////////////////////////////////////////
#ifndef DEBUG

VOID LogInitialize (VOID) {}
VOID LogTerminate (VOID) {}
VOID LogCreate (HWND hWndVSB) {}
VOID LogDestroy (HWND hWndVSB) {}
VOID cdecl LogMessage (HWND hWndVSB, WORD wLogMsgID, ...) {}

// This allows the exported functions to be found....
LONG FAR PASCAL  VSBLogSubclassWndProc (HWND hWnd, WORD msg,
                                        WORD wparam, DWORD lparam) {return 0L;}
LONG FAR PASCAL  VSBLogDlgProc         (HWND hWnd, WORD msg,
                                        WORD wparam, DWORD lparam) {return 0L;}
LONG FAR PASCAL  VSBLogMarkDlgProc     (HWND hWnd, WORD msg,
                                        WORD wparam, DWORD lparam) {return 0L;}

#else

//*****************************************************************************
// Global Variables used by this module only
//*****************************************************************************

static DWORD      ulLine;
static char       szLogMask [MAX_LEN_LOGMASK];
static char       szTopOfLog [MAX_LEN_LOGMESSAGE];
static char       szMarkMessage [MAX_LEN_LOGMESSAGE];


//**************************************************************************<f>
// Func: LogInitialize                                Author: Daryl Warinner
//
// Desc: This function initializes the log semaphore and loads some
//       required strings from the resource file.
//
//       NOTE: Throughout this module, I lock the semaphore before
//             performing string manipulations.  The string buffers are
//             shared between all instances of VSBs, and it is necessary
//             to prevent multiple VSBs from altering the strings
//             simultaneously.  This also prevents re-entrancy to wsprintf
//             and other functions that are not re-entrant.
//
// Retn: VOID
//
//*****************************************************************************


VOID LogInitialize ()
   {

   if (!szLogMask[0])
      LoadString (hmodDLL, IDS_LOG_MSGMASK,
                  szLogMask,
                  sizeof (szLogMask));

   if (!szTopOfLog[0])
      LoadString (hmodDLL, IDS_LM_TOPOFLOG,
                  szTopOfLog,
                  sizeof (szTopOfLog));
   }


//**************************************************************************<f>
// Func: LogTerminate                                 Author: Daryl Warinner
//
// Desc: This function is called when the VSB is terminating.  It does
//       nothing now, but it may in the future....
//
// Retn: VOID
//
//*****************************************************************************

VOID LogTerminate ()
   {

   // Nothing for now....
   }


//**************************************************************************<f>
// Func: LogCreate                                    Author: Daryl Warinner
//
// Desc: This function creates an instance of a log window for a VSB.
//       It allocates a LOGINFO structure and saves a pointer to it in
//       the VSBs Window Word data structure.  It also subclasses the
//       "real" scroll bar to trap Right Button Double Clicks.
//
// Retn: VOID
//
//*****************************************************************************

VOID LogCreate (HWND hWndVSB)
   {
   HANDLE            hvsbdata;
   LPVSBDATA         lpvsbdata;
   HANDLE            hLogInfo;
   LPLOGINFO         lpLogInfo;
   LOGCREATEPARAMS   logcreateparams;
   FARPROC           lpLogDlgProc;

   if (!(hvsbdata = GetWindowWord (hWndVSB, VSB_WWOFF_PVSB)))
      return;
   if (!(lpvsbdata = (LPVSBDATA)GlobalLock (hvsbdata)))
      return;

   // Allocate the LogInfo structure and attach it to the Window word struct
   if (hLogInfo = GlobalAlloc (GMEM_MOVEABLE |
                               GMEM_ZEROINIT |
                               GMEM_NODISCARD,
                               sizeof (LOGINFO)))
      {
      if (lpLogInfo = (LPLOGINFO)GlobalLock (hLogInfo))
         {
         lpvsbdata->hLogInfo = hLogInfo;

         // Subclass the Scroll Bar to intercept calls for Log Information
         (DWORD)lpLogInfo->pfnSBWndProc = SetWindowLong (lpvsbdata->hWndScrollBar,
                                                        GWL_WNDPROC,
                                                 (DWORD)VSBLogSubclassWndProc);

         logcreateparams.wData   = sizeof (logcreateparams);
         logcreateparams.wType   = NULL;
         logcreateparams.hWndVSB  = hWndVSB;
         logcreateparams.wPlayer = lpvsbdata->wPlayer;

         lpLogDlgProc = MakeProcInstance ((FARPROC)VSBLogDlgProc, hmodDLL);
         lpLogInfo->hWndVSBLOG = CreateDialogParam (hmodDLL, "IDD_VSBLOG",
                                                   GetDesktopWindow (),
                                                   lpLogDlgProc,
                                                   (DWORD)(LPVOID)&logcreateparams);

         lpLogInfo->hWndLOGBOX = GetDlgItem (lpLogInfo->hWndVSBLOG, IDC_LOGBOX);
         GlobalUnlock (lpvsbdata->hLogInfo);
         }
      else
      lpvsbdata->hLogInfo = (HANDLE) NULL;
      }
   else
      lpvsbdata->hLogInfo = (HANDLE) NULL;

   GlobalUnlock (hvsbdata);
   }


//**************************************************************************<f>
// Func: LogDestroy                                   Author: Daryl Warinner
//
// Desc: This function is called when the VSB is being destroyed.  It
//       frees the LOGINFO structure and removes the references to it
//       from the VSBs window word data structure.  It also UN-subclasses
//       the "real" scroll bar.
//
// Retn: VOID
//
//*****************************************************************************

VOID LogDestroy (HWND hWndVSB)
   {
   HANDLE       hvsbdata;
   LPVSBDATA    lpvsbdata;
   LPLOGINFO    lpLogInfo;

   if (!(hvsbdata = GetWindowWord (hWndVSB, VSB_WWOFF_PVSB)))
      return;
   if (!(lpvsbdata = (LPVSBDATA)GlobalLock (hvsbdata)))
      return;

   lpLogInfo = (LPLOGINFO)GlobalLock (lpvsbdata->hLogInfo);

   if (lpLogInfo)
      {
      if (lpvsbdata->hWndScrollBar)
         {
         SetWindowLong (lpvsbdata->hWndScrollBar, GWL_WNDPROC,
                        (DWORD)lpLogInfo->pfnSBWndProc);
         }
      lpLogInfo->pfnSBWndProc = (PFNWP) NULL;

      DestroyWindow (lpLogInfo->hWndVSBLOG);
      lpLogInfo->hWndVSBLOG = (HWND) NULL;
      lpLogInfo->hWndLOGBOX = (HWND) NULL;

      GlobalUnlock (lpvsbdata->hLogInfo);
      GlobalFree (lpvsbdata->hLogInfo);

      lpvsbdata->hLogInfo = (HANDLE) NULL;
      }
   GlobalUnlock (hvsbdata);
   }


//**************************************************************************<f>
// Func: LogMessage                                Author: Daryl Warinner
//
// Desc: This function places a log message into the log list box.
//       It supports a variable number of parameters, and loads the
//       actual string from the Resource file.  To add log messages,
//       you must:
//
//          1) Define the string in VSB.RC
//          2) Define the string ID in VSBLOG.H
//          3) Place a call to LogMessage () in the code.
//
// Retn: VOID
//
//*****************************************************************************

VOID cdecl LogMessage (HWND hWndVSB, WORD wLogMsgID, ...)
   {
   static char  szLoadString [MAX_LEN_LOADSTRING];
   static char  szLogMessage [MAX_LEN_LOGMESSAGE];
   static char  szLineNumber [MAX_LEN_LINESTRING];
   static char  szLogEntry [MAX_LEN_LOGENTRY];
   LPSTR        pArguments;
   HANDLE       hvsbdata;
   LPVSBDATA    lpvsbdata;
   LPLOGINFO    lpLogInfo;
   int          itemindex;

   // Retrieve the Window Word data structure
   if (!(hvsbdata = GetWindowWord (hWndVSB, VSB_WWOFF_PVSB)))
      return;
   if (!(lpvsbdata = (LPVSBDATA)GlobalLock (hvsbdata)))
      return;

   lpLogInfo = (LPLOGINFO)GlobalLock (lpvsbdata->hLogInfo);
   if (lpLogInfo)
      {
      // Load the message string from the resource file
      LoadString (hmodDLL, wLogMsgID, szLoadString, sizeof (szLoadString));

      // Copy the message (w/ arguments) into a temporary buffer
      pArguments = (LPSTR)&wLogMsgID + sizeof (wLogMsgID);
      wvsprintf (szLogMessage, szLoadString, pArguments);

      // Convert the current line number into an ASCII string
      ulLine++;
      ulLine &= 0x0000FFFFL;                       // Lop off at 65535
      ultoa (ulLine, szLineNumber, 10);

      // Copy the line number and message string into the buffer
      wsprintf (szLogEntry, szLogMask, (LPSTR)szLineNumber, (LPSTR)szLogMessage);

      // Insert the buffered string into the list box
      itemindex = (int) SendMessage (lpLogInfo->hWndLOGBOX,
                                       LB_INSERTSTRING,
                                       -1,
                                       (DWORD)(LPVOID)szLogEntry);

      // Make sure the string is visible
      SendMessage (lpLogInfo->hWndLOGBOX, LB_SETTOPINDEX,
                  itemindex, NULL);

      if (itemindex > MAX_LOGMESSAGES)
         {
         SendMessage (lpLogInfo->hWndLOGBOX,
                      LB_DELETESTRING,
                      0,
                      (DWORD) NULL);
         }
      }
   GlobalUnlock (lpvsbdata->hLogInfo);
   GlobalUnlock (hvsbdata);
   }


//**************************************************************************<f>
// Func: VSBLogSubclassWndProc                        Author: Daryl Warinner
//
// Parm: HWND     hWnd;    -- The user interface window handle
//       WORD     msg;     -- The message
//       WORD     wparam;  -- message specific
//       DWORD    lparam;  -- message specific
//
// Desc: This window procedure subclasses the "real" scroll bar and traps
//       out right button double clicks to activate (show) the Log Window.
//
//*****************************************************************************

LONG FAR PASCAL VSBLogSubclassWndProc (HWND hWnd, WORD msg, WORD wparam, DWORD lparam)
   {
   HANDLE       hvsbdata;
   LPVSBDATA    lpvsbdata;
   LPLOGINFO    lpLogInfo;
   LONG         rc;

   // Retrieve the Window Word data structure
   if (!(hvsbdata = GetWindowWord (GetWindowWord (hWnd, GWW_HWNDPARENT), VSB_WWOFF_PVSB)))
      return 0;
   if (!(lpvsbdata = (LPVSBDATA)GlobalLock (hvsbdata)))
      return 0;

   if (!(lpLogInfo = (LPLOGINFO)GlobalLock (lpvsbdata->hLogInfo)))
      return 0;

   if (msg == WM_RBUTTONDBLCLK)
      {
      ShowWindow (lpLogInfo->hWndVSBLOG, TRUE);
      SetFocus (lpLogInfo->hWndVSBLOG);
      rc = (LONG)0;
      }
   else
      {
      rc = (lpLogInfo->pfnSBWndProc)(hWnd, msg, wparam, lparam);
      }

   GlobalUnlock (lpvsbdata->hLogInfo);
   GlobalUnlock (hvsbdata);
   return (rc);
   }


//**************************************************************************<f>
// Func: VSBLogDlgProc                                Author: Daryl Warinner
//
// Parm: HWND     hWnd;    -- The user interface window handle
//       WORD     msg;     -- The message
//       WORD     wparam;  -- message specific
//       DWORD    lparam;  -- message specific
//
// Desc: This is the window procedure for the VSBLOG modeless dialog box.
//
//*****************************************************************************

LONG FAR PASCAL VSBLogDlgProc (HWND hWnd, WORD msg, WORD wparam, DWORD lparam)
   {
   static char       szTitle [MAX_LEN_LOGTITLE];
   static char       szLoadString [MAX_LEN_LOADSTRING];
   HWND              hWndLOGBOX;
   int               itemindex;
   LPLOGCREATEPARAMS lplogcreateparams;
   WORD              wReturn;
   FARPROC           lpLogMarkDlgProc;

   switch (msg)
      {
      //-----------------------------------------------------------------------
      case WM_INITDIALOG:
         lplogcreateparams = (LPLOGCREATEPARAMS) (lparam);

         // Load the TITLE
         LoadString (hmodDLL,
                     IDS_LOG_TITLE,
                     szTitle,
                     sizeof (szTitle));

         LoadString (hmodDLL,
                     (IDS_LOG_TITLE + lplogcreateparams->wPlayer),
                     szLoadString,
                     sizeof (szLoadString));

         lstrcat (szTitle, szLoadString);

         SetWindowText (hWnd, szTitle);

         // Initialize the Log List Box
         hWndLOGBOX = GetDlgItem (hWnd, IDC_LOGBOX);

         SendMessage (hWndLOGBOX, LB_RESETCONTENT, NULL, NULL);

         itemindex = (int) SendMessage (hWndLOGBOX,
                                          LB_INSERTSTRING,
                                          -1,
                                          (DWORD)(LPVOID)szTopOfLog);

         SendMessage (hWndLOGBOX, LB_SETTOPINDEX, itemindex, NULL);
         return ((LONG)FALSE);
         break;

      //-----------------------------------------------------------------------
      case WM_COMMAND:
         hWndLOGBOX = GetDlgItem (hWnd, IDC_LOGBOX);
         switch (wparam)        // Control ID
            {
            case IDC_OK:
               // Hide the window
               ShowWindow (hWnd, FALSE);
               break;

            case IDC_MARK:
               // Bring up and process the mark dialog box
               lpLogMarkDlgProc = MakeProcInstance ((FARPROC)VSBLogMarkDlgProc,
                                                    hmodDLL );
               wReturn = DialogBox (hmodDLL, "IDD_VSBLOGMARK",
                                    hWnd, lpLogMarkDlgProc);

               if (wReturn == IDC_MARKADD)
                  {
                  itemindex = (int) SendMessage (hWndLOGBOX,
                                                   LB_INSERTSTRING,
                                                   -1,
                                                   (DWORD)(LPVOID)szMarkMessage);

                  SendMessage (hWndLOGBOX, LB_SETTOPINDEX, itemindex, NULL);
                  }

               // Clear the message string
               strnset (szMarkMessage, 0, sizeof (szMarkMessage));
               break;

            case IDC_CLEAR:
               // Re-initialize the Log List Box
               SendMessage (hWndLOGBOX, LB_RESETCONTENT, NULL, NULL);
               ulLine = 0L;
               itemindex = (int) SendMessage (hWndLOGBOX,
                                              LB_INSERTSTRING,
                                              -1,
                                             (DWORD)(LPVOID)szTopOfLog);

               SendMessage (hWndLOGBOX, LB_SETTOPINDEX, itemindex, NULL);
               break;
            }

         return ((LONG) 0);
         break;

      //-----------------------------------------------------------------------
      default:
         return ((LONG) 0);
         break;
      }
   }


//**************************************************************************<f>
// Func: VSBLogMarkDlgProc                            Author: Daryl Warinner
//
// Parm: HWND     hWnd;    -- The user interface window handle
//       WORD     msg;     -- The message
//       WORD     wparam;  -- message specific
//       DWORD    lparam;  -- message specific
//
// Desc: This window procedure processes the Mark string dialog box.
//
//*****************************************************************************

LONG FAR PASCAL VSBLogMarkDlgProc (HWND hWnd, WORD msg, WORD wparam, DWORD lparam)
   {

   switch (msg)
      {
      //-----------------------------------------------------------------------
      case WM_INITDIALOG:
         SendDlgItemMessage (hWnd,
                             IDC_EDITBOX,
                             EM_LIMITTEXT,
                             MAX_LEN_LOGMESSAGE,
                             NULL);
         return (FALSE);
         break;

      //-----------------------------------------------------------------------
      case WM_COMMAND:
         switch (wparam)        // Control ID
            {
            case IDC_MARKADD:
               // Get the string (we were already locked)
               GetWindowText (GetDlgItem (hWnd, IDC_EDITBOX),
                              szMarkMessage,
                              MAX_LEN_LOGMESSAGE);
               // Fall Through

            case IDC_MARKCANCEL:
               // Kill the dialog box
               EndDialog (hWnd, wparam);
               break;
            }

         return ((LONG) 0);
         break;

      //-----------------------------------------------------------------------
      default:
         return ((LONG) 0);
         break;
      }
   }

#endif
///////////////////////////////////////////////////////////////////////////////

//*****************************************************************************
// END OF FILE
//*****************************************************************************
